@@ -64,6 +64,8 @@ typedef struct {
 	gboolean use_bundle_signing_time;
 	/* bit mask for allowed formats */
 	guint bundle_formats_mask;
+	/* enable complete read before mount */
+	gboolean perform_pre_check;
 
 	gchar *autoinstall_path;
 	gchar *preinstall_handler;
@@ -82,6 +84,8 @@ typedef struct {
 	gchar *encryption_cert;
 
 	GHashTable *slots;
+	/* flag to ensure slot states were determined */
+	gboolean slot_states_determined;
 } RaucConfig;
 
 typedef enum {
@@ -94,9 +98,6 @@ typedef enum {
 #define R_SLOT_ERROR r_slot_error_quark()
 GQuark r_slot_error_quark(void);
 
-typedef struct {
-} RaucSlotGroup;
-
 /**
  * Parses a bundle format list into a mask.
  *
