@@ -4,6 +4,8 @@
 
 #include "bundle.h"
 #include "manifest.h"
+#include "slot.h"
+#include "update_handler.h"
 
 #define R_INSTALL_ERROR r_install_error_quark()
 GQuark r_install_error_quark(void);
@@ -32,9 +34,20 @@ typedef struct {
 	gint status_result;
 	/* install options */
 	gboolean ignore_compatible;
+	gchar *transaction;
 	RaucBundleAccessArgs access_args;
 } RaucInstallArgs;
 
+/**
+ * Update the external mount points of a slot.
+ *
+ * @param error return location for a GError
+ *
+ * @return TRUE if succeeded, FALSE if failed
+ */
+gboolean update_external_mount_points(GError **error)
+G_GNUC_WARN_UNUSED_RESULT;
+
 /**
  * Determines the states (ACTIVE | INACTIVE | BOOTED) of the slots specified in
  * system configuration.
@@ -117,8 +130,17 @@ void install_args_free(RaucInstallArgs *args);
  */
 gboolean install_run(RaucInstallArgs *args);
 
+typedef struct {
+	RaucImage *image;
+
+	RaucSlot *target_slot;
+	img_to_slot_handler slot_handler;
+} RImageInstallPlan;
+
+void r_image_install_plan_free(gpointer value);
+
 /**
- * Checks and returns list of images to install
+ * Builds and returns an array of RImageInstallPlans.
  *
  * Check is performed against target_group.
  *
@@ -129,8 +151,8 @@ gboolean install_run(RaucInstallArgs *args);
  * @param target_group target group to verify against
  * @param error Return location for a GError
  *
- * @return Returns a map slotclass (gchar*) -> image (RaucImage *),
+ * @return Returns GPtrArray of RImageInstallPlans
  *         or NULL if an error occurred
  */
-GList* get_install_images(const RaucManifest *manifest, GHashTable *target_group, GError **error)
+GPtrArray* r_install_make_plans(const RaucManifest *manifest, GHashTable *target_group, GError **error)
 G_GNUC_WARN_UNUSED_RESULT;
