@@ -2,7 +2,8 @@
 
 #include <glib.h>
 
-void mark_active(RaucSlot *slot, GError **error);
+gboolean mark_active(RaucSlot *slot, GError **error)
+G_GNUC_WARN_UNUSED_RESULT;
 
 gboolean mark_run(const gchar *state,
 		const gchar *slot_identifier,
