@@ -3,6 +3,16 @@
 #include <gio/gio.h>
 #include <glib.h>
 
+#define R_UTILS_ERROR r_utils_error_quark()
+
+GQuark r_utils_error_quark(void);
+
+typedef enum {
+	R_UTILS_ERROR_FAILED,
+	R_UTILS_ERROR_INAPPROPRIATE_IOCTL,
+	R_UTILS_ERROR_INVALID_ENV_KEY,
+} RUtilsError;
+
 #define BIT(nr) (1UL << (nr))
 
 /* Evaluate EXPRESSION, and repeat as long as it returns -1 with `errno'
@@ -58,6 +68,20 @@ G_GNUC_WARN_UNUSED_RESULT;
 		R_LOG_LEVEL_TRACE,    \
 		__VA_ARGS__)
 
+/**
+ * Adds elements of a zero-terminated GStrv/gchar** to an existing GPtrArray
+ *
+ * @param ptrarray GPtrArray to add to
+ * @param argvp arguments to add
+ * @param copy whether to just add the pointer (FALSE) or copy the underlying data (TRUE)
+ */
+static inline void r_ptr_array_addv(GPtrArray *ptrarray, gchar **argvp, gboolean copy)
+{
+	for (gchar **addarg = argvp; *addarg != NULL; addarg++) {
+		g_ptr_array_add(ptrarray, copy ? g_strdup(*addarg) : *addarg);
+	}
+}
+
 /**
  * Read file content into a GBytes.
  *
@@ -173,6 +197,16 @@ gchar * key_file_consume_string(
 		GError **error)
 G_GNUC_WARN_UNUSED_RESULT;
 
+/**
+ * Ensure that the input string contains neither whitespace nor tab.
+ *
+ * @param str string to check.
+ *
+ * @return TRUE if str contains neither whitespace nor tab, FALSE otherwise
+ */
+gboolean value_check_tab_whitespace(const gchar *str, GError **error)
+G_GNUC_WARN_UNUSED_RESULT;
+
 /**
  * Get integer argument from key and remove key from key_file.
  */
@@ -223,3 +257,31 @@ G_GNUC_WARN_UNUSED_RESULT;
 
 guint get_sectorsize(gint fd)
 G_GNUC_WARN_UNUSED_RESULT;
+
+goffset get_device_size(gint fd, GError **error)
+G_GNUC_WARN_UNUSED_RESULT;
+
+/**
+ * Converts a key for use in an environment variable name.
+ *
+ * Only alphanumeric characters and '_' are allowed. '-' is converted to '_'.
+ *
+ * @param key string to convert
+ * @param error return location for a GError, or NULL
+ *
+ * @return the newly alloacted and converted string
+ */
+gchar *r_prepare_env_key(const gchar *key, GError **error)
+G_GNUC_WARN_UNUSED_RESULT;
+
+/**
+ * Atomically updates a symlink (if needed).
+ *
+ * @param target new target for the symlink
+ * @param name filename of the symlink to update
+ * @param error return location for a GError, or NULL
+ *
+ * @return the newly alloacted and converted string
+ */
+gboolean r_update_symlink(const gchar *target, const gchar *name, GError **error)
+G_GNUC_WARN_UNUSED_RESULT;
