@@ -33,6 +33,9 @@
 #ifndef CRAMFS_MAGIC
 #define CRAMFS_MAGIC 0x28cd3d45
 #endif
+#ifndef EXFAT_SUPER_MAGIC
+#define EXFAT_SUPER_MAGIC 0x2011bab0
+#endif
 #ifndef EXT4_SUPER_MAGIC /* also covers ext2/3 */
 #define EXT4_SUPER_MAGIC 0xef53
 #endif
@@ -134,9 +137,7 @@ static gboolean mksquashfs(const gchar *bundlename, const gchar *contentdir, GEr
 					"Failed to parse mksquashfs extra args: ");
 			goto out;
 		}
-		for (gchar **mksquashfs_args = mksquashfs_argvp; *mksquashfs_args != NULL; mksquashfs_args++) {
-			g_ptr_array_add(args, g_strdup(*mksquashfs_args));
-		}
+		r_ptr_array_addv(args, mksquashfs_argvp, TRUE);
 	}
 	g_ptr_array_add(args, NULL);
 
@@ -269,9 +270,7 @@ static gboolean casync_make_arch(const gchar *idxpath, const gchar *contentpath,
 					"Failed to parse casync extra args: ");
 			goto out;
 		}
-		for (gchar **casync_args = casync_argvp; *casync_args != NULL; casync_args++) {
-			g_ptr_array_add(iargs, g_strdup(*casync_args));
-		}
+		r_ptr_array_addv(iargs, casync_argvp, TRUE);
 	}
 	g_ptr_array_add(iargs, NULL);
 
@@ -376,9 +375,7 @@ static gboolean casync_make_blob(const gchar *idxpath, const gchar *contentpath,
 					"Failed to parse casync extra args: ");
 			goto out;
 		}
-		for (gchar **casync_args = casync_argvp; *casync_args != NULL; casync_args++) {
-			g_ptr_array_add(args, g_strdup(*casync_args));
-		}
+		r_ptr_array_addv(args, casync_argvp, TRUE);
 	}
 	g_ptr_array_add(args, NULL);
 
@@ -406,6 +403,19 @@ out:
 	return res;
 }
 
+static gboolean image_is_archive(RaucImage* image)
+{
+	g_return_val_if_fail(image, FALSE);
+	g_return_val_if_fail(image->filename, FALSE);
+
+	if (g_pattern_match_simple("*.tar*", image->filename) ||
+	    g_pattern_match_simple("*.catar", image->filename)) {
+		return TRUE;
+	}
+
+	return FALSE;
+}
+
 static gboolean generate_adaptive_data(RaucManifest *manifest, const gchar *dir, GError **error)
 {
 	GError *ierror = NULL;
@@ -427,7 +437,13 @@ static gboolean generate_adaptive_data(RaucManifest *manifest, const gchar *dir,
 				g_autofree gchar *indexname = g_strconcat(image->filename, ".block-hash-index", NULL);
 				g_autofree gchar *indexpath = g_build_filename(dir, indexname, NULL);
 				g_autoptr(RaucHashIndex) index = NULL;
-				int fd = g_open(imagepath, O_RDONLY | O_CLOEXEC);
+				int fd = -1;
+
+				if (image_is_archive(image)) {
+					g_warning("Generating block hash index requires a block device image but %s looks like an archive", image->filename);
+				}
+
+				fd = g_open(imagepath, O_RDONLY | O_CLOEXEC);
 				if (fd < 0) {
 					int err = errno;
 					g_set_error(
@@ -859,6 +875,11 @@ gboolean create_bundle(const gchar *bundlename, const gchar *contentdir, GError
 		goto out;
 	}
 
+	/* print warnings collected while parsing */
+	for (guint i =  0; i < manifest->warnings->len; i++) {
+		g_print("%s\n", (gchar *)g_ptr_array_index(manifest->warnings, i));
+	}
+
 	res = sync_manifest_with_contentdir(manifest, contentdir, &ierror);
 	if (!res) {
 		g_propagate_error(error, ierror);
@@ -1034,19 +1055,6 @@ out:
 	return res;
 }
 
-static gboolean image_is_archive(RaucImage* image)
-{
-	g_return_val_if_fail(image, FALSE);
-	g_return_val_if_fail(image->filename, FALSE);
-
-	if (g_pattern_match_simple("*.tar*", image->filename) ||
-	    g_pattern_match_simple("*.catar", image->filename)) {
-		return TRUE;
-	}
-
-	return FALSE;
-}
-
 static gboolean convert_to_casync_bundle(RaucBundle *bundle, const gchar *outbundle, GError **error)
 {
 	GError *ierror = NULL;
@@ -1231,7 +1239,7 @@ gboolean encrypt_bundle(RaucBundle *bundle, const gchar *outbundle, GError **err
 	 * receives an encrypted CMS but no encrypted payload (which is what we
 	 * actually want to protect).
 	 */
-	if (bundle->manifest->bundle_format != R_MANIFEST_FORMAT_CRYPT) {
+	if (!bundle->manifest || bundle->manifest->bundle_format != R_MANIFEST_FORMAT_CRYPT) {
 		g_set_error(error, R_BUNDLE_ERROR, R_BUNDLE_ERROR_SIGNATURE, "Refused to encrypt input bundle that is not in 'crypt' format.");
 		return FALSE;
 	}
@@ -1323,7 +1331,8 @@ static gboolean is_remote_scheme(const gchar *scheme)
 	return (g_strcmp0(scheme, "http") == 0) ||
 	       (g_strcmp0(scheme, "https") == 0) ||
 	       (g_strcmp0(scheme, "sftp") == 0) ||
-	       (g_strcmp0(scheme, "ftp") == 0);
+	       (g_strcmp0(scheme, "ftp") == 0) ||
+	       (g_strcmp0(scheme, "ftps") == 0);
 }
 
 static gboolean take_bundle_ownership(int bundle_fd, GError **error)
@@ -1483,6 +1492,7 @@ static gboolean check_bundle_access(int bundle_fd, GError **error)
 			case AFS_SUPER_MAGIC:
 			case BTRFS_SUPER_MAGIC:
 			case CRAMFS_MAGIC:
+			case EXFAT_SUPER_MAGIC:
 			case EXT4_SUPER_MAGIC: /* also covers ext2/3 */
 			case F2FS_SUPER_MAGIC:
 			case ISOFS_SUPER_MAGIC:
@@ -1883,7 +1893,7 @@ gboolean check_bundle(const gchar *bundlename, RaucBundle **bundle, CheckBundleP
 	gboolean verify = !(params & CHECK_BUNDLE_NO_VERIFY);
 	g_autoptr(RaucBundle) ibundle = g_new0(RaucBundle, 1);
 	g_autoptr(GBytes) manifest_bytes = NULL;
-	gchar *bundlescheme = NULL;
+	g_autofree gchar *bundlescheme = NULL;
 	gboolean detached;
 
 	g_return_val_if_fail(bundlename, FALSE);
@@ -2008,7 +2018,7 @@ gboolean check_bundle(const gchar *bundlename, RaucBundle **bundle, CheckBundleP
 		g_debug("CMS type is 'enveloped'. Attempting to decrypt..");
 
 		if (r_context()->config->encryption_key == NULL) {
-			g_set_error(error, R_BUNDLE_ERROR, R_BUNDLE_ERROR_SIGNATURE, "Encrypted bundle detected, but no decryption key given");
+			g_set_error(error, R_BUNDLE_ERROR, R_BUNDLE_ERROR_SIGNATURE, "Encrypted bundle detected, but no decryption key given. Use --key=<PEMFILE|PKCS11-URL> to provide one.");
 			res = FALSE;
 			goto out;
 		}
@@ -2414,6 +2424,12 @@ gboolean extract_bundle(RaucBundle *bundle, const gchar *outputdir, GError **err
 
 	r_context_begin_step("extract_bundle", "Extracting bundle", 2);
 
+	if (g_file_test(outputdir, G_FILE_TEST_EXISTS)) {
+		res = FALSE;
+		g_set_error(error, G_FILE_ERROR, G_FILE_ERROR_EXIST, "output directory %s exists already", outputdir);
+		goto out;
+	}
+
 	res = check_bundle_payload(bundle, &ierror);
 	if (!res) {
 		g_propagate_error(error, ierror);
@@ -2495,6 +2511,49 @@ out:
 	return res;
 }
 
+static gboolean read_complete_dm_device(gchar *dev, GError **error)
+{
+	int fd = -1;
+	const goffset chunk_size = 65536;
+	g_autofree void* buf = NULL;
+	ssize_t r;
+	gboolean ret = TRUE;
+
+	g_return_val_if_fail(dev != NULL, FALSE);
+	g_return_val_if_fail(error == NULL || *error == NULL, FALSE);
+
+	fd = g_open(dev, O_RDONLY | O_CLOEXEC, 0);
+	if (fd < 0) {
+		int err = errno;
+		g_set_error(error,
+				G_FILE_ERROR,
+				g_file_error_from_errno(err),
+				"Failed to open %s: %s", dev, g_strerror(err));
+		return FALSE;
+	}
+
+	buf = g_malloc0(chunk_size);
+
+	for (goffset chunk = 0;; chunk++) {
+		r = pread(fd, buf, chunk_size, chunk*chunk_size);
+		if (r == 0)
+			break;
+		else if (r < 0)	{
+			int err = errno;
+			g_set_error(error,
+					G_FILE_ERROR,
+					g_file_error_from_errno(err),
+					"Check %s device failed between %"G_GOFFSET_FORMAT " and %"G_GOFFSET_FORMAT " bytes with error: %s", dev, chunk*sizeof(buf), (chunk+1)*sizeof(buf), g_strerror(err));
+			ret = FALSE;
+			break;
+		}
+	}
+
+	g_close(fd, NULL);
+
+	return ret;
+}
+
 /*
  * Sets up dm-verity for reading verity bundles.
  *
@@ -2533,6 +2592,18 @@ static gboolean prepare_verity(RaucBundle *bundle, gchar *loopname, gchar* mount
 		return FALSE;
 	}
 
+	if (r_context()->config->perform_pre_check) {
+		res = read_complete_dm_device(dm_verity->upper_dev, &ierror);
+		if (!res) {
+			/* ensure the already-set-up dm-verity layer is cleaned */
+			if (!r_dm_remove(dm_verity, FALSE, &ierror_dm))	{
+				g_warning("Failed to remove dm-verity device: %s", ierror_dm->message);
+			}
+			g_propagate_error(error, ierror);
+			return FALSE;
+		}
+	}
+
 	res = r_mount_bundle(dm_verity->upper_dev, mount_point, &ierror);
 
 	if (!r_dm_remove(dm_verity, TRUE, &ierror_dm)) {
@@ -2604,6 +2675,22 @@ static gboolean prepare_crypt(RaucBundle *bundle, gchar *loopname, gchar* mount_
 		return FALSE;
 	}
 
+	if (r_context()->config->perform_pre_check) {
+		res = read_complete_dm_device(dm_crypt->upper_dev, &ierror);
+		if (!res) {
+			/* ensure the already-set-up dm-verity and dm-crypt layers are cleaned */
+			if (!r_dm_remove(dm_crypt, TRUE, &ierror_dm)) {
+				g_warning("Failed to remove dm-crypt device: %s", ierror_dm->message);
+				g_clear_error(&ierror_dm);
+			}
+			if (!r_dm_remove(dm_verity, TRUE, &ierror_dm)) {
+				g_warning("Failed to remove dm-verity device: %s", ierror_dm->message);
+			}
+			g_propagate_error(error, ierror);
+			return FALSE;
+		}
+	}
+
 	res = r_mount_bundle(dm_crypt->upper_dev, mount_point, &ierror);
 
 	if (!r_dm_remove(dm_crypt, TRUE, &ierror_dm)) {
@@ -2696,11 +2783,13 @@ gboolean mount_bundle(RaucBundle *bundle, GError **error)
 					error,
 					ierror,
 					"failed to load manifest from bundle: ");
+			ierror = NULL;
 			goto umount;
 		}
 		res = check_manifest_internal(manifest, &ierror);
 		if (!res) {
 			g_propagate_error(error, ierror);
+			ierror = NULL;
 			goto umount;
 		}
 
