@@ -156,36 +156,269 @@ out:
 	return res;
 }
 
+static GHashTable *parse_slots(const char *filename, const char *data_directory, GKeyFile *key_file, GError **error)
+{
+	GError *ierror = NULL;
+	g_auto(GStrv) groups = NULL;
+	gsize group_count;
+	g_autoptr(GHashTable) slots = NULL;
+	g_autoptr(GList) slotlist = NULL;
+	g_autoptr(GHashTable) bootnames = NULL;
+
+	slots = g_hash_table_new_full(g_str_hash, g_str_equal, NULL, r_slot_free);
+	bootnames = g_hash_table_new(g_str_hash, g_str_equal);
+
+	groups = g_key_file_get_groups(key_file, &group_count);
+	for (gsize i = 0; i < group_count; i++) {
+		g_auto(GStrv) groupsplit = g_strsplit(groups[i], ".", -1);
+
+		/* We treat sections starting with "slot." as slots */
+		if (g_str_equal(groupsplit[0], RAUC_SLOT_PREFIX)) {
+			g_autoptr(RaucSlot) slot = g_new0(RaucSlot, 1);
+			gchar* value;
+
+			/* Assure slot strings consist of 3 parts, delimited by dots */
+			if (g_strv_length(groupsplit) != 3) {
+				g_set_error(
+						error,
+						R_CONFIG_ERROR,
+						R_CONFIG_ERROR_INVALID_FORMAT,
+						"Invalid slot name format: %s", groups[i]);
+				return NULL;
+			}
+
+			value = g_strconcat(groupsplit[1], ".", groupsplit[2], NULL);
+			slot->name = g_intern_string(value);
+			g_free(value);
+
+			/* If we have a data_directory, use a slot.<class>.<index>
+			 * subdirectory for per-slot data. */
+			if (data_directory)
+				slot->data_directory = g_build_filename(data_directory, groups[i], NULL);
+
+			slot->description = key_file_consume_string(key_file, groups[i], "description", NULL);
+
+			slot->sclass = g_intern_string(groupsplit[1]);
+
+			value = resolve_path_take(filename,
+					key_file_consume_string(key_file, groups[i], "device", &ierror));
+			if (!value) {
+				g_propagate_error(error, ierror);
+				return NULL;
+			}
+			slot->device = value;
+
+			value = key_file_consume_string(key_file, groups[i], "type", NULL);
+			if (!value)
+				value = g_strdup("raw");
+			slot->type = value;
+
+			if (!r_slot_is_valid_type(slot->type)) {
+				g_set_error(
+						error,
+						R_CONFIG_ERROR,
+						R_CONFIG_ERROR_SLOT_TYPE,
+						"Unsupported slot type '%s' for slot %s selected in system config", slot->type, slot->name);
+				return NULL;
+			}
+
+			/* check if the device has an appropriate path */
+			if (g_str_equal(slot->type, "jffs2") && !g_str_has_prefix(slot->device, "/dev/")) {
+				g_set_error(
+						error,
+						R_CONFIG_ERROR,
+						R_CONFIG_ERROR_INVALID_DEVICE,
+						"%s: device must be located in /dev/ for jffs2", groups[i]);
+				return NULL;
+			}
+
+			value = key_file_consume_string(key_file, groups[i], "bootname", NULL);
+
+			slot->bootname = value;
+			if (slot->bootname) {
+				/* Ensure that the bootname does not contain whitespace or tab */
+				if (!value_check_tab_whitespace(value, &ierror)) {
+					g_propagate_prefixed_error(error, ierror,
+							"Invalid bootname for slot %s: ", slot->name);
+					return NULL;
+				}
+
+				/* check if we have seen this bootname on another slot */
+				if (g_hash_table_contains(bootnames, slot->bootname)) {
+					g_set_error(
+							error,
+							R_CONFIG_ERROR,
+							R_CONFIG_ERROR_DUPLICATE_BOOTNAME,
+							"Bootname '%s' is set on more than one slot",
+							slot->bootname);
+					return NULL;
+				}
+				g_hash_table_add(bootnames, slot->bootname);
+			}
+
+			/* Collect name of parent here for easing remaining key checking.
+			 * Will be resolved to slot->parent pointer after config parsing loop. */
+			slot->parent_name = key_file_consume_string(key_file, groups[i], "parent", NULL);
+
+			slot->allow_mounted = g_key_file_get_boolean(key_file, groups[i], "allow-mounted", &ierror);
+			if (g_error_matches(ierror, G_KEY_FILE_ERROR, G_KEY_FILE_ERROR_KEY_NOT_FOUND)) {
+				slot->allow_mounted = FALSE;
+				g_clear_error(&ierror);
+			} else if (ierror) {
+				g_propagate_error(error, ierror);
+				return NULL;
+			}
+			g_key_file_remove_key(key_file, groups[i], "allow-mounted", NULL);
+
+			slot->readonly = g_key_file_get_boolean(key_file, groups[i], "readonly", &ierror);
+			if (g_error_matches(ierror, G_KEY_FILE_ERROR, G_KEY_FILE_ERROR_KEY_NOT_FOUND)) {
+				slot->readonly = FALSE;
+				g_clear_error(&ierror);
+			} else if (ierror) {
+				g_propagate_error(error, ierror);
+				return NULL;
+			}
+			g_key_file_remove_key(key_file, groups[i], "readonly", NULL);
+
+			slot->install_same = g_key_file_get_boolean(key_file, groups[i], "install-same", &ierror);
+			if (g_error_matches(ierror, G_KEY_FILE_ERROR, G_KEY_FILE_ERROR_KEY_NOT_FOUND)) {
+				g_clear_error(&ierror);
+				/* try also deprecated flag force-install-same */
+				slot->install_same = g_key_file_get_boolean(key_file, groups[i], "force-install-same", &ierror);
+				if (g_error_matches(ierror, G_KEY_FILE_ERROR, G_KEY_FILE_ERROR_KEY_NOT_FOUND)) {
+					g_clear_error(&ierror);
+					/* try also deprecated flag ignore-checksum */
+					slot->install_same = g_key_file_get_boolean(key_file, groups[i], "ignore-checksum", &ierror);
+					if (g_error_matches(ierror, G_KEY_FILE_ERROR, G_KEY_FILE_ERROR_KEY_NOT_FOUND)) {
+						slot->install_same = TRUE;
+						g_clear_error(&ierror);
+					} else if (ierror) {
+						g_propagate_error(error, ierror);
+						return NULL;
+					}
+				} else if (ierror) {
+					g_propagate_error(error, ierror);
+					return NULL;
+				}
+			} else if (ierror) {
+				g_propagate_error(error, ierror);
+				return NULL;
+			}
+			g_key_file_remove_key(key_file, groups[i], "install-same", NULL);
+			g_key_file_remove_key(key_file, groups[i], "force-install-same", NULL);
+			g_key_file_remove_key(key_file, groups[i], "ignore-checksum", NULL);
+
+			slot->extra_mount_opts = key_file_consume_string(key_file, groups[i], "extra-mount-opts", NULL);
+
+			slot->resize = g_key_file_get_boolean(key_file, groups[i], "resize", &ierror);
+			if (g_error_matches(ierror, G_KEY_FILE_ERROR, G_KEY_FILE_ERROR_KEY_NOT_FOUND)) {
+				slot->resize = FALSE;
+				g_clear_error(&ierror);
+			} else if (ierror) {
+				g_propagate_error(error, ierror);
+				return NULL;
+			}
+			g_key_file_remove_key(key_file, groups[i], "resize", NULL);
+
+			if (g_strcmp0(slot->type, "boot-mbr-switch") == 0 ||
+			    g_strcmp0(slot->type, "boot-gpt-switch") == 0 ||
+			    g_strcmp0(slot->type, "boot-raw-fallback") == 0) {
+				slot->region_start = key_file_consume_binary_suffixed_string(key_file, groups[i],
+						"region-start", &ierror);
+				if (ierror) {
+					g_propagate_prefixed_error(error, ierror, "mandatory for %s: ", slot->type);
+					return NULL;
+				}
+
+				slot->region_size = key_file_consume_binary_suffixed_string(key_file, groups[i],
+						"region-size", &ierror);
+				if (ierror) {
+					g_propagate_prefixed_error(error, ierror, "mandatory for %s: ", slot->type);
+					return NULL;
+				}
+			}
+
+			if (!check_remaining_keys(key_file, groups[i], &ierror)) {
+				g_propagate_error(error, ierror);
+				return NULL;
+			}
+
+			g_key_file_remove_group(key_file, groups[i], NULL);
+
+			g_hash_table_insert(slots, (gchar*)slot->name, slot);
+			slot = NULL;
+		}
+	}
+
+	/* Add parent pointers */
+	slotlist = g_hash_table_get_keys(slots);
+	for (GList *l = slotlist; l != NULL; l = l->next) {
+		RaucSlot *slot;
+		RaucSlot *parent;
+		RaucSlot *child;
+
+		slot = g_hash_table_lookup(slots, l->data);
+		if (!slot->parent_name) {
+			continue;
+		}
+
+		parent = g_hash_table_lookup(slots, slot->parent_name);
+		if (!parent) {
+			g_set_error(
+					error,
+					R_CONFIG_ERROR,
+					R_CONFIG_ERROR_PARENT,
+					"Parent slot '%s' not found!", slot->parent_name);
+			return NULL;
+		}
+
+		child = g_hash_table_lookup(slots, l->data);
+		child->parent = parent;
+
+		if (child->bootname) {
+			g_set_error(
+					error,
+					R_CONFIG_ERROR,
+					R_CONFIG_ERROR_CHILD_HAS_BOOTNAME,
+					"Child slot '%s' has bootname set",
+					child->name);
+			return NULL;
+		}
+	}
+
+	if (!fix_grandparent_links(slots, &ierror)) {
+		g_propagate_error(error, ierror);
+		return NULL;
+	}
+
+	return g_steal_pointer(&slots);
+}
+
 gboolean load_config(const gchar *filename, RaucConfig **config, GError **error)
 {
 	GError *ierror = NULL;
 	g_autoptr(RaucConfig) c = g_new0(RaucConfig, 1);
-	gboolean res = FALSE;
 	g_autoptr(GKeyFile) key_file = NULL;
-	g_auto(GStrv) groups = NULL;
-	gsize group_count;
-	g_autoptr(GList) slotlist = NULL;
-	GHashTable *slots = NULL;
-	g_autoptr(GHashTable) bootnames = NULL;
-	GList *l;
 	gboolean dtbvariant;
-	gchar *variant_data;
+	g_autofree gchar *variant_data = NULL;
 	g_autofree gchar *bundle_formats = NULL;
 
+	g_return_val_if_fail(filename, FALSE);
+	g_return_val_if_fail(config && *config == NULL, FALSE);
+	g_return_val_if_fail(error == NULL || *error == NULL, FALSE);
+
 	key_file = g_key_file_new();
 
-	res = g_key_file_load_from_file(key_file, filename, G_KEY_FILE_NONE, &ierror);
-	if (!res) {
+	if (!g_key_file_load_from_file(key_file, filename, G_KEY_FILE_NONE, &ierror)) {
 		g_propagate_error(error, ierror);
-		goto free;
+		return FALSE;
 	}
 
 	/* parse [system] section */
 	c->system_compatible = key_file_consume_string(key_file, "system", "compatible", &ierror);
 	if (!c->system_compatible) {
 		g_propagate_error(error, ierror);
-		res = FALSE;
-		goto free;
+		return FALSE;
 	}
 	c->system_bootloader = key_file_consume_string(key_file, "system", "bootloader", NULL);
 	if (!c->system_bootloader) {
@@ -194,8 +427,7 @@ gboolean load_config(const gchar *filename, RaucConfig **config, GError **error)
 				R_CONFIG_ERROR,
 				R_CONFIG_ERROR_BOOTLOADER,
 				"No bootloader selected in system config");
-		res = FALSE;
-		goto free;
+		return FALSE;
 	}
 
 	if (!r_boot_is_supported_bootloader(c->system_bootloader)) {
@@ -204,8 +436,7 @@ gboolean load_config(const gchar *filename, RaucConfig **config, GError **error)
 				R_CONFIG_ERROR,
 				R_CONFIG_ERROR_BOOTLOADER,
 				"Unsupported bootloader '%s' selected in system config", c->system_bootloader);
-		res = FALSE;
-		goto free;
+		return FALSE;
 	}
 
 	if (g_strcmp0(c->system_bootloader, "barebox") == 0) {
@@ -225,8 +456,7 @@ gboolean load_config(const gchar *filename, RaucConfig **config, GError **error)
 			g_clear_error(&ierror);
 		} else if (ierror) {
 			g_propagate_error(error, ierror);
-			res = FALSE;
-			goto free;
+			return FALSE;
 		}
 		g_key_file_remove_key(key_file, "system", "efi-use-bootnext", NULL);
 	} else if (g_strcmp0(c->system_bootloader, "custom") == 0) {
@@ -238,13 +468,53 @@ gboolean load_config(const gchar *filename, RaucConfig **config, GError **error)
 					R_CONFIG_ERROR,
 					R_CONFIG_ERROR_BOOTLOADER,
 					"No custom bootloader backend defined");
-			res = FALSE;
-			goto free;
+			return FALSE;
 		}
 	}
 
-	c->boot_default_attempts = key_file_consume_integer(key_file, "system", "boot-attempts", NULL);
-	c->boot_attempts_primary = key_file_consume_integer(key_file, "system", "boot-attempts-primary", NULL);
+	c->boot_default_attempts = key_file_consume_integer(key_file, "system", "boot-attempts", &ierror);
+	if (g_error_matches(ierror, G_KEY_FILE_ERROR, G_KEY_FILE_ERROR_KEY_NOT_FOUND)) {
+		c->boot_default_attempts = 0; /* to indicate 'unset' */
+		g_clear_error(&ierror);
+	} else if (ierror) {
+		g_propagate_error(error, ierror);
+		return FALSE;
+	}
+	if (c->boot_default_attempts < 0) {
+		g_set_error(
+				error,
+				R_CONFIG_ERROR,
+				R_CONFIG_ERROR_BOOTLOADER,
+				"Value for \"boot-attempts\" must not be negative");
+		return FALSE;
+	}
+
+	c->boot_attempts_primary = key_file_consume_integer(key_file, "system", "boot-attempts-primary", &ierror);
+	if (g_error_matches(ierror, G_KEY_FILE_ERROR, G_KEY_FILE_ERROR_KEY_NOT_FOUND)) {
+		c->boot_attempts_primary = 0; /* to indicate 'unset' */
+		g_clear_error(&ierror);
+	} else if (ierror) {
+		g_propagate_error(error, ierror);
+		return FALSE;
+	}
+	if (c->boot_attempts_primary < 0) {
+		g_set_error(
+				error,
+				R_CONFIG_ERROR,
+				R_CONFIG_ERROR_BOOTLOADER,
+				"Value for \"boot-attempts-primary\" must not be negative");
+		return FALSE;
+	}
+	if (c->boot_default_attempts > 0 || c->boot_attempts_primary > 0) {
+		if ((g_strcmp0(c->system_bootloader, "uboot") != 0) && (g_strcmp0(c->system_bootloader, "barebox") != 0)) {
+			g_set_error(
+					error,
+					R_CONFIG_ERROR,
+					R_CONFIG_ERROR_BOOTLOADER,
+					"Configuring boot attempts is valid for uboot or barebox only (not for %s)", c->system_bootloader);
+			return FALSE;
+		}
+	}
 
 	c->max_bundle_download_size = g_key_file_get_uint64(key_file, "system", "max-bundle-download-size", &ierror);
 	if (g_error_matches(ierror, G_KEY_FILE_ERROR, G_KEY_FILE_ERROR_KEY_NOT_FOUND)) {
@@ -254,8 +524,7 @@ gboolean load_config(const gchar *filename, RaucConfig **config, GError **error)
 		g_clear_error(&ierror);
 	} else if (ierror) {
 		g_propagate_error(error, ierror);
-		res = FALSE;
-		goto free;
+		return FALSE;
 	}
 	if (c->max_bundle_download_size == 0) {
 		g_set_error(
@@ -263,8 +532,7 @@ gboolean load_config(const gchar *filename, RaucConfig **config, GError **error)
 				R_CONFIG_ERROR,
 				R_CONFIG_ERROR_MAX_BUNDLE_DOWNLOAD_SIZE,
 				"Invalid value (%" G_GUINT64_FORMAT ") for key \"max-bundle-download-size\" in system config", c->max_bundle_download_size);
-		res = FALSE;
-		goto free;
+		return FALSE;
 	}
 	g_key_file_remove_key(key_file, "system", "max-bundle-download-size", NULL);
 
@@ -280,8 +548,7 @@ gboolean load_config(const gchar *filename, RaucConfig **config, GError **error)
 		g_clear_error(&ierror);
 	} else if (ierror) {
 		g_propagate_error(error, ierror);
-		res = FALSE;
-		goto free;
+		return FALSE;
 	}
 	g_key_file_remove_key(key_file, "system", "activate-installed", NULL);
 
@@ -294,8 +561,7 @@ gboolean load_config(const gchar *filename, RaucConfig **config, GError **error)
 		g_clear_error(&ierror);
 	} else if (ierror) {
 		g_propagate_error(error, ierror);
-		res = FALSE;
-		goto free;
+		return FALSE;
 	}
 	g_key_file_remove_key(key_file, "system", "variant-dtb", NULL);
 	if (dtbvariant)
@@ -308,8 +574,7 @@ gboolean load_config(const gchar *filename, RaucConfig **config, GError **error)
 		g_clear_error(&ierror);
 	} else if (ierror) {
 		g_propagate_error(error, ierror);
-		res = FALSE;
-		goto free;
+		return FALSE;
 	}
 	if (variant_data) {
 		if (c->system_variant_type != R_CONFIG_SYS_VARIANT_NONE) {
@@ -318,12 +583,11 @@ gboolean load_config(const gchar *filename, RaucConfig **config, GError **error)
 					R_CONFIG_ERROR,
 					R_CONFIG_ERROR_INVALID_FORMAT,
 					"Only one of the keys 'variant-file', variant-dtb','variant-name' is allowed");
-			res = FALSE;
-			goto free;
+			return FALSE;
 		}
 
 		c->system_variant_type = R_CONFIG_SYS_VARIANT_FILE;
-		c->system_variant = variant_data;
+		c->system_variant = g_steal_pointer(&variant_data);
 	}
 
 	/* parse 'variant-name' key */
@@ -333,8 +597,7 @@ gboolean load_config(const gchar *filename, RaucConfig **config, GError **error)
 		g_clear_error(&ierror);
 	} else if (ierror) {
 		g_propagate_error(error, ierror);
-		res = FALSE;
-		goto free;
+		return FALSE;
 	}
 	if (variant_data) {
 		if (c->system_variant_type != R_CONFIG_SYS_VARIANT_NONE) {
@@ -343,12 +606,11 @@ gboolean load_config(const gchar *filename, RaucConfig **config, GError **error)
 					R_CONFIG_ERROR,
 					R_CONFIG_ERROR_INVALID_FORMAT,
 					"Only one of the keys 'variant-file', variant-dtb','variant-name' is allowed");
-			res = FALSE;
-			goto free;
+			return FALSE;
 		}
 
 		c->system_variant_type = R_CONFIG_SYS_VARIANT_NAME;
-		c->system_variant = variant_data;
+		c->system_variant = g_steal_pointer(&variant_data);
 	}
 
 	/* parse data/status location
@@ -361,7 +623,7 @@ gboolean load_config(const gchar *filename, RaucConfig **config, GError **error)
 	 * - central status file and shared data directory
 	 *   (``statusfile=/data/central.raucs`` and ``data-directory=/data/rauc``)
 	 * - central status file in shared data directory
-	 *   (``data-directory=/data/rauc``, implies ``statusfile=/data/rauc/central.rauc``)
+	 *   (``data-directory=/data/rauc``, implies ``statusfile=/data/rauc/central.raucs``)
 	 */
 	c->data_directory = resolve_path_take(filename,
 			key_file_consume_string(key_file, "system", "data-directory", &ierror));
@@ -369,8 +631,7 @@ gboolean load_config(const gchar *filename, RaucConfig **config, GError **error)
 		g_clear_error(&ierror);
 	} else if (ierror) {
 		g_propagate_error(error, ierror);
-		res = FALSE;
-		goto free;
+		return FALSE;
 	}
 
 	c->statusfile_path = key_file_consume_string(key_file, "system", "statusfile", &ierror);
@@ -384,8 +645,7 @@ gboolean load_config(const gchar *filename, RaucConfig **config, GError **error)
 		g_clear_error(&ierror);
 	} else if (ierror) {
 		g_propagate_error(error, ierror);
-		res = FALSE;
-		goto free;
+		return FALSE;
 	}
 
 	if (g_strcmp0(c->statusfile_path, "per-slot") == 0) {
@@ -395,8 +655,7 @@ gboolean load_config(const gchar *filename, RaucConfig **config, GError **error)
 					R_CONFIG_ERROR,
 					R_CONFIG_ERROR_DATA_DIRECTORY,
 					"Using data-directory= with statusfile=per-slot is not supported.");
-			res = FALSE;
-			goto free;
+			return FALSE;
 		}
 		g_message("Using per-slot statusfile");
 	} else {
@@ -416,20 +675,27 @@ gboolean load_config(const gchar *filename, RaucConfig **config, GError **error)
 		g_clear_error(&ierror);
 	} else if (ierror) {
 		g_propagate_error(error, ierror);
-		res = FALSE;
-		goto free;
+		return FALSE;
 	} else {
 		if (!parse_bundle_formats(&c->bundle_formats_mask, bundle_formats, &ierror)) {
 			g_propagate_error(error, ierror);
-			res = FALSE;
-			goto free;
+			return FALSE;
 		}
 	}
 
+	c->perform_pre_check = g_key_file_get_boolean(key_file, "system", "perform-pre-check", &ierror);
+	if (g_error_matches(ierror, G_KEY_FILE_ERROR, G_KEY_FILE_ERROR_KEY_NOT_FOUND)) {
+		c->perform_pre_check = FALSE;
+		g_clear_error(&ierror);
+	} else if (ierror) {
+		g_propagate_error(error, ierror);
+		return FALSE;
+	}
+	g_key_file_remove_key(key_file, "system", "perform-pre-check", NULL);
+
 	if (!check_remaining_keys(key_file, "system", &ierror)) {
 		g_propagate_error(error, ierror);
-		res = FALSE;
-		goto free;
+		return FALSE;
 	}
 	g_key_file_remove_group(key_file, "system", NULL);
 
@@ -446,8 +712,7 @@ gboolean load_config(const gchar *filename, RaucConfig **config, GError **error)
 		g_clear_error(&ierror);
 	} else if (ierror) {
 		g_propagate_error(error, ierror);
-		res = FALSE;
-		goto free;
+		return FALSE;
 	}
 	g_key_file_remove_key(key_file, "keyring", "check-crl", NULL);
 
@@ -458,8 +723,7 @@ gboolean load_config(const gchar *filename, RaucConfig **config, GError **error)
 		g_clear_error(&ierror);
 	} else if (ierror) {
 		g_propagate_error(error, ierror);
-		res = FALSE;
-		goto free;
+		return FALSE;
 	}
 	g_key_file_remove_key(key_file, "keyring", "allow-partial-chain", NULL);
 
@@ -470,8 +734,7 @@ gboolean load_config(const gchar *filename, RaucConfig **config, GError **error)
 		g_clear_error(&ierror);
 	} else if (ierror) {
 		g_propagate_error(error, ierror);
-		res = FALSE;
-		goto free;
+		return FALSE;
 	}
 	g_key_file_remove_key(key_file, "keyring", "use-bundle-signing-time", NULL);
 
@@ -482,14 +745,12 @@ gboolean load_config(const gchar *filename, RaucConfig **config, GError **error)
 		g_clear_error(&ierror);
 	} else if (ierror) {
 		g_propagate_error(error, ierror);
-		res = FALSE;
-		goto free;
+		return FALSE;
 	}
 
 	if (!check_remaining_keys(key_file, "keyring", &ierror)) {
 		g_propagate_error(error, ierror);
-		res = FALSE;
-		goto free;
+		return FALSE;
 	}
 	g_key_file_remove_group(key_file, "keyring", NULL);
 
@@ -504,14 +765,12 @@ gboolean load_config(const gchar *filename, RaucConfig **config, GError **error)
 		g_clear_error(&ierror);
 	} else if (ierror) {
 		g_propagate_error(error, ierror);
-		res = FALSE;
-		goto free;
+		return FALSE;
 	}
 	g_key_file_remove_key(key_file, "casync", "use-desync", NULL);
 	if (!check_remaining_keys(key_file, "casync", &ierror)) {
 		g_propagate_error(error, ierror);
-		res = FALSE;
-		goto free;
+		return FALSE;
 	}
 	g_key_file_remove_group(key_file, "casync", NULL);
 
@@ -522,8 +781,7 @@ gboolean load_config(const gchar *filename, RaucConfig **config, GError **error)
 	c->streaming_tls_ca = key_file_consume_string(key_file, "streaming", "tls-ca", NULL);
 	if (!check_remaining_keys(key_file, "streaming", &ierror)) {
 		g_propagate_error(error, ierror);
-		res = FALSE;
-		goto free;
+		return FALSE;
 	}
 	g_key_file_remove_group(key_file, "streaming", NULL);
 
@@ -534,8 +792,7 @@ gboolean load_config(const gchar *filename, RaucConfig **config, GError **error)
 			key_file_consume_string(key_file, "encryption", "cert", NULL));
 	if (!check_remaining_keys(key_file, "encryption", &ierror)) {
 		g_propagate_error(error, ierror);
-		res = FALSE;
-		goto free;
+		return FALSE;
 	}
 	g_key_file_remove_group(key_file, "encryption", NULL);
 
@@ -544,8 +801,7 @@ gboolean load_config(const gchar *filename, RaucConfig **config, GError **error)
 			key_file_consume_string(key_file, "autoinstall", "path", NULL));
 	if (!check_remaining_keys(key_file, "autoinstall", &ierror)) {
 		g_propagate_error(error, ierror);
-		res = FALSE;
-		goto free;
+		return FALSE;
 	}
 	g_key_file_remove_group(key_file, "autoinstall", NULL);
 
@@ -560,274 +816,26 @@ gboolean load_config(const gchar *filename, RaucConfig **config, GError **error)
 			key_file_consume_string(key_file, "handlers", "post-install", NULL));
 	if (!check_remaining_keys(key_file, "handlers", &ierror)) {
 		g_propagate_error(error, ierror);
-		res = FALSE;
-		goto free;
+		return FALSE;
 	}
 	g_key_file_remove_group(key_file, "handlers", NULL);
 
 	/* parse [slot.*.#] sections */
-	slots = g_hash_table_new_full(g_str_hash, g_str_equal, NULL, r_slot_free);
-	bootnames = g_hash_table_new(g_str_hash, g_str_equal);
-
-	groups = g_key_file_get_groups(key_file, &group_count);
-	for (gsize i = 0; i < group_count; i++) {
-		gchar **groupsplit;
-
-		groupsplit = g_strsplit(groups[i], ".", -1);
-
-		/* We treat sections starting with "slot." as slots */
-		if (g_str_equal(groupsplit[0], RAUC_SLOT_PREFIX)) {
-			g_autoptr(RaucSlot) slot = g_new0(RaucSlot, 1);
-			gchar* value;
-
-			/* Assure slot strings consist of 3 parts, delimited by dots */
-			if (g_strv_length(groupsplit) != 3) {
-				g_set_error(
-						error,
-						R_CONFIG_ERROR,
-						R_CONFIG_ERROR_INVALID_FORMAT,
-						"Invalid slot name format: %s", groups[i]);
-				res = FALSE;
-				goto free;
-			}
-
-			value = g_strconcat(groupsplit[1], ".", groupsplit[2], NULL);
-			if (!value) {
-				g_set_error(
-						error,
-						R_CONFIG_ERROR,
-						R_CONFIG_ERROR_INVALID_FORMAT,
-						"Invalid slot name");
-				res = FALSE;
-				goto free;
-			}
-			slot->name = g_intern_string(value);
-			g_free(value);
-
-			/* If we have a data_directory, use a slot.<class>.<index>
-			 * subdirectory for per-slot data. */
-			if (c->data_directory)
-				slot->data_directory = g_build_filename(c->data_directory, groups[i], NULL);
-
-			slot->description = key_file_consume_string(key_file, groups[i], "description", NULL);
-
-			slot->sclass = g_intern_string(groupsplit[1]);
-
-			value = resolve_path_take(filename,
-					key_file_consume_string(key_file, groups[i], "device", &ierror));
-			if (!value) {
-				g_propagate_error(error, ierror);
-				res = FALSE;
-				goto free;
-			}
-			slot->device = value;
-
-			value = key_file_consume_string(key_file, groups[i], "type", NULL);
-			if (!value)
-				value = g_strdup("raw");
-			slot->type = value;
-
-			if (!r_slot_is_valid_type(slot->type)) {
-				g_set_error(
-						error,
-						R_CONFIG_ERROR,
-						R_CONFIG_ERROR_SLOT_TYPE,
-						"Unsupported slot type '%s' for slot %s selected in system config", slot->type, slot->name);
-				res = FALSE;
-				goto free;
-			}
-
-			/* check if the device has an appropriate path */
-			if (g_str_equal(slot->type, "jffs2") && !g_str_has_prefix(slot->device, "/dev/")) {
-				g_set_error(
-						error,
-						R_CONFIG_ERROR,
-						R_CONFIG_ERROR_INVALID_DEVICE,
-						"%s: device must be located in /dev/ for jffs2", groups[i]);
-				res = FALSE;
-				goto free;
-			}
-
-			value = key_file_consume_string(key_file, groups[i], "bootname", NULL);
-			slot->bootname = value;
-			if (slot->bootname) {
-				/* check if we have seen this bootname on another slot */
-				if (g_hash_table_contains(bootnames, slot->bootname)) {
-					g_set_error(
-							error,
-							R_CONFIG_ERROR,
-							R_CONFIG_ERROR_DUPLICATE_BOOTNAME,
-							"Bootname '%s' is set on more than one slot",
-							slot->bootname);
-					res = FALSE;
-					goto free;
-				}
-				g_hash_table_add(bootnames, slot->bootname);
-			}
-
-			/* Collect name of parent here for easing remaining key checking.
-			 * Will be resolved to slot->parent pointer after config parsing loop. */
-			slot->parent_name = key_file_consume_string(key_file, groups[i], "parent", NULL);
-
-			slot->allow_mounted = g_key_file_get_boolean(key_file, groups[i], "allow-mounted", &ierror);
-			if (g_error_matches(ierror, G_KEY_FILE_ERROR, G_KEY_FILE_ERROR_KEY_NOT_FOUND)) {
-				slot->allow_mounted = FALSE;
-				g_clear_error(&ierror);
-			} else if (ierror) {
-				g_propagate_error(error, ierror);
-				res = FALSE;
-				goto free;
-			}
-			g_key_file_remove_key(key_file, groups[i], "allow-mounted", NULL);
-
-			slot->readonly = g_key_file_get_boolean(key_file, groups[i], "readonly", &ierror);
-			if (g_error_matches(ierror, G_KEY_FILE_ERROR, G_KEY_FILE_ERROR_KEY_NOT_FOUND)) {
-				slot->readonly = FALSE;
-				g_clear_error(&ierror);
-			} else if (ierror) {
-				g_propagate_error(error, ierror);
-				res = FALSE;
-				goto free;
-			}
-			g_key_file_remove_key(key_file, groups[i], "readonly", NULL);
-
-			slot->install_same = g_key_file_get_boolean(key_file, groups[i], "install-same", &ierror);
-			if (g_error_matches(ierror, G_KEY_FILE_ERROR, G_KEY_FILE_ERROR_KEY_NOT_FOUND)) {
-				g_clear_error(&ierror);
-				/* try also deprecated flag force-install-same */
-				slot->install_same = g_key_file_get_boolean(key_file, groups[i], "force-install-same", &ierror);
-				if (g_error_matches(ierror, G_KEY_FILE_ERROR, G_KEY_FILE_ERROR_KEY_NOT_FOUND)) {
-					g_clear_error(&ierror);
-					/* try also deprecated flag ignore-checksum */
-					slot->install_same = g_key_file_get_boolean(key_file, groups[i], "ignore-checksum", &ierror);
-					if (g_error_matches(ierror, G_KEY_FILE_ERROR, G_KEY_FILE_ERROR_KEY_NOT_FOUND)) {
-						slot->install_same = TRUE;
-						g_clear_error(&ierror);
-					} else if (ierror) {
-						g_propagate_error(error, ierror);
-						res = FALSE;
-						goto free;
-					}
-				} else if (ierror) {
-					g_propagate_error(error, ierror);
-					res = FALSE;
-					goto free;
-				}
-			} else if (ierror) {
-				g_propagate_error(error, ierror);
-				res = FALSE;
-				goto free;
-			}
-			g_key_file_remove_key(key_file, groups[i], "install-same", NULL);
-			g_key_file_remove_key(key_file, groups[i], "force-install-same", NULL);
-			g_key_file_remove_key(key_file, groups[i], "ignore-checksum", NULL);
-
-			slot->extra_mount_opts = key_file_consume_string(key_file, groups[i], "extra-mount-opts", NULL);
-
-			slot->resize = g_key_file_get_boolean(key_file, groups[i], "resize", &ierror);
-			if (g_error_matches(ierror, G_KEY_FILE_ERROR, G_KEY_FILE_ERROR_KEY_NOT_FOUND)) {
-				slot->resize = FALSE;
-				g_clear_error(&ierror);
-			} else if (ierror) {
-				g_propagate_error(error, ierror);
-				res = FALSE;
-				goto free;
-			}
-			g_key_file_remove_key(key_file, groups[i], "resize", NULL);
-
-			if (g_strcmp0(slot->type, "boot-mbr-switch") == 0 ||
-			    g_strcmp0(slot->type, "boot-gpt-switch") == 0 ||
-			    g_strcmp0(slot->type, "boot-raw-fallback") == 0) {
-				slot->region_start = key_file_consume_binary_suffixed_string(key_file, groups[i],
-						"region-start", &ierror);
-				if (ierror) {
-					g_propagate_prefixed_error(error, ierror, "mandatory for %s: ", slot->type);
-					res = FALSE;
-					goto free;
-				}
-
-				slot->region_size = key_file_consume_binary_suffixed_string(key_file, groups[i],
-						"region-size", &ierror);
-				if (ierror) {
-					g_propagate_prefixed_error(error, ierror, "mandatory for %s: ", slot->type);
-					res = FALSE;
-					goto free;
-				}
-			}
-
-			if (!check_remaining_keys(key_file, groups[i], &ierror)) {
-				g_propagate_error(error, ierror);
-				res = FALSE;
-				goto free;
-			}
-
-			g_key_file_remove_group(key_file, groups[i], NULL);
-
-			g_hash_table_insert(slots, (gchar*)slot->name, slot);
-			slot = NULL;
-		}
-		g_strfreev(groupsplit);
-	}
-
-	/* Add parent pointers */
-	slotlist = g_hash_table_get_keys(slots);
-	for (l = slotlist; l != NULL; l = l->next) {
-		RaucSlot *slot;
-		RaucSlot *parent;
-		RaucSlot *child;
-
-		slot = g_hash_table_lookup(slots, l->data);
-		if (!slot->parent_name) {
-			continue;
-		}
-
-		parent = g_hash_table_lookup(slots, slot->parent_name);
-		if (!parent) {
-			g_set_error(
-					error,
-					R_CONFIG_ERROR,
-					R_CONFIG_ERROR_PARENT,
-					"Parent slot '%s' not found!", slot->parent_name);
-			res = FALSE;
-			goto free;
-		}
-
-		child = g_hash_table_lookup(slots, l->data);
-		child->parent = parent;
-
-		if (child->bootname) {
-			g_set_error(
-					error,
-					R_CONFIG_ERROR,
-					R_CONFIG_ERROR_CHILD_HAS_BOOTNAME,
-					"Child slot '%s' has bootname set",
-					child->name);
-			res = FALSE;
-			goto free;
-		}
-	}
-
-	if (!fix_grandparent_links(slots, &ierror)) {
+	c->slots = parse_slots(filename, c->data_directory, key_file, &ierror);
+	if (!c->slots) {
 		g_propagate_error(error, ierror);
-		res = FALSE;
-		goto free;
+		return FALSE;
 	}
 
-	c->slots = slots;
-
 	if (!check_remaining_groups(key_file, &ierror)) {
 		g_propagate_error(error, ierror);
-		res = FALSE;
-		goto free;
+		return FALSE;
 	}
 
-	res = TRUE;
-free:
-	if (res)
-		*config = g_steal_pointer(&c);
-	else
-		*config = NULL;
-	return res;
+	/* on success, return config struct */
+	*config = g_steal_pointer(&c);
+
+	return TRUE;
 }
 
 RaucSlot *find_config_slot_by_device(RaucConfig *config, const gchar *device)
@@ -859,7 +867,9 @@ void free_config(RaucConfig *config)
 	g_free(config->mount_prefix);
 	g_free(config->store_path);
 	g_free(config->tmp_path);
+	g_free(config->casync_install_args);
 	g_free(config->grubenv_path);
+	g_free(config->data_directory);
 	g_free(config->statusfile_path);
 	g_free(config->keyring_path);
 	g_free(config->keyring_directory);
@@ -887,19 +897,13 @@ static void status_file_get_slot_status(GKeyFile *key_file, const gchar *group,
 	if (!g_key_file_has_group(key_file, group))
 		g_debug("Group %s not found in key file.", group);
 
-	g_free(slotstatus->bundle_compatible);
-	g_free(slotstatus->bundle_version);
-	g_free(slotstatus->bundle_description);
-	g_free(slotstatus->bundle_build);
-	g_free(slotstatus->status);
-	g_clear_pointer(&slotstatus->checksum.digest, g_free);
-	g_free(slotstatus->installed_timestamp);
-	g_free(slotstatus->activated_timestamp);
+	r_slot_clear_status(slotstatus);
 
 	slotstatus->bundle_compatible = key_file_consume_string(key_file, group, "bundle.compatible", NULL);
 	slotstatus->bundle_version = key_file_consume_string(key_file, group, "bundle.version", NULL);
 	slotstatus->bundle_description = key_file_consume_string(key_file, group, "bundle.description", NULL);
 	slotstatus->bundle_build = key_file_consume_string(key_file, group, "bundle.build", NULL);
+	slotstatus->bundle_hash = key_file_consume_string(key_file, group, "bundle.hash", NULL);
 	slotstatus->status = key_file_consume_string(key_file, group, "status", NULL);
 
 	digest = key_file_consume_string(key_file, group, "sha256", NULL);
@@ -909,6 +913,7 @@ static void status_file_get_slot_status(GKeyFile *key_file, const gchar *group,
 		slotstatus->checksum.size = g_key_file_get_uint64(key_file, group, "size", NULL);
 	}
 
+	slotstatus->installed_txn = key_file_consume_string(key_file, group, "installed.transaction", NULL);
 	slotstatus->installed_timestamp = key_file_consume_string(key_file, group, "installed.timestamp", NULL);
 	count = g_key_file_get_uint64(key_file, group, "installed.count", &ierror);
 	if (g_error_matches(ierror, G_KEY_FILE_ERROR, G_KEY_FILE_ERROR_INVALID_VALUE))
@@ -956,6 +961,7 @@ static void status_file_set_slot_status(GKeyFile *key_file, const gchar *group,
 	status_file_set_string_or_remove_key(key_file, group, "bundle.version", slotstatus->bundle_version);
 	status_file_set_string_or_remove_key(key_file, group, "bundle.description", slotstatus->bundle_description);
 	status_file_set_string_or_remove_key(key_file, group, "bundle.build", slotstatus->bundle_build);
+	status_file_set_string_or_remove_key(key_file, group, "bundle.hash", slotstatus->bundle_hash);
 	status_file_set_string_or_remove_key(key_file, group, "status", slotstatus->status);
 
 	if (slotstatus->checksum.digest && slotstatus->checksum.type == G_CHECKSUM_SHA256) {
@@ -966,19 +972,29 @@ static void status_file_set_slot_status(GKeyFile *key_file, const gchar *group,
 		g_key_file_remove_key(key_file, group, "size", NULL);
 	}
 
+	status_file_set_string_or_remove_key(key_file, group, "installed.transaction", slotstatus->installed_txn);
+
 	if (slotstatus->installed_timestamp) {
 		g_key_file_set_string(key_file, group, "installed.timestamp", slotstatus->installed_timestamp);
-		g_key_file_set_uint64(key_file, group, "installed.count", slotstatus->installed_count);
 	} else {
 		g_key_file_remove_key(key_file, group, "installed.timestamp", NULL);
+	}
+
+	if (slotstatus->installed_count > 0) {
+		g_key_file_set_uint64(key_file, group, "installed.count", slotstatus->installed_count);
+	} else {
 		g_key_file_remove_key(key_file, group, "installed.count", NULL);
 	}
 
 	if (slotstatus->activated_timestamp) {
 		g_key_file_set_string(key_file, group, "activated.timestamp", slotstatus->activated_timestamp);
-		g_key_file_set_uint64(key_file, group, "activated.count", slotstatus->activated_count);
 	} else {
 		g_key_file_remove_key(key_file, group, "activated.timestamp", NULL);
+	}
+
+	if (slotstatus->activated_count > 0) {
+		g_key_file_set_uint64(key_file, group, "activated.count", slotstatus->activated_count);
+	} else {
 		g_key_file_remove_key(key_file, group, "activated.count", NULL);
 	}
 
