@@ -5,12 +5,14 @@
 #include "config_file.h"
 #include "context.h"
 #include "network.h"
+#include "install.h"
 #include "signature.h"
 #include "utils.h"
 
 RaucContext *context = NULL;
+gboolean context_configuring = FALSE;
 
-static const gchar *regex_match(const gchar *pattern, const gchar *string)
+static gchar *regex_match(const gchar *pattern, const gchar *string)
 {
 	g_autoptr(GRegex) regex = NULL;
 	g_autoptr(GMatchInfo) match = NULL;
@@ -25,11 +27,39 @@ static const gchar *regex_match(const gchar *pattern, const gchar *string)
 	return NULL;
 }
 
-static const gchar* get_cmdline_bootname(void)
+static gchar* get_machine_id(void)
+{
+	gchar *contents = NULL;
+	g_autoptr(GError) ierror = NULL;
+
+	if (!g_file_get_contents("/etc/machine-id", &contents, NULL, &ierror)) {
+		g_info("Failed to get machine-id: %s", ierror->message);
+		return NULL;
+	}
+
+	/* file contains newline, modify in-place */
+	return g_strchomp(contents);
+}
+
+static gchar* get_boot_id(void)
+{
+	gchar *contents = NULL;
+	g_autoptr(GError) ierror = NULL;
+
+	if (!g_file_get_contents("/proc/sys/kernel/random/boot_id", &contents, NULL, &ierror)) {
+		g_warning("Failed to get boot_id: %s", ierror->message);
+		return NULL;
+	}
+
+	/* file contains newline, modify in-place */
+	return g_strchomp(contents);
+}
+
+static gchar* get_cmdline_bootname(void)
 {
 	g_autofree gchar *contents = NULL;
 	g_autofree gchar *realdev = NULL;
-	const char *bootname = NULL;
+	gchar *bootname = NULL;
 
 	if (context->mock.proc_cmdline)
 		contents = g_strdup(context->mock.proc_cmdline);
@@ -37,7 +67,7 @@ static const gchar* get_cmdline_bootname(void)
 		return NULL;
 
 	if (strstr(contents, "rauc.external") != NULL)
-		return "_external_";
+		return g_strdup("_external_");
 
 	bootname = regex_match("rauc\\.slot=(\\S+)", contents);
 	if (bootname)
@@ -109,31 +139,32 @@ static const gchar* get_cmdline_bootname(void)
 	return bootname;
 }
 
-static gboolean launch_and_wait_variables_handler(gchar *handler_name, GHashTable *variables, GError **error)
+/**
+ * Launches a handler and obtains variables from output by looking for
+ * 'RAUC_<SOMETHING>=value' lines to put them into a key/value store (GHashTable).
+ *
+ * @param handler_name name / path of handler script to start
+ * @param[out] variables Return location for a GHashTable table with obtained key/value-pairs
+ * @param[out] error Return location for a GError, or NULL
+ *
+ * @return TRUE on success, otherwise FALSE
+ */
+static gboolean launch_and_wait_variables_handler(gchar *handler_name, GHashTable **variables, GError **error)
 {
 	g_autoptr(GSubprocessLauncher) handlelaunch = NULL;
 	g_autoptr(GSubprocess) handleproc = NULL;
 	GError *ierror = NULL;
-	GHashTableIter iter;
-	gchar *key = NULL;
-	gchar *value = NULL;
 	g_autoptr(GDataInputStream) datainstream = NULL;
 	GInputStream *instream;
+	g_autoptr(GHashTable) vars = NULL;
 	gchar* outline;
 
 	g_return_val_if_fail(handler_name, FALSE);
-	g_return_val_if_fail(variables, FALSE);
+	g_return_val_if_fail(variables && *variables == NULL, FALSE);
 	g_return_val_if_fail(error == NULL || *error == NULL, FALSE);
 
 	handlelaunch = g_subprocess_launcher_new(G_SUBPROCESS_FLAGS_STDOUT_PIPE | G_SUBPROCESS_FLAGS_STDERR_MERGE);
 
-	/* we copy the variables from the hashtable and add them to the
-	   subprocess environment */
-	g_hash_table_iter_init(&iter, variables);
-	while (g_hash_table_iter_next(&iter, (gpointer*) &key, (gpointer*) &value)) {
-		g_subprocess_launcher_setenv(handlelaunch, g_strdup(key), g_strdup(value), 1);
-	}
-
 	handleproc = g_subprocess_launcher_spawn(
 			handlelaunch,
 			&ierror, handler_name,
@@ -148,6 +179,8 @@ static gboolean launch_and_wait_variables_handler(gchar *handler_name, GHashTabl
 	instream = g_subprocess_get_stdout_pipe(handleproc);
 	datainstream = g_data_input_stream_new(instream);
 
+	vars = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, g_free);
+
 	do {
 		outline = g_data_input_stream_read_line(datainstream, NULL, NULL, NULL);
 		if (!outline)
@@ -157,11 +190,12 @@ static gboolean launch_and_wait_variables_handler(gchar *handler_name, GHashTabl
 			g_auto(GStrv) split = g_strsplit(outline, "=", 2);
 
 			if (g_strv_length(split) != 2) {
+				g_message("Failed to convert '%s' line to variable", outline);
 				g_free(outline);
 				continue;
 			}
 
-			g_hash_table_insert(variables, g_strdup(split[0]), g_strdup(split[1]));
+			g_hash_table_insert(vars, g_strdup(split[0]), g_strdup(split[1]));
 		}
 
 		g_free(outline);
@@ -172,6 +206,8 @@ static gboolean launch_and_wait_variables_handler(gchar *handler_name, GHashTabl
 		return FALSE;
 	}
 
+	*variables = g_steal_pointer(&vars);
+
 	return TRUE;
 }
 
@@ -219,59 +255,51 @@ static gchar* get_variant_from_file(const gchar* filename, GError **error)
 	return contents;
 }
 
+static GHashTable *get_system_info_from_handler(GError **error)
+{
+	g_autoptr(GHashTable) vars = NULL;
+	GError *ierror = NULL;
+	GHashTableIter iter;
+	gchar *key = NULL;
+	gchar *value = NULL;
 
-gboolean r_context_configure(GError **error)
+	if (!g_file_test(context->config->systeminfo_handler, G_FILE_TEST_EXISTS)) {
+		g_set_error(error, G_FILE_ERROR, G_FILE_ERROR_NOENT,  "System info handler script/binary '%s' not found.", context->config->systeminfo_handler);
+		return NULL;
+	}
+
+	g_message("Getting Systeminfo: %s", context->config->systeminfo_handler);
+	if (!launch_and_wait_variables_handler(context->config->systeminfo_handler, &vars, &ierror)) {
+		g_propagate_prefixed_error(error, ierror, "Failed to read system-info variables: ");
+		return NULL;
+	}
+
+	g_hash_table_iter_init(&iter, vars);
+	while (g_hash_table_iter_next(&iter, (gpointer*) &key, (gpointer*) &value)) {
+		/* legacy handling */
+		if (g_strcmp0(key, "RAUC_SYSTEM_SERIAL") == 0) {
+			g_clear_pointer(&context->system_serial, g_free);
+			context->system_serial = g_strdup(value);
+		} else if (g_strcmp0(key, "RAUC_SYSTEM_VARIANT") == 0) {
+			/* set variant (overrides possible previous value) */
+			g_clear_pointer(&context->config->system_variant, g_free);
+			context->config->system_variant = g_strdup(value);
+		}
+	}
+
+	return g_steal_pointer(&vars);
+}
+
+/**
+ * Configures options that are only relevant when running as update service on
+ * the target device.
+ */
+static gboolean r_context_configure_target(GError **error)
 {
-	gboolean res = TRUE;
 	GError *ierror = NULL;
-	RContextConfigMode configmode;
-	const gchar *configpath = NULL;
 
 	g_return_val_if_fail(error == NULL || *error == NULL, FALSE);
 
-	g_assert_nonnull(context);
-	g_assert_false(context->busy);
-
-	g_clear_pointer(&context->config, free_config);
-	configmode = context->configmode;
-	if (context->configpath) {
-		/* explicitly set on the command line */
-		configmode = R_CONTEXT_CONFIG_MODE_REQUIRED;
-		configpath = context->configpath;
-	} else {
-		/* the default path */
-		configpath = "/etc/rauc/system.conf";
-	}
-	switch (configmode) {
-		case R_CONTEXT_CONFIG_MODE_NONE:
-			default_config(&context->config);
-			break;
-		case R_CONTEXT_CONFIG_MODE_AUTO:
-			if (load_config(configpath, &context->config, &ierror)) {
-				g_message("valid %s found, using it", configpath);
-				if (!context->configpath)
-					context->configpath = g_strdup(configpath);
-			} else if (ierror->domain != G_FILE_ERROR) {
-				g_propagate_prefixed_error(error, ierror, "Failed to load system config (%s): ", configpath);
-				return FALSE;
-			} else {
-				/* This is a hack as we cannot get rid of config easily */
-				default_config(&context->config);
-			}
-			break;
-		case R_CONTEXT_CONFIG_MODE_REQUIRED:
-			if (!load_config(configpath, &context->config, &ierror)) {
-				g_propagate_prefixed_error(error, ierror, "Failed to load system config (%s): ", configpath);
-				return FALSE;
-			}
-			if (!context->configpath)
-				context->configpath = g_strdup(configpath);
-			break;
-		default:
-			g_error("invalid context config mode %d", configmode);
-			break;
-	}
-
 	if (context->config->data_directory) {
 		if (g_mkdir_with_parents(context->config->data_directory, 0700) != 0) {
 			int err = errno;
@@ -304,41 +332,86 @@ gboolean r_context_configure(GError **error)
 		context->config->system_variant = variant;
 	}
 
-	if (context->config->systeminfo_handler &&
-	    g_file_test(context->config->systeminfo_handler, G_FILE_TEST_EXISTS)) {
-		g_autoptr(GHashTable) vars = NULL;
-		GHashTableIter iter;
-		gchar *key = NULL;
-		gchar *value = NULL;
-
-		vars = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, g_free);
-
-		g_message("Getting Systeminfo: %s", context->config->systeminfo_handler);
-		res = launch_and_wait_variables_handler(context->config->systeminfo_handler, vars, &ierror);
-		if (!res) {
-			g_propagate_prefixed_error(error, ierror, "Failed to read system-info variables: ");
+	g_clear_pointer(&context->system_info, g_hash_table_destroy);
+	if (context->config->systeminfo_handler) {
+		context->system_info = get_system_info_from_handler(&ierror);
+		if (!context->system_info) {
+			g_propagate_error(error, ierror);
 			return FALSE;
 		}
-
-		g_hash_table_iter_init(&iter, vars);
-		while (g_hash_table_iter_next(&iter, (gpointer*) &key, (gpointer*) &value)) {
-			if (g_strcmp0(key, "RAUC_SYSTEM_SERIAL") == 0) {
-				context->system_serial = g_strdup(value);
-			} else if (g_strcmp0(key, "RAUC_SYSTEM_VARIANT") == 0) {
-				/* set variant (overrides possible previous value) */
-				g_free(context->config->system_variant);
-				context->config->system_variant = g_strdup(value);
-			} else {
-				g_message("Ignoring unknown variable %s", key);
-			}
-		}
+	} else {
+		/* Ensure the hash table is always created so that we do not need to check this later */
+		context->system_info = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, g_free);
 	}
 
 	if (r_whitespace_removed(context->config->system_variant))
 		g_warning("Ignoring surrounding whitespace in system variant: %s", context->config->system_variant);
 
 	if (context->bootslot == NULL) {
-		context->bootslot = g_strdup(get_cmdline_bootname());
+		context->bootslot = get_cmdline_bootname();
+	}
+
+	g_clear_pointer(&context->boot_id, g_free);
+	g_clear_pointer(&context->machine_id, g_free);
+
+	context->boot_id = get_boot_id();
+	g_debug("Obtained system boot ID: '%s'", context->boot_id);
+	context->machine_id = get_machine_id();
+	g_debug("Obtained system machine ID: '%s'", context->machine_id);
+
+	return TRUE;
+}
+
+gboolean r_context_configure(GError **error)
+{
+	GError *ierror = NULL;
+	RContextConfigMode configmode;
+	const gchar *configpath = NULL;
+
+	g_return_val_if_fail(error == NULL || *error == NULL, FALSE);
+
+	g_assert_nonnull(context);
+	g_assert_false(context->busy);
+
+	g_clear_pointer(&context->config, free_config);
+	configmode = context->configmode;
+	if (context->configpath) {
+		/* explicitly set on the command line */
+		configmode = R_CONTEXT_CONFIG_MODE_REQUIRED;
+		configpath = context->configpath;
+	} else {
+		/* the default path */
+		configpath = "/etc/rauc/system.conf";
+	}
+	switch (configmode) {
+		case R_CONTEXT_CONFIG_MODE_NONE:
+			default_config(&context->config);
+			break;
+		case R_CONTEXT_CONFIG_MODE_AUTO:
+			if (load_config(configpath, &context->config, &ierror)) {
+				g_message("valid %s found, using it", configpath);
+				if (!context->configpath)
+					context->configpath = g_strdup(configpath);
+			} else if (ierror->domain != G_FILE_ERROR) {
+				g_propagate_prefixed_error(error, ierror, "Failed to load system config (%s): ", configpath);
+				return FALSE;
+			} else {
+				g_clear_error(&ierror);
+				/* This is a hack as we cannot get rid of config easily */
+				default_config(&context->config);
+			}
+			break;
+		case R_CONTEXT_CONFIG_MODE_REQUIRED:
+			if (!load_config(configpath, &context->config, &ierror)) {
+				g_propagate_prefixed_error(error, ierror, "Failed to load system config (%s): ", configpath);
+				return FALSE;
+			}
+			if (!context->configpath)
+				context->configpath = g_strdup(configpath);
+			break;
+		default:
+			g_error("invalid context config mode %d", configmode);
+			break;
 	}
 
 	if (context->mountprefix) {
@@ -358,6 +431,20 @@ gboolean r_context_configure(GError **error)
 		context->config->encryption_key = g_strdup(context->encryption_key);
 	}
 
+	/* When no context is required, we can safely assume that we do not
+	 * operate on the target but are used as a (host) tool.
+	 * In this case, skip all the necessary target-related context setup steps
+	 */
+	if (configmode != R_CONTEXT_CONFIG_MODE_REQUIRED) {
+		context->pending = FALSE;
+
+		return TRUE;
+	}
+
+	if (!r_context_configure_target(error)) {
+		return FALSE;
+	}
+
 	context->pending = FALSE;
 
 	return TRUE;
@@ -420,12 +507,11 @@ static void r_context_send_progress(gboolean op_finished, gboolean success)
 
 	/* call installer callback with percentage and message */
 	if (op_finished) {
+		g_autofree gchar *old = step->description;
 		if (success)
-			step->description = g_strdup_printf("%s done.",
-					step->description);
+			step->description = g_strdup_printf("%s done.", old);
 		else
-			step->description = g_strdup_printf("%s failed.",
-					step->description);
+			step->description = g_strdup_printf("%s failed.", old);
 	}
 
 	/* handle missing callback gracefully */
@@ -436,6 +522,12 @@ static void r_context_send_progress(gboolean op_finished, gboolean success)
 
 void r_context_begin_step(const gchar *name, const gchar *description,
 		gint substeps)
+{
+	r_context_begin_step_weighted(name, description, substeps, 1);
+}
+
+void r_context_begin_step_weighted(const gchar *name, const gchar *description,
+		gint substeps, gint weight)
 {
 	RaucProgressStep *step = g_new0(RaucProgressStep, 1);
 	RaucProgressStep *parent;
@@ -446,6 +538,7 @@ void r_context_begin_step(const gchar *name, const gchar *description,
 	/* set properties */
 	step->name = g_strdup(name);
 	step->description = g_strdup(description);
+	step->weight = weight;
 	step->substeps_total = substeps;
 	step->substeps_done = 0;
 	step->percent_done = 0;
@@ -463,7 +556,7 @@ void r_context_begin_step(const gchar *name, const gchar *description,
 					parent->substeps_done + 1,
 					parent->substeps_total);
 
-		step->percent_total = parent->percent_total
+		step->percent_total = step->weight * parent->percent_total
 		                      / parent->substeps_total;
 
 		g_assert_cmpint(step->percent_total, <=,
@@ -495,6 +588,21 @@ void r_context_begin_step_formatted(const gchar *name, gint substeps, const gcha
 	r_context_begin_step(name, desc_formatted, substeps);
 }
 
+void r_context_begin_step_weighted_formatted(const gchar *name, gint substeps, gint weight, const gchar *description, ...)
+{
+	va_list args;
+	g_autofree gchar *desc_formatted = NULL;
+
+	g_return_if_fail(name);
+	g_return_if_fail(description);
+
+	va_start(args, description);
+	desc_formatted = g_strdup_vprintf(description, args);
+	va_end(args);
+
+	r_context_begin_step_weighted(name, desc_formatted, substeps, weight);
+}
+
 void r_context_end_step(const gchar *name, gboolean success)
 {
 	RaucProgressStep *step;
@@ -534,7 +642,7 @@ void r_context_end_step(const gchar *name, gboolean success)
 	/* increment step count and percentage on parent step */
 	if (g_list_next(context->progress)) {
 		parent = g_list_next(context->progress)->data;
-		parent->substeps_done++;
+		parent->substeps_done += step->weight;
 
 		/* clean up explicit percentage */
 		if (step->last_explicit_percent != 0)
@@ -644,9 +752,15 @@ const RaucContext *r_context(void)
 
 	g_assert_nonnull(context);
 
-	if (context->pending)
+	if (context_configuring)
+		g_error("Detected call of r_context() while still setting up context! Aborted to avoid infinite recursion!");
+
+	if (context->pending) {
+		context_configuring = TRUE;
 		if (!r_context_configure(&ierror))
 			g_error("Failed to initialize context: %s", ierror->message);
+		context_configuring = FALSE;
+	}
 
 	return context;
 }
@@ -674,7 +788,11 @@ void r_context_clean(void)
 		g_clear_pointer(&context->intermediatepaths, g_strfreev);
 		g_clear_pointer(&context->mountprefix, g_free);
 		g_clear_pointer(&context->bootslot, g_free);
+		g_clear_pointer(&context->boot_id, g_free);
+		g_clear_pointer(&context->machine_id, g_free);
 		g_clear_pointer(&context->system_serial, g_free);
+		g_clear_pointer(&context->system_info, g_hash_table_destroy);
+
 		g_clear_pointer(&context->handlerextra, g_free);
 
 		g_clear_pointer(&context->install_info, r_context_install_info_free);
