@@ -37,12 +37,14 @@ gboolean status_detailed = FALSE;
 gchar *output_format = NULL;
 gchar *keypath = NULL;
 gchar *certpath = NULL;
+gchar **intermediate = NULL;
 gchar *signing_keyring = NULL;
 gchar *mksquashfs_args = NULL;
 gchar *casync_args = NULL;
 gchar **recipients = NULL;
 gchar *handler_args = NULL;
 gchar *bootslot = NULL;
+gchar *installation_txn = NULL;
 gboolean utf8_supported = FALSE;
 RaucBundleAccessArgs access_args = {0};
 
@@ -219,7 +221,7 @@ static gboolean install_start(int argc, char **argv)
 	r_exit_status = 1;
 
 	if (argc < 3) {
-		g_printerr("A bundle filename name must be provided\n");
+		g_printerr("A bundle path or URL must be provided\n");
 		goto out;
 	}
 
@@ -241,6 +243,7 @@ static gboolean install_start(int argc, char **argv)
 	args->status_result = 2;
 
 	args->ignore_compatible = install_ignore_compatible;
+	args->transaction = installation_txn;
 	if (access_args.tls_cert)
 		args->access_args.tls_cert = g_strdup(access_args.tls_cert);
 	if (access_args.tls_key)
@@ -257,6 +260,8 @@ static gboolean install_start(int argc, char **argv)
 		g_auto(GVariantDict) dict = G_VARIANT_DICT_INIT(NULL);
 
 		g_variant_dict_insert(&dict, "ignore-compatible", "b", args->ignore_compatible);
+		if (args->transaction)
+			g_variant_dict_insert(&dict, "transaction-id", "s", args->transaction);
 		if (args->access_args.tls_cert)
 			g_variant_dict_insert(&dict, "tls-cert", "s", args->access_args.tls_cert);
 		if (args->access_args.tls_key)
@@ -298,6 +303,13 @@ static gboolean install_start(int argc, char **argv)
 			goto out_loop;
 		}
 	} else {
+		if (!determine_slot_states(&error)) {
+			g_printerr("Failed to determine slot states: %s\n", error->message);
+			g_clear_error(&error);
+			r_exit_status = 1;
+			return TRUE;
+		}
+
 		r_context_register_progress_callback(print_progress_callback);
 		install_run(args);
 	}
@@ -856,6 +868,7 @@ static gchar *info_formatter_shell(RaucManifest *manifest)
 	formatter_shell_append(text, "RAUC_MF_VERSION", manifest->update_version);
 	formatter_shell_append(text, "RAUC_MF_DESCRIPTION", manifest->update_description);
 	formatter_shell_append(text, "RAUC_MF_BUILD", manifest->update_build);
+	formatter_shell_append(text, "RAUC_MF_HASH", manifest->hash);
 	g_string_append_printf(text, "RAUC_MF_IMAGES=%d\n", g_list_length(manifest->images));
 
 	hooks = g_ptr_array_new();
@@ -870,6 +883,34 @@ static gchar *info_formatter_shell(RaucManifest *manifest)
 
 	g_ptr_array_unref(hooks);
 
+	if (manifest->meta && g_hash_table_size(manifest->meta)) {
+		GHashTableIter iter;
+		GHashTable *kvs;
+		const gchar *group;
+
+		g_hash_table_iter_init(&iter, manifest->meta);
+		while (g_hash_table_iter_next(&iter, (gpointer*)&group, (gpointer*)&kvs)) {
+			GHashTableIter kvs_iter;
+			const gchar *key, *value;
+			g_autofree gchar *env_group = r_prepare_env_key(group, NULL);
+
+			if (!env_group)
+				continue;
+
+			g_hash_table_iter_init(&kvs_iter, kvs);
+			while (g_hash_table_iter_next(&kvs_iter, (gpointer*)&key, (gpointer*)&value)) {
+				g_autofree gchar *env_key = r_prepare_env_key(key, NULL);
+				g_autofree gchar *var = NULL;
+
+				if (!env_key)
+					continue;
+
+				var = g_strdup_printf("RAUC_META_%s_%s", env_group, env_key);
+				formatter_shell_append(text, var, value);
+			}
+		}
+	}
+
 	cnt = 0;
 	for (GList *l = manifest->images; l != NULL; l = l->next) {
 		RaucImage *img = l->data;
@@ -949,9 +990,31 @@ static gchar *info_formatter_readable(RaucManifest *manifest)
 		g_string_append_printf(text, "  Verity Hash: \t'%s'\n", manifest->bundle_verity_hash);
 		g_string_append_printf(text, "  Verity Size: \t%"G_GUINT64_FORMAT "\n", manifest->bundle_verity_size);
 	}
+	g_string_append_printf(text, "Manifest Hash:\t'%s'\n\n", manifest->hash);
 
 	g_ptr_array_unref(hooks);
 
+	if (manifest->meta && g_hash_table_size(manifest->meta)) {
+		GHashTableIter iter;
+		GHashTable *kvs;
+		const gchar *group;
+
+		g_string_append_printf(text, "Metadata:\n");
+
+		g_hash_table_iter_init(&iter, manifest->meta);
+		while (g_hash_table_iter_next(&iter, (gpointer*)&group, (gpointer*)&kvs)) {
+			GHashTableIter kvs_iter;
+			const gchar *key, *value;
+
+			g_string_append_printf(text, "\t%s:\n", group);
+
+			g_hash_table_iter_init(&kvs_iter, kvs);
+			while (g_hash_table_iter_next(&kvs_iter, (gpointer*)&key, (gpointer*)&value)) {
+				g_string_append_printf(text, "\t\t%s: %s\n", key, value);
+			}
+		}
+	}
+
 	cnt = g_list_length(manifest->images);
 	g_string_append_printf(text, "\n%d Image%s%s\n", cnt, cnt == 1 ? "" : "s", cnt > 0 ? ":" : "");
 	cnt = 1;
@@ -961,9 +1024,10 @@ static gchar *info_formatter_readable(RaucManifest *manifest)
 		if (img->variant)
 			g_string_append_printf(text, "\tVariant:   %s\n", img->variant);
 		if (img->filename) {
+			g_autofree gchar* formatted_size = g_format_size_full(img->checksum.size, G_FORMAT_SIZE_LONG_FORMAT);
 			g_string_append_printf(text, "\tFilename:  %s\n", img->filename);
 			g_string_append_printf(text, "\tChecksum:  %s\n", img->checksum.digest);
-			g_string_append_printf(text, "\tSize:      %"G_GOFFSET_FORMAT "\n", img->checksum.size);
+			g_string_append_printf(text, "\tSize:      %s\n", formatted_size);
 		} else {
 			g_string_append_printf(text, "\t(no image file)\n");
 		}
@@ -999,6 +1063,23 @@ static gchar *info_formatter_readable(RaucManifest *manifest)
 }
 
 
+#if ENABLE_JSON
+/* Takes a GStrv and adds a JSON array to the builder. If the GStrv is NULL, an
+ * empty array is added. */
+static void strv_to_json_array(JsonBuilder *builder, GStrv strv)
+{
+	g_return_if_fail(JSON_IS_BUILDER(builder));
+
+	json_builder_begin_array(builder);
+	if (strv) {
+		for (gchar **m = strv; *m != NULL; m++) {
+			json_builder_add_string_value(builder, *m);
+		}
+	}
+	json_builder_end_array(builder);
+}
+#endif
+
 static gchar* info_formatter_json_base(RaucManifest *manifest, gboolean pretty)
 {
 #if ENABLE_JSON
@@ -1027,6 +1108,9 @@ static gchar* info_formatter_json_base(RaucManifest *manifest, gboolean pretty)
 	}
 	json_builder_end_array(builder);
 
+	json_builder_set_member_name(builder, "hash");
+	json_builder_add_string_value(builder, manifest->hash);
+
 	json_builder_set_member_name(builder, "images");
 	json_builder_begin_array(builder);
 
@@ -1057,13 +1141,7 @@ static gchar* info_formatter_json_base(RaucManifest *manifest, gboolean pretty)
 		}
 		json_builder_end_array(builder);
 		json_builder_set_member_name(builder, "adaptive");
-		json_builder_begin_array(builder);
-		if (img->adaptive) {
-			for (gchar **m = img->adaptive; *m != NULL; m++) {
-				json_builder_add_string_value(builder, *m);
-			}
-		}
-		json_builder_end_array(builder);
+		strv_to_json_array(builder, img->adaptive);
 		json_builder_end_object(builder);
 		json_builder_end_object(builder);
 	}
@@ -1093,6 +1171,22 @@ static gchar* info_formatter_json_pretty(RaucManifest *manifest)
 	return info_formatter_json_base(manifest, TRUE);
 }
 
+static gchar* info_formatter_json_2(RaucManifest *manifest)
+{
+#if ENABLE_JSON
+	g_autoptr(JsonGenerator) gen = json_generator_new();
+	g_autoptr(GVariant) dict = r_manifest_to_dict(manifest);
+	g_autoptr(JsonNode) root = json_gvariant_serialize(dict);
+
+	json_generator_set_root(gen, root);
+	json_generator_set_pretty(gen, TRUE);
+	return json_generator_to_data(gen, NULL);
+#else
+	g_error("json support is disabled");
+	return NULL;
+#endif
+}
+
 static gboolean info_start(int argc, char **argv)
 {
 	g_autofree gchar *bundlelocation = NULL;
@@ -1105,7 +1199,7 @@ static gboolean info_start(int argc, char **argv)
 	CheckBundleParams check_bundle_params = CHECK_BUNDLE_DEFAULT;
 
 	if (argc < 3) {
-		g_printerr("A file name must be provided\n");
+		g_printerr("A bundle path or URL must be provided\n");
 		r_exit_status = 1;
 		return FALSE;
 	}
@@ -1124,6 +1218,8 @@ static gboolean info_start(int argc, char **argv)
 		formatter = info_formatter_json;
 	} else if (ENABLE_JSON && g_strcmp0(output_format, "json-pretty") == 0) {
 		formatter = info_formatter_json_pretty;
+	} else if (ENABLE_JSON && g_strcmp0(output_format, "json-2") == 0) {
+		formatter = info_formatter_json_2;
 	} else {
 		g_printerr("Unknown output format: '%s'\n", output_format);
 		goto out;
@@ -1259,16 +1355,22 @@ static void r_string_append_slot(GString *text, RaucSlot *slot, RaucStatusPrint
 			g_string_append_printf(text, "\n              description=%s", slot_state->bundle_description);
 		if (slot_state->bundle_build)
 			g_string_append_printf(text, "\n              build=%s", slot_state->bundle_build);
+		if (slot_state->bundle_hash)
+			g_string_append_printf(text, "\n              hash=%s", slot_state->bundle_hash);
 		if (slot_state->checksum.digest && slot_state->checksum.type == G_CHECKSUM_SHA256) {
+			g_autofree gchar* formatted_size = g_format_size_full(slot_state->checksum.size, G_FORMAT_SIZE_LONG_FORMAT);
 			g_string_append_printf(text, "\n          checksum:");
 			g_string_append_printf(text, "\n              sha256=%s", slot_state->checksum.digest);
-			g_string_append_printf(text, "\n              size=%"G_GOFFSET_FORMAT, slot_state->checksum.size);
+			g_string_append_printf(text, "\n              size=%s", formatted_size);
 		}
+		g_string_append_printf(text, "\n          installed:");
 		if (slot_state->installed_timestamp) {
-			g_string_append_printf(text, "\n          installed:");
 			g_string_append_printf(text, "\n              timestamp=%s", slot_state->installed_timestamp);
 			g_string_append_printf(text, "\n              count=%u", slot_state->installed_count);
 		}
+		if (slot_state->installed_txn) {
+			g_string_append_printf(text, "\n              transaction=%s", slot_state->installed_txn);
+		}
 		if (slot_state->activated_timestamp) {
 			g_string_append_printf(text, "\n          activated:");
 			g_string_append_printf(text, "\n              timestamp=%s", slot_state->activated_timestamp);
@@ -1304,14 +1406,14 @@ static gchar* r_status_formatter_readable(RaucStatusPrint *status)
 	slotclasses = r_slot_get_root_classes(status->slots);
 
 	for (gchar **cls = slotclasses; *cls != NULL; cls++) {
-		GList *slots = NULL;
+		g_autoptr(GList) slots = NULL;
 
 		slots = r_slot_get_all_of_class(status->slots, *cls);
 
 		for (GList *l = slots; l != NULL; l = l->next) {
 			RaucSlot *xslot = l->data;
 
-			GList *children = NULL;
+			g_autoptr(GList) children = NULL;
 
 			r_string_append_slot(text, xslot, status);
 
@@ -1391,6 +1493,7 @@ static gchar* r_status_formatter_shell(RaucStatusPrint *status)
 			formatter_shell_append_n(text, "RAUC_SLOT_STATUS_BUNDLE_VERSION", slotcnt, slot_state->bundle_version);
 			formatter_shell_append_n(text, "RAUC_SLOT_STATUS_BUNDLE_DESCRIPTION", slotcnt, slot_state->bundle_description);
 			formatter_shell_append_n(text, "RAUC_SLOT_STATUS_BUNDLE_BUILD", slotcnt, slot_state->bundle_build);
+			formatter_shell_append_n(text, "RAUC_SLOT_STATUS_BUNDLE_HASH", slotcnt, slot_state->bundle_hash);
 			formatter_shell_append_n(text, "RAUC_SLOT_STATUS_CHECKSUM_SHA256", slotcnt, slot_state->checksum.digest);
 			str = g_strdup_printf("%"G_GOFFSET_FORMAT, slot_state->checksum.size);
 			formatter_shell_append_n(text, "RAUC_SLOT_STATUS_CHECKSUM_SIZE", slotcnt, str);
@@ -1483,6 +1586,10 @@ static gchar* r_status_formatter_json(RaucStatusPrint *status, gboolean pretty)
 				json_builder_set_member_name(builder, "build");
 				json_builder_add_string_value(builder, slot_state->bundle_build);
 			}
+			if (slot_state->bundle_hash) {
+				json_builder_set_member_name(builder, "hash");
+				json_builder_add_string_value(builder, slot_state->bundle_hash);
+			}
 			json_builder_end_object(builder);               /* bundle */
 			if (slot_state->checksum.digest && slot_state->checksum.type == G_CHECKSUM_SHA256) {
 				json_builder_set_member_name(builder, "checksum");
@@ -1547,10 +1654,12 @@ static RaucSlotStatus* r_variant_get_slot_state(GVariant *vardict)
 	g_variant_dict_lookup(&dict, "bundle.version", "s", &slot_state->bundle_version);
 	g_variant_dict_lookup(&dict, "bundle.description", "s", &slot_state->bundle_description);
 	g_variant_dict_lookup(&dict, "bundle.build", "s", &slot_state->bundle_build);
+	g_variant_dict_lookup(&dict, "bundle.hash", "s", &slot_state->bundle_hash);
 	g_variant_dict_lookup(&dict, "status", "s", &slot_state->status);
 	if (g_variant_dict_lookup(&dict, "sha256", "s", &slot_state->checksum.digest))
 		slot_state->checksum.type = G_CHECKSUM_SHA256;
 	g_variant_dict_lookup(&dict, "size", "t", &slot_state->checksum.size);
+	g_variant_dict_lookup(&dict, "installed.transaction", "s", &slot_state->installed_txn);
 	g_variant_dict_lookup(&dict, "installed.timestamp", "s", &slot_state->installed_timestamp);
 	g_variant_dict_lookup(&dict, "installed.count", "u", &slot_state->installed_count);
 	g_variant_dict_lookup(&dict, "activated.timestamp", "s", &slot_state->activated_timestamp);
@@ -1894,8 +2003,16 @@ static gboolean status_start(int argc, char **argv)
 G_GNUC_UNUSED
 static gboolean service_start(int argc, char **argv)
 {
+	g_autoptr(GError) ierror = NULL;
+
 	g_debug("service start");
 
+	if (!determine_slot_states(&ierror)) {
+		g_printerr("Failed to determine slot states: %s\n", ierror->message);
+		r_exit_status = 1;
+		return TRUE;
+	}
+
 	r_exit_status = r_service_run() ? 0 : 1;
 
 	return TRUE;
@@ -1909,7 +2026,7 @@ static gboolean mount_start(int argc, char **argv)
 	gboolean res = FALSE;
 
 	if (argc < 3) {
-		g_printerr("A file name must be provided\n");
+		g_printerr("A bundle path or URL must be provided\n");
 		goto out;
 	}
 
@@ -1991,10 +2108,11 @@ typedef struct {
 
 static GOptionEntry entries_install[] = {
 	{"ignore-compatible", '\0', 0, G_OPTION_ARG_NONE, &install_ignore_compatible, "disable compatible check", NULL},
+	{"transaction-id", '\0', 0, G_OPTION_ARG_STRING, &installation_txn, "custom transaction id", "UUID"},
 #if ENABLE_SERVICE == 1
 	{"progress", '\0', 0, G_OPTION_ARG_NONE, &install_progressbar, "show progress bar", NULL},
 #else
-	{"handler-args", '\0', 0, G_OPTION_ARG_STRING, &handler_args, "extra handler arguments", "ARGS"},
+	{"handler-args", '\0', 0, G_OPTION_ARG_STRING, &handler_args, "extra arguments for full custom handler", "ARGS"},
 	{"override-boot-slot", '\0', 0, G_OPTION_ARG_STRING, &bootslot, "override auto-detection of booted slot", "BOOTNAME"},
 #endif
 	{0}
@@ -2030,11 +2148,13 @@ static GOptionEntry entries_convert[] = {
 };
 
 static GOptionEntry entries_extract_signature[] = {
+	{"key", '\0', G_OPTION_FLAG_NOALIAS, G_OPTION_ARG_FILENAME, &keypath, "decryption key file or PKCS#11 URL", "PEMFILE|PKCS11-URL"},
 	{"trust-environment", '\0', 0, G_OPTION_ARG_NONE, &trust_environment, "trust environment and skip bundle access checks", NULL},
 	{0}
 };
 
 static GOptionEntry entries_extract[] = {
+	{"key", '\0', G_OPTION_FLAG_NOALIAS, G_OPTION_ARG_FILENAME, &keypath, "decryption key file or PKCS#11 URL", "PEMFILE|PKCS11-URL"},
 	{"trust-environment", '\0', 0, G_OPTION_ARG_NONE, &trust_environment, "trust environment and skip bundle access checks", NULL},
 	{0}
 };
@@ -2042,6 +2162,7 @@ static GOptionEntry entries_extract[] = {
 static GOptionEntry entries_info[] = {
 	{"no-verify", '\0', 0, G_OPTION_ARG_NONE, &verification_disabled, "disable bundle verification", NULL},
 	{"no-check-time", '\0', 0, G_OPTION_ARG_NONE, &no_check_time, "don't check validity period of certificates against current time", NULL},
+	{"key", '\0', G_OPTION_FLAG_NOALIAS, G_OPTION_ARG_FILENAME, &keypath, "decryption key file or PKCS#11 URL", "PEMFILE|PKCS11-URL"},
 	{"output-format", '\0', 0, G_OPTION_ARG_STRING, &output_format, "output format", "FORMAT"},
 	{"dump-cert", '\0', 0, G_OPTION_ARG_NONE, &info_dumpcert, "dump certificate", NULL},
 	{"dump-recipients", '\0', 0, G_OPTION_ARG_NONE, &info_dumprecipients, "dump recipients", NULL},
@@ -2058,7 +2179,7 @@ static GOptionEntry entries_status[] = {
 };
 
 static GOptionEntry entries_service[] = {
-	{"handler-args", '\0', 0, G_OPTION_ARG_STRING, &handler_args, "extra handler arguments", "ARGS"},
+	{"handler-args", '\0', 0, G_OPTION_ARG_STRING, &handler_args, "extra arguments for full custom handler", "ARGS"},
 	{"override-boot-slot", '\0', 0, G_OPTION_ARG_STRING, &bootslot, "override auto-detection of booted slot", "BOOTNAME"},
 	{0}
 };
@@ -2066,6 +2187,7 @@ static GOptionEntry entries_service[] = {
 static GOptionEntry entries_signing[] = {
 	{"cert", '\0', G_OPTION_FLAG_NOALIAS, G_OPTION_ARG_FILENAME, &certpath, "signing cert file or PKCS#11 URL", "PEMFILE|PKCS11-URL"},
 	{"key", '\0', G_OPTION_FLAG_NOALIAS, G_OPTION_ARG_FILENAME, &keypath, "signing key file or PKCS#11 URL", "PEMFILE|PKCS11-URL"},
+	{"intermediate", '\0', G_OPTION_FLAG_NOALIAS, G_OPTION_ARG_FILENAME_ARRAY, &intermediate, "intermediate CA file or PKCS#11 URL", "PEMFILE|PKCS11-URL"},
 	{0}
 };
 
@@ -2143,7 +2265,7 @@ static void create_option_groups(void)
 static void cmdline_handler(int argc, char **argv)
 {
 	gboolean help = FALSE, debug = FALSE, version = FALSE;
-	gchar *confpath = NULL, *keyring = NULL, **intermediate = NULL, *mount = NULL;
+	gchar *confpath = NULL, *keyring = NULL, *mount = NULL;
 	char *cmdarg = NULL;
 	g_autoptr(GOptionContext) context = NULL;
 	GOptionEntry entries[] = {
@@ -2152,11 +2274,11 @@ static void cmdline_handler(int argc, char **argv)
 		{"cert", '\0', G_OPTION_FLAG_HIDDEN, G_OPTION_ARG_FILENAME, &certpath, "cert file or PKCS#11 URL", "PEMFILE|PKCS11-URL"},
 		{"key", '\0', G_OPTION_FLAG_HIDDEN, G_OPTION_ARG_FILENAME, &keypath, "key file or PKCS#11 URL", "PEMFILE|PKCS11-URL"},
 		{"keyring", '\0', 0, G_OPTION_ARG_FILENAME, &keyring, "keyring file", "PEMFILE"},
-		{"intermediate", '\0', 0, G_OPTION_ARG_FILENAME_ARRAY, &intermediate, "intermediate CA file name", "PEMFILE"},
+		{"intermediate", '\0', G_OPTION_FLAG_HIDDEN, G_OPTION_ARG_FILENAME_ARRAY, &intermediate, "intermediate CA file or PKCS#11 URL", "PEMFILE|PKCS11-URL"},
 		{"mount", '\0', 0, G_OPTION_ARG_FILENAME, &mount, "mount prefix", "PATH"},
 		{"debug", 'd', 0, G_OPTION_ARG_NONE, &debug, "enable debug output", NULL},
 		{"version", '\0', 0, G_OPTION_ARG_NONE, &version, "display version", NULL},
-		{"help", 'h', 0, G_OPTION_ARG_NONE, &help, NULL, NULL},
+		{"help", 'h', 0, G_OPTION_ARG_NONE, &help, "display help and exit", NULL},
 		{0}
 	};
 
@@ -2192,7 +2314,7 @@ static void cmdline_handler(int argc, char **argv)
 		{EXTRACT, "extract", "extract <BUNDLENAME> <OUTPUTDIR>",
 		 "Extract the bundle content",
 		 extract_start, extract_group, R_CONTEXT_CONFIG_MODE_AUTO, FALSE},
-		{INFO, "info", "info <FILE>",
+		{INFO, "info", "info <BUNDLE>",
 		 "Print bundle info",
 		 info_start, info_group, R_CONTEXT_CONFIG_MODE_AUTO, FALSE},
 		{STATUS, "status", "status",
@@ -2226,7 +2348,7 @@ static void cmdline_handler(int argc, char **argv)
 			"Command-specific help:\n"
 			"  rauc <COMMAND> --help\n"
 			"\n"
-			"List of rauc commands:\n"
+			"List of rauc bundle handling commands:\n"
 #if ENABLE_CREATE == 1
 			"  bundle\t\tCreate a bundle\n"
 			"  resign\t\tResign an already signed bundle\n"
@@ -2236,16 +2358,19 @@ static void cmdline_handler(int argc, char **argv)
 			"  extract-signature\tExtract the bundle signature\n"
 #endif
 			"  extract\t\tExtract the bundle content\n"
-			"  install\t\tInstall a bundle\n"
 			"  info\t\t\tShow bundle information\n"
-			"  mount\t\t\tMount a bundle\n"
+			"\n"
+			"List of rauc target commands:\n"
 #if ENABLE_SERVICE == 1
 			"  service\t\tStart RAUC service\n"
 #endif
+			"  install\t\tInstall a bundle\n"
 			"  status\t\tShow status\n"
+			"  mount\t\t\tMount a bundle\n"
 			"  write-slot\t\tWrite image to slot and bypass all update logic\n"
 			"\n"
 			"Environment variables:\n"
+			"  RAUC_KEY_PASSPHRASE Passphrase to use for accessing key files (signing only)\n"
 			"  RAUC_PKCS11_MODULE  Library filename for PKCS#11 module (signing only)\n"
 			"  RAUC_PKCS11_PIN     PIN to use for accessing PKCS#11 keys (signing only)");
 
