@@ -1,3 +1,5 @@
+#include <string.h>
+
 #include "checksum.h"
 #include "config_file.h"
 #include "context.h"
@@ -73,9 +75,13 @@ static gboolean parse_image(GKeyFile *key_file, const gchar *group, RaucImage **
 
 	iimage->filename = key_file_consume_string(key_file, group, "filename", &ierror);
 	/* 'filename' is optional only for 'install' hooks */
-	if (iimage->filename == NULL && !iimage->hooks.install) {
-		g_propagate_error(error, ierror);
-		goto out;
+	if (iimage->filename == NULL) {
+		if (!iimage->hooks.install) {
+			g_propagate_error(error, ierror);
+			goto out;
+		} else {
+			g_clear_error(&ierror);
+		}
 	}
 
 	iimage->adaptive = g_key_file_get_string_list(key_file, group, "adaptive", NULL, NULL);
@@ -94,6 +100,65 @@ out:
 	return res;
 }
 
+static gboolean parse_meta(GKeyFile *key_file, const gchar *group, RaucManifest *raucm, GError **error)
+{
+	g_auto(GStrv) groupsplit = NULL;
+	g_auto(GStrv) keys = NULL;
+	g_autoptr(GHashTable) kvs = NULL;
+	g_autofree gchar *env_section = NULL;
+	GError *ierror = NULL;
+
+	g_return_val_if_fail(key_file != NULL, FALSE);
+	g_return_val_if_fail(group != NULL, FALSE);
+	g_return_val_if_fail(raucm != NULL, FALSE);
+	g_return_val_if_fail(error == NULL || *error == NULL, FALSE);
+
+	groupsplit = g_strsplit(group, ".", 2);
+	if ((g_strv_length(groupsplit) != 2) || strchr(groupsplit[1], '.')) {
+		g_set_error(error, G_KEY_FILE_ERROR, G_KEY_FILE_ERROR_PARSE,
+				"invalid metadata section name '%s' (must contain a single '.')", group);
+		return FALSE;
+	}
+
+	env_section = r_prepare_env_key(groupsplit[1], &ierror);
+	if (!env_section) {
+		g_propagate_prefixed_error(
+				error,
+				ierror,
+				"Invalid metadata section name '%s': ", groupsplit[1]);
+		return FALSE;
+	}
+
+	kvs = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, g_free);
+
+	keys = g_key_file_get_keys(key_file, group, NULL, NULL);
+	for (GStrv key = keys; *key; key++) {
+		g_autofree gchar *value = key_file_consume_string(key_file, group, *key, &ierror);
+		g_autofree gchar *env_key = NULL;
+
+		if (!value) {
+			g_propagate_error(error, ierror);
+			return FALSE;
+		}
+
+		env_key = r_prepare_env_key(*key, &ierror);
+		if (!env_key) {
+			g_propagate_prefixed_error(
+					error,
+					ierror,
+					"Invalid metadata key name '%s': ", *key);
+			return FALSE;
+		}
+
+		g_hash_table_insert(kvs, g_strdup(*key), g_steal_pointer(&value));
+	}
+
+	g_hash_table_insert(raucm->meta, g_strdup(groupsplit[1]), g_steal_pointer(&kvs));
+	g_key_file_remove_group(key_file, group, NULL);
+
+	return TRUE;
+}
+
 /* Parses key_file into RaucManifest structure
  *
  * key_file - input key file
@@ -118,6 +183,9 @@ static gboolean parse_manifest(GKeyFile *key_file, RaucManifest **manifest, GErr
 	g_return_val_if_fail(manifest != NULL && *manifest == NULL, FALSE);
 	g_return_val_if_fail(error == NULL || *error == NULL, FALSE);
 
+	/* initialize empty warnings array */
+	raucm->warnings = g_ptr_array_new_with_free_func(g_free);
+
 	/* parse [update] section */
 	raucm->update_compatible = key_file_consume_string(key_file, "update", "compatible", &ierror);
 	if (!raucm->update_compatible) {
@@ -135,6 +203,18 @@ static gboolean parse_manifest(GKeyFile *key_file, RaucManifest **manifest, GErr
 
 	/* parse [bundle] section */
 	tmp = key_file_consume_string(key_file, "bundle", "format", NULL);
+	if (tmp == NULL) {
+		g_ptr_array_add(raucm->warnings, g_strdup(
+				"WARNING: The manifest does not specify a bundle format, defaulting to 'plain'."));
+		g_ptr_array_add(raucm->warnings, g_strdup(
+				"  We recommend using the 'verity' format instead, if possible."));
+		g_ptr_array_add(raucm->warnings, g_strdup(
+				"  To silence this warning, select the 'plain' format explicitly."));
+		g_ptr_array_add(raucm->warnings, g_strdup(
+				"  See https://rauc.readthedocs.io/en/latest/reference.html#sec-ref-formats for details.'"));
+	} else {
+		raucm->bundle_format_explicit = TRUE;
+	}
 	if (tmp == NULL || g_strcmp0(tmp, "plain") == 0) {
 		raucm->bundle_format = R_MANIFEST_FORMAT_PLAIN;
 	} else if ((g_strcmp0(tmp, "verity") == 0) || (g_strcmp0(tmp, "crypt") == 0)) {
@@ -163,6 +243,10 @@ static gboolean parse_manifest(GKeyFile *key_file, RaucManifest **manifest, GErr
 	/* parse [handler] section */
 	raucm->handler_name = key_file_consume_string(key_file, "handler", "filename", NULL);
 	raucm->handler_args = key_file_consume_string(key_file, "handler", "args", NULL);
+	if (raucm->handler_args && !raucm->handler_name) {
+		g_set_error(error, G_KEY_FILE_ERROR, G_KEY_FILE_ERROR_PARSE,
+				"Setting 'args' requires a full custom handler to be defined under 'filename' in group '[handler]'.");
+	}
 	if (!check_remaining_keys(key_file, "handler", &ierror)) {
 		g_propagate_error(error, ierror);
 		return FALSE;
@@ -189,6 +273,8 @@ static gboolean parse_manifest(GKeyFile *key_file, RaucManifest **manifest, GErr
 	}
 	g_key_file_remove_group(key_file, "hooks", NULL);
 
+	raucm->meta = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, (GDestroyNotify)g_hash_table_destroy);
+
 	groups = g_key_file_get_groups(key_file, &group_count);
 	for (gsize i = 0; i < group_count; i++) {
 		/* parse [image.<slotclass>] sections */
@@ -202,9 +288,12 @@ static gboolean parse_manifest(GKeyFile *key_file, RaucManifest **manifest, GErr
 
 			raucm->images = g_list_append(raucm->images, image);
 		}
-		/* ignore [meta.<label>] sections */
+		/* parse [meta.<label>] sections */
 		if (g_str_has_prefix(groups[i], "meta.")) {
-			g_key_file_remove_group(key_file, groups[i], NULL);
+			if (!parse_meta(key_file, groups[i], raucm, &ierror)) {
+				g_propagate_error(error, ierror);
+				return FALSE;
+			}
 		}
 	}
 
@@ -224,54 +313,62 @@ gboolean load_manifest_mem(GBytes *mem, RaucManifest **manifest, GError **error)
 	g_autoptr(GKeyFile) key_file = NULL;
 	const gchar *data;
 	gsize length;
-	gboolean res = FALSE;
+	g_autofree gchar *manifest_checksum = NULL;
 
 	data = g_bytes_get_data(mem, &length);
 	if (data == NULL) {
 		g_set_error(error, R_MANIFEST_ERROR, R_MANIFEST_ERROR_NO_DATA, "No data available");
-		goto out;
+		return FALSE;
 	}
 
+	manifest_checksum = g_compute_checksum_for_data(G_CHECKSUM_SHA256, (guchar*) data, length);
+
 	key_file = g_key_file_new();
 
-	res = g_key_file_load_from_data(key_file, data, length, G_KEY_FILE_NONE, &ierror);
-	if (!res) {
+	if (!g_key_file_load_from_data(key_file, data, length, G_KEY_FILE_NONE, &ierror)) {
 		g_propagate_error(error, ierror);
-		goto out;
+		return FALSE;
 	}
 
-	res = parse_manifest(key_file, manifest, &ierror);
-	if (!res) {
+	if (!parse_manifest(key_file, manifest, &ierror)) {
 		g_propagate_error(error, ierror);
-		goto out;
+		return FALSE;
 	}
 
-out:
-	return res;
+	(*manifest)->hash = g_steal_pointer(&manifest_checksum);
+
+	return TRUE;
 }
 
 gboolean load_manifest_file(const gchar *filename, RaucManifest **manifest, GError **error)
 {
 	GError *ierror = NULL;
+	g_autofree gchar *data;
+	gsize length;
 	g_autoptr(GKeyFile) key_file = NULL;
-	gboolean res = FALSE;
+	g_autofree gchar *manifest_checksum = NULL;
+
+	if (!g_file_get_contents(filename, &data, &length, &ierror)) {
+		g_propagate_error(error, ierror);
+		return FALSE;
+	}
+	manifest_checksum = g_compute_checksum_for_data(G_CHECKSUM_SHA256, (guchar*) data, length);
 
 	key_file = g_key_file_new();
 
-	res = g_key_file_load_from_file(key_file, filename, G_KEY_FILE_NONE, &ierror);
-	if (!res) {
+	if (!g_key_file_load_from_data(key_file, data, length, G_KEY_FILE_NONE, &ierror)) {
 		g_propagate_error(error, ierror);
-		goto out;
+		return FALSE;
 	}
 
-	res = parse_manifest(key_file, manifest, &ierror);
-	if (!res) {
+	if (!parse_manifest(key_file, manifest, &ierror)) {
 		g_propagate_error(error, ierror);
-		goto out;
+		return FALSE;
 	}
 
-out:
-	return res;
+	(*manifest)->hash = g_steal_pointer(&manifest_checksum);
+
+	return TRUE;
 }
 
 static gboolean check_manifest_common(const RaucManifest *mf, GError **error)
@@ -500,6 +597,9 @@ static GKeyFile *prepare_manifest(const RaucManifest *mf)
 {
 	g_autoptr(GKeyFile) key_file = NULL;
 	GPtrArray *hooks = g_ptr_array_new_full(3, g_free);
+	GHashTableIter iter;
+	GHashTable *meta_kvs;
+	const gchar *meta_group;
 
 	key_file = g_key_file_new();
 
@@ -517,6 +617,8 @@ static GKeyFile *prepare_manifest(const RaucManifest *mf)
 
 	switch (mf->bundle_format) {
 		case R_MANIFEST_FORMAT_PLAIN:
+			if (mf->bundle_format_explicit)
+				g_key_file_set_string(key_file, "bundle", "format", "plain");
 			break;
 		case R_MANIFEST_FORMAT_CRYPT: {
 			if (mf->bundle_crypt_key)
@@ -602,6 +704,20 @@ static GKeyFile *prepare_manifest(const RaucManifest *mf)
 					(const gchar * const *)image->adaptive, g_strv_length(image->adaptive));
 	}
 
+	if (mf->meta) {
+		g_hash_table_iter_init(&iter, mf->meta);
+		while (g_hash_table_iter_next(&iter, (gpointer*)&meta_group, (gpointer*)&meta_kvs)) {
+			GHashTableIter kvs_iter;
+			const gchar *key, *value;
+
+			g_hash_table_iter_init(&kvs_iter, meta_kvs);
+			while (g_hash_table_iter_next(&kvs_iter, (gpointer*)&key, (gpointer*)&value)) {
+				g_autofree gchar *group = g_strdup_printf("meta.%s", meta_group);
+				g_key_file_set_string(key_file, group, key, value);
+			}
+		}
+	}
+
 	return g_steal_pointer(&key_file);
 }
 
@@ -641,6 +757,125 @@ gboolean save_manifest_file(const gchar *filename, const RaucManifest *mf, GErro
 	return res;
 }
 
+GVariant* r_manifest_to_dict(const RaucManifest *manifest)
+{
+	GVariantDict root_dict;
+	GVariantDict grp_dict;
+	g_auto(GVariantBuilder) builder = G_VARIANT_BUILDER_INIT(G_VARIANT_TYPE_ARRAY);
+	GHashTableIter iter;
+	GHashTable *kvs;
+	const gchar *group;
+
+	g_variant_dict_init(&root_dict, NULL);
+
+	if (manifest->hash)
+		g_variant_dict_insert(&root_dict, "manifest-hash", "s", manifest->hash);
+
+	/* construct 'update' dict */
+	g_variant_dict_init(&grp_dict, NULL);
+	if (manifest->update_compatible)
+		g_variant_dict_insert(&grp_dict, "compatible", "s", manifest->update_compatible);
+	if (manifest->update_version)
+		g_variant_dict_insert(&grp_dict, "version", "s", manifest->update_version);
+	if (manifest->update_description)
+		g_variant_dict_insert(&grp_dict, "description", "s", manifest->update_description);
+	if (manifest->update_build)
+		g_variant_dict_insert(&grp_dict, "build", "s", manifest->update_build);
+	g_variant_dict_insert(&root_dict, "update", "v", g_variant_dict_end(&grp_dict));
+
+	/* construct 'bundle' dict */
+	g_variant_dict_init(&grp_dict, NULL);
+	g_variant_dict_insert(&grp_dict, "format", "s", r_manifest_bundle_format_to_str(manifest->bundle_format));
+
+	if (manifest->bundle_verity_hash)
+		g_variant_dict_insert(&grp_dict, "verity-hash", "s", manifest->bundle_verity_hash);
+	if (manifest->bundle_verity_salt)
+		g_variant_dict_insert(&grp_dict, "verity-salt", "s", manifest->bundle_verity_salt);
+	if (manifest->bundle_verity_size)
+		g_variant_dict_insert(&grp_dict, "verity-size", "t", manifest->bundle_verity_size);
+	g_variant_dict_insert(&root_dict, "bundle", "v", g_variant_dict_end(&grp_dict));
+
+	/* construct 'hooks' dict */
+	if (manifest->hook_name) {
+		g_variant_dict_init(&grp_dict, NULL);
+		if (manifest->hook_name)
+			g_variant_dict_insert(&grp_dict, "filename", "s", manifest->hook_name);
+		g_variant_builder_init(&builder, G_VARIANT_TYPE("as"));
+		if (manifest->hooks.install_check)
+			g_variant_builder_add(&builder, "s", "install-check");
+		g_variant_dict_insert(&grp_dict, "hooks", "v", g_variant_builder_end(&builder));
+		g_variant_dict_insert(&root_dict, "hooks", "v", g_variant_dict_end(&grp_dict));
+	}
+
+	/* construct 'handler' dict */
+	if (manifest->handler_name) {
+		g_variant_dict_init(&grp_dict, NULL);
+		if (manifest->handler_name)
+			g_variant_dict_insert(&grp_dict, "filename", "s", manifest->handler_name);
+		if (manifest->handler_args)
+			g_variant_dict_insert(&grp_dict, "args", "s", manifest->handler_args);
+		g_variant_dict_insert(&root_dict, "handler", "v", g_variant_dict_end(&grp_dict));
+	}
+
+	/* construct 'images' array of dicts */
+	g_variant_builder_init(&builder, G_VARIANT_TYPE("aa{sv}"));
+	for (GList *l = manifest->images; l != NULL; l = l->next) {
+		const RaucImage *img = l->data;
+		g_auto(GVariantBuilder) hooks = G_VARIANT_BUILDER_INIT(G_VARIANT_TYPE("as"));
+
+		g_variant_builder_open(&builder, G_VARIANT_TYPE("a{sv}"));
+		g_variant_builder_add(&builder, "{sv}", "slot-class", g_variant_new_string(img->slotclass));
+		if (img->variant)
+			g_variant_builder_add(&builder, "{sv}", "variant", g_variant_new_string(img->variant));
+		if (img->filename)
+			g_variant_builder_add(&builder, "{sv}", "filename", g_variant_new_string(img->filename));
+		if (img->checksum.digest)
+			g_variant_builder_add(&builder, "{sv}", "checksum", g_variant_new_string(img->checksum.digest));
+		if (img->checksum.size)
+			g_variant_builder_add(&builder, "{sv}", "size", g_variant_new_uint64(img->checksum.size));
+
+		if (img->hooks.pre_install)
+			g_variant_builder_add(&hooks, "s", "pre-install");
+		if (img->hooks.install)
+			g_variant_builder_add(&hooks, "s", "install");
+		if (img->hooks.post_install)
+			g_variant_builder_add(&hooks, "s", "post-install");
+		g_variant_builder_add(&builder, "{sv}", "hooks", g_variant_builder_end(&hooks));
+
+		if (img->adaptive)
+			g_variant_builder_add(&builder, "{sv}", "adaptive", g_variant_new_strv((const gchar * const*)(img->adaptive), -1));
+
+		g_variant_builder_close(&builder);
+	}
+	g_variant_dict_insert(&root_dict, "images", "v", g_variant_builder_end(&builder));
+
+	/* construct 'meta' nested dicts */
+	g_variant_builder_init(&builder, G_VARIANT_TYPE("a{sa{ss}}"));
+	g_hash_table_iter_init(&iter, manifest->meta);
+	while (g_hash_table_iter_next(&iter, (gpointer*)&group, (gpointer*)&kvs)) {
+		GHashTableIter kvs_iter;
+		const gchar *key, *value;
+
+		g_variant_builder_open(&builder, G_VARIANT_TYPE("{sa{ss}}"));
+		g_variant_builder_add(&builder, "s", group);
+
+		g_variant_builder_open(&builder, G_VARIANT_TYPE("a{ss}"));
+		g_hash_table_iter_init(&kvs_iter, kvs);
+		while (g_hash_table_iter_next(&kvs_iter, (gpointer*)&key, (gpointer*)&value)) {
+			g_variant_builder_open(&builder, G_VARIANT_TYPE("{ss}"));
+			g_variant_builder_add(&builder, "s", key);
+			g_variant_builder_add(&builder, "s", value);
+			g_variant_builder_close(&builder);
+		}
+		g_variant_builder_close(&builder);
+
+		g_variant_builder_close(&builder);
+	}
+	g_variant_dict_insert(&root_dict, "meta", "v", g_variant_builder_end(&builder));
+
+	return g_variant_dict_end(&root_dict);
+}
+
 void r_free_image(gpointer data)
 {
 	RaucImage *image = (RaucImage*) data;
@@ -667,10 +902,14 @@ void free_manifest(RaucManifest *manifest)
 	g_free(manifest->update_build);
 	g_free(manifest->bundle_verity_hash);
 	g_free(manifest->bundle_verity_salt);
+	g_free(manifest->bundle_crypt_key);
 	g_free(manifest->handler_name);
 	g_free(manifest->handler_args);
 	g_free(manifest->hook_name);
 	g_list_free_full(manifest->images, r_free_image);
+	g_clear_pointer(&manifest->meta, (GDestroyNotify)g_hash_table_destroy);
+	g_free(manifest->hash);
+	g_clear_pointer(&manifest->warnings, (GDestroyNotify)g_ptr_array_unref);
 	g_free(manifest);
 }
 
