@@ -403,6 +403,7 @@ static size_t header_cb(char *buffer, size_t size, size_t nitems, void *userdata
 static void prepare_curl(struct RaucNBDTransfer *xfer)
 {
 	CURLcode code = 0;
+	CURLcode tunnel_code = 0;
 	g_assert_null(xfer->easy);
 
 	xfer->easy = curl_easy_init();
@@ -438,8 +439,16 @@ static void prepare_curl(struct RaucNBDTransfer *xfer)
 	code |= curl_easy_setopt(xfer->easy, CURLOPT_FAILONERROR, 1L);
 	code |= curl_easy_setopt(xfer->easy, CURLOPT_NETRC, CURL_NETRC_OPTIONAL);
 
+	/* use a shorter timeout instead of the 5 minute default */
+	code |= curl_easy_setopt(xfer->easy, CURLOPT_CONNECTTIMEOUT, 20L);
+
 	/* a proxy may be configured using .netrc */
-	code |= curl_easy_setopt(xfer->easy, CURLOPT_HTTPPROXYTUNNEL, 1L);
+	tunnel_code = curl_easy_setopt(xfer->easy, CURLOPT_HTTPPROXYTUNNEL, 1L);
+	if (tunnel_code == CURLE_UNKNOWN_OPTION) {
+		g_debug("no proxy support available in libcurl (failed to set CURLOPT_HTTPPROXYTUNNEL)");
+	} else {
+		code |= tunnel_code;
+	}
 	code |= curl_easy_setopt(xfer->easy, CURLOPT_SUPPRESS_CONNECT_HEADERS, 1L);
 
 	code |= curl_easy_setopt(xfer->easy, CURLOPT_PRIVATE, xfer);
@@ -661,6 +670,9 @@ static gboolean finish_configure(struct RaucNBDContext *ctx, struct RaucNBDTrans
 	g_autoptr(GVariant) v = NULL;
 	guint32 reply_size;
 
+	/* This can only be called after the client has sent a configure command. */
+	g_assert_nonnull(ctx->url);
+
 	if (!xfer->done) { /* retry */
 		res = TRUE;
 		goto out;
@@ -897,7 +909,7 @@ gboolean r_nbd_run_server(gint sock, GError **error)
 				sleep(1);
 				start_request(&ctx, xfer);
 			}
-		};
+		}
 	}
 
 	r_stats_show(ctx.dl_size, NULL);
@@ -1222,7 +1234,7 @@ out:
 	return res;
 }
 
-gboolean r_nbd_read(gint sock, guint8 *data, size_t size, off64_t offset, GError **error)
+gboolean r_nbd_read(gint sock, guint8 *data, size_t size, off_t offset, GError **error)
 {
 	struct nbd_request request = {0};
 	struct nbd_reply reply = {0};
