@@ -51,6 +51,28 @@ static gboolean service_install_cleanup(gpointer data)
 	return G_SOURCE_REMOVE;
 }
 
+/*
+ * Constructs RaucBundleAccessArgs from a GVariant dictionary.
+ */
+static void convert_dict_to_bundle_access_args(
+		GVariantDict *dict,
+		RaucBundleAccessArgs *access_args)
+{
+	g_return_if_fail(dict);
+	g_return_if_fail(access_args);
+
+	if (g_variant_dict_lookup(dict, "tls-cert", "s", &access_args->tls_cert))
+		g_variant_dict_remove(dict, "tls-cert");
+	if (g_variant_dict_lookup(dict, "tls-key", "s", &access_args->tls_key))
+		g_variant_dict_remove(dict, "tls-key");
+	if (g_variant_dict_lookup(dict, "tls-ca", "s", &access_args->tls_ca))
+		g_variant_dict_remove(dict, "tls-ca");
+	if (g_variant_dict_lookup(dict, "tls-no-verify", "b", &access_args->tls_no_verify))
+		g_variant_dict_remove(dict, "tls-no-verify");
+	if (g_variant_dict_lookup(dict, "http-headers", "^as", &access_args->http_headers))
+		g_variant_dict_remove(dict, "http-headers");
+}
+
 static gboolean r_on_handle_install_bundle(
 		RInstaller *interface,
 		GDBusMethodInvocation *invocation,
@@ -79,16 +101,11 @@ static gboolean r_on_handle_install_bundle(
 
 	if (g_variant_dict_lookup(&dict, "ignore-compatible", "b", &args->ignore_compatible))
 		g_variant_dict_remove(&dict, "ignore-compatible");
-	if (g_variant_dict_lookup(&dict, "tls-cert", "s", &args->access_args.tls_cert))
-		g_variant_dict_remove(&dict, "tls-cert");
-	if (g_variant_dict_lookup(&dict, "tls-key", "s", &args->access_args.tls_key))
-		g_variant_dict_remove(&dict, "tls-key");
-	if (g_variant_dict_lookup(&dict, "tls-ca", "s", &args->access_args.tls_ca))
-		g_variant_dict_remove(&dict, "tls-ca");
-	if (g_variant_dict_lookup(&dict, "tls-no-verify", "b", &args->access_args.tls_no_verify))
-		g_variant_dict_remove(&dict, "tls-no-verify");
-	if (g_variant_dict_lookup(&dict, "http-headers", "^as", &args->access_args.http_headers))
-		g_variant_dict_remove(&dict, "http-headers");
+
+	if (g_variant_dict_lookup(&dict, "transaction-id", "s", &args->transaction))
+		g_variant_dict_remove(&dict, "transaction-id");
+
+	convert_dict_to_bundle_access_args(&dict, &args->access_args);
 
 	/* Check for unhandled keys */
 	g_variant_iter_init(&iter, g_variant_dict_end(&dict));
@@ -133,24 +150,44 @@ static gboolean r_on_handle_install(RInstaller *interface,
 	return r_on_handle_install_bundle(interface, invocation, arg_source, NULL);
 }
 
-static gboolean r_on_handle_info(RInstaller *interface,
+static gboolean r_on_handle_inspect_bundle(RInstaller *interface,
 		GDBusMethodInvocation  *invocation,
-		const gchar *arg_bundle)
+		const gchar *arg_bundle, GVariant *arg_args)
 {
+	RaucBundleAccessArgs access_args = {0};
+	g_auto(GVariantDict) dict = G_VARIANT_DICT_INIT(arg_args);
+	g_autoptr(GVariant) remaining = NULL;
+	GVariantIter iter;
+	gchar *key;
 	g_autoptr(RaucManifest) manifest = NULL;
 	g_autoptr(RaucBundle) bundle = NULL;
+	g_autofree gchar *message = NULL;
 	GError *error = NULL;
 	gboolean res = TRUE;
 
 	g_print("bundle: %s\n", arg_bundle);
 
 	res = !r_context_get_busy();
-	if (!res)
-		goto out;
-
-	res = check_bundle(arg_bundle, &bundle, CHECK_BUNDLE_DEFAULT, NULL, &error);
 	if (!res) {
-		g_warning("%s", error->message);
+		message = g_strdup("already processing a different method");
+		goto out;
+	}
+
+	convert_dict_to_bundle_access_args(&dict, &access_args);
+
+	/* Check for unhandled keys */
+	remaining = g_variant_dict_end(&dict);
+	g_variant_iter_init(&iter, remaining);
+	while (g_variant_iter_next(&iter, "{sv}", &key, NULL)) {
+		message = g_strdup_printf("Unsupported key: %s", key);
+		g_free(key);
+		res = FALSE;
+		goto out;
+	}
+
+	res = check_bundle(arg_bundle, &bundle, CHECK_BUNDLE_DEFAULT, &access_args, &error);
+	if (!res) {
+		message = g_strdup(error->message);
 		g_clear_error(&error);
 		goto out;
 	}
@@ -160,29 +197,50 @@ static gboolean r_on_handle_info(RInstaller *interface,
 	} else {
 		res = load_manifest_from_bundle(bundle, &manifest, &error);
 		if (!res) {
-			g_warning("%s\n", error->message);
+			message = g_strdup(error->message);
 			g_clear_error(&error);
 			goto out;
 		}
 	}
 
 out:
-	if (res) {
+	if (!res) {
+		g_dbus_method_invocation_return_error(invocation,
+				G_IO_ERROR,
+				G_IO_ERROR_FAILED_HANDLED,
+				"%s", message);
+		return TRUE;
+	}
+
+	if (arg_args) {
+		GVariant *info_variant;
+
+		info_variant = r_manifest_to_dict(manifest);
+
+		r_installer_complete_inspect_bundle(
+				interface,
+				invocation,
+				info_variant);
+	} else {
+		/* arg_args unset means legacy API */
 		r_installer_complete_info(
 				interface,
 				invocation,
 				manifest->update_compatible,
 				manifest->update_version ? manifest->update_version : "");
-	} else {
-		g_dbus_method_invocation_return_error(invocation,
-				G_IO_ERROR,
-				G_IO_ERROR_FAILED_HANDLED,
-				"rauc info error");
 	}
 
 	return TRUE;
 }
 
+static gboolean r_on_handle_info(RInstaller *interface,
+		GDBusMethodInvocation  *invocation,
+		const gchar *arg_bundle)
+{
+	g_message("Using deprecated 'Info' D-Bus Method (replaced by 'InspectBundle')");
+	return r_on_handle_inspect_bundle(interface, invocation, arg_bundle, NULL);
+}
+
 static gboolean r_on_handle_mark(RInstaller *interface,
 		GDBusMethodInvocation  *invocation,
 		const gchar *arg_state,
@@ -190,7 +248,6 @@ static gboolean r_on_handle_mark(RInstaller *interface,
 {
 	g_autofree gchar *slot_name = NULL;
 	g_autofree gchar *message = NULL;
-	GError *ierror = NULL;
 	gboolean res;
 
 	res = !r_context_get_busy();
@@ -199,13 +256,6 @@ static gboolean r_on_handle_mark(RInstaller *interface,
 		goto out;
 	}
 
-	res = determine_slot_states(&ierror);
-	if (!res) {
-		message = g_strdup_printf("Failed to determine slot states: %s\n", ierror->message);
-		g_clear_error(&ierror);
-		goto out;
-	}
-
 	res = mark_run(arg_state, arg_slot_identifier, &slot_name, &message);
 
 out:
@@ -267,6 +317,9 @@ static GVariant* convert_slot_status_to_dict(RaucSlot *slot)
 	if (slot_state->bundle_build)
 		g_variant_dict_insert(&dict, "bundle.build", "s", slot_state->bundle_build);
 
+	if (slot_state->bundle_hash)
+		g_variant_dict_insert(&dict, "bundle.hash", "s", slot_state->bundle_hash);
+
 	if (slot_state->status)
 		g_variant_dict_insert(&dict, "status", "s", slot_state->status);
 
@@ -275,6 +328,9 @@ static GVariant* convert_slot_status_to_dict(RaucSlot *slot)
 		g_variant_dict_insert(&dict, "size", "t", (guint64) slot_state->checksum.size);
 	}
 
+	if (slot_state->installed_txn)
+		g_variant_dict_insert(&dict, "installed.transaction", "s", slot_state->installed_txn);
+
 	if (slot_state->installed_timestamp) {
 		g_variant_dict_insert(&dict, "installed.timestamp", "s", slot_state->installed_timestamp);
 		g_variant_dict_insert(&dict, "installed.count", "u", slot_state->installed_count);
@@ -306,12 +362,12 @@ static GVariant* create_slotstatus_array(GError **error)
 
 	g_assert_nonnull(r_installer);
 
-	res = determine_slot_states(&ierror);
+	res = update_external_mount_points(&ierror);
 	if (!res) {
 		g_propagate_prefixed_error(
 				error,
 				ierror,
-				"Failed to determine slot states: ");
+				"Failed to update mount points: ");
 		return NULL;
 	}
 
@@ -429,7 +485,7 @@ out:
 	return res;
 }
 
-void set_last_error(gchar *message)
+void set_last_error(const gchar *message)
 {
 	if (r_installer)
 		r_installer_set_last_error(r_installer, message);
@@ -472,6 +528,10 @@ static void r_on_bus_acquired(GDBusConnection *connection,
 			G_CALLBACK(r_on_handle_info),
 			NULL);
 
+	g_signal_connect(r_installer, "handle-inspect-bundle",
+			G_CALLBACK(r_on_handle_inspect_bundle),
+			NULL);
+
 	g_signal_connect(r_installer, "handle-mark",
 			G_CALLBACK(r_on_handle_mark),
 			NULL);
@@ -554,17 +614,10 @@ static gboolean r_on_signal(gpointer user_data)
 
 gboolean r_service_run(void)
 {
-	GError *ierror = NULL;
 	gboolean service_return = TRUE;
 	GBusType bus_type = (!g_strcmp0(g_getenv("DBUS_STARTER_BUS_TYPE"), "session"))
 	                    ? G_BUS_TYPE_SESSION : G_BUS_TYPE_SYSTEM;
 
-	if (!r_context_configure(&ierror)) {
-		g_printerr("Failed to initialize context: %s\n", ierror->message);
-		g_clear_error(&ierror);
-		return FALSE;
-	}
-
 	service_loop = g_main_loop_new(NULL, FALSE);
 	g_unix_signal_add(SIGTERM, r_on_signal, NULL);
 
@@ -584,5 +637,7 @@ gboolean r_service_run(void)
 	g_main_loop_unref(service_loop);
 	service_loop = NULL;
 
+	g_clear_pointer(&r_installer, g_object_unref);
+
 	return service_return;
 }
