@@ -2,6 +2,7 @@
 #include <fcntl.h>
 #include <glib.h>
 #include <glib/gstdio.h>
+#include <gio/gunixinputstream.h>
 #include <gio/gunixoutputstream.h>
 #include <mtd/ubi-user.h>
 #include <string.h>
@@ -30,6 +31,35 @@ GQuark r_update_error_quark(void)
 	return g_quark_from_static_string("r_update_error_quark");
 }
 
+static gboolean check_image_size(int fd, const RaucImage *image, GError **error)
+{
+	GError *ierror = NULL;
+	goffset dev_size;
+
+	g_return_val_if_fail(image, FALSE);
+	g_return_val_if_fail(error == NULL || *error == NULL, FALSE);
+
+	dev_size = get_device_size(fd, &ierror);
+	if (g_error_matches(ierror, R_UTILS_ERROR, R_UTILS_ERROR_INAPPROPRIATE_IOCTL)) {
+		g_clear_error(&ierror);
+		g_info("Slot is not a block device, skipping size check");
+		return TRUE;
+	}
+
+	if (dev_size < image->checksum.size) {
+		if (ierror) {
+			g_propagate_error(error, ierror);
+		} else {
+			g_set_error(error, R_UPDATE_ERROR, R_UPDATE_ERROR_FAILED,
+					"Slot (%"G_GOFFSET_FORMAT " bytes) is too small for image (%"G_GOFFSET_FORMAT " bytes).",
+					dev_size, image->checksum.size);
+		}
+		return FALSE;
+	}
+
+	return TRUE;
+}
+
 /* the fd will only live as long as the returned output stream */
 static GUnixOutputStream* open_slot_device(RaucSlot *slot, int *fd, GError **error)
 {
@@ -60,6 +90,35 @@ static GUnixOutputStream* open_slot_device(RaucSlot *slot, int *fd, GError **err
 	return outstream;
 }
 
+static GUnixInputStream* open_file_as_unix_input_stream(const gchar *filename, int *fd, GError **error)
+{
+	GUnixInputStream *instream = NULL;
+	int fd_out;
+
+	g_return_val_if_fail(filename, NULL);
+	g_return_val_if_fail(error == NULL || *error == NULL, NULL);
+
+	fd_out = g_open(filename, O_RDONLY);
+	if (fd_out < 0) {
+		int err = errno;
+		g_set_error(error, G_IO_ERROR, g_io_error_from_errno(err),
+				"Failed to open file %s: %s", filename, g_strerror(err));
+		return NULL;
+	}
+
+	instream = G_UNIX_INPUT_STREAM(g_unix_input_stream_new(fd_out, TRUE));
+	if (instream == NULL) {
+		g_set_error(error, R_UPDATE_ERROR, R_UPDATE_ERROR_FAILED,
+				"Failed to create stream for file %s", filename);
+		return NULL;
+	}
+
+	if (fd != NULL)
+		*fd = fd_out;
+
+	return instream;
+}
+
 #if ENABLE_EMMC_BOOT_SUPPORT == 1
 static gboolean clear_slot(RaucSlot *slot, GError **error)
 {
@@ -180,6 +239,139 @@ static gboolean ubifs_ioctl(RaucImage *image, int fd, GError **error)
 	return TRUE;
 }
 
+static gssize copy_with_progress(GOutputStream *out_stream, GInputStream *image_stream,
+		goffset image_size, GError **error)
+{
+	GError *ierror = NULL;
+	gsize out_size = 0;
+	goffset sum_size = 0;
+	gint last_percent = -1, percent;
+	gchar buffer[8192];
+	gssize in_size;
+
+	do {
+		gboolean ret;
+
+		in_size = g_input_stream_read(image_stream,
+				buffer, 8192, NULL, &ierror);
+		if (in_size == -1) {
+			g_propagate_error(error, ierror);
+			return -1;
+		}
+		ret = g_output_stream_write_all(out_stream, buffer,
+				in_size, &out_size, NULL, &ierror);
+		if (!ret) {
+			g_propagate_error(error, ierror);
+			return -1;
+		}
+
+		sum_size += out_size;
+
+		percent = sum_size * 100 / image_size;
+		/* emit progress info (but only when in progress context) */
+		if (r_context()->progress && percent != last_percent) {
+			last_percent = percent;
+			r_context_set_step_percentage("copy_image", percent);
+		}
+	} while (out_size);
+
+	return sum_size;
+}
+
+static gboolean splice_with_progress(GUnixInputStream *image_stream,
+		GUnixOutputStream *out_stream, GError **error)
+{
+	int in_fd = g_unix_input_stream_get_fd(image_stream);
+	int out_fd = g_unix_output_stream_get_fd(out_stream);
+	struct stat stat = {};
+	ssize_t out_size = 0;
+	goffset sum_size = 0;
+	gint last_percent = -1, percent;
+
+	if (fstat(in_fd, &stat)) {
+		int err = errno;
+		g_set_error(error,
+				G_FILE_ERROR,
+				g_file_error_from_errno(err),
+				"failed to fstat input: %s", g_strerror(err));
+		return FALSE;
+	}
+
+	do {
+		/* Splice in 1M blocks */
+		out_size = splice(in_fd, NULL, out_fd, NULL, 1UL*1024*1024, 0);
+		if (out_size == -1) {
+			int err = errno;
+			g_set_error(error, G_IO_ERROR, g_io_error_from_errno(err),
+					"%s", g_strerror(err));
+			return FALSE;
+		}
+
+		sum_size += out_size;
+
+		percent = sum_size * 100 / stat.st_size;
+		/* emit progress info (but only when in progress context) */
+		if (r_context()->progress && percent != last_percent) {
+			last_percent = percent;
+			r_context_set_step_percentage("copy_image", percent);
+		}
+	} while (out_size);
+
+	return TRUE;
+}
+
+static gboolean splice_file_to_outstream(const gchar *filename,
+		GUnixOutputStream *out_stream, GError **error)
+{
+	GError *ierror = NULL;
+	g_autoptr(GUnixInputStream) image_stream = NULL;
+
+	g_return_val_if_fail(filename, FALSE);
+	g_return_val_if_fail(out_stream, FALSE);
+	g_return_val_if_fail(error == NULL || *error == NULL, FALSE);
+
+	image_stream = open_file_as_unix_input_stream(filename, NULL, &ierror);
+	if (image_stream == NULL) {
+		g_propagate_prefixed_error(
+				error,
+				ierror,
+				"Failed to open input file for splicing: ");
+		return FALSE;
+	}
+
+	if (!splice_with_progress(image_stream, out_stream, &ierror)) {
+		g_propagate_error(error, ierror);
+		return FALSE;
+	}
+
+	if (g_output_stream_close(G_OUTPUT_STREAM(out_stream), NULL, &ierror) != TRUE) {
+		g_propagate_prefixed_error(
+				error,
+				ierror,
+				"Failed closing output pipe: ");
+		return FALSE;
+	}
+
+	if (g_input_stream_close(G_INPUT_STREAM(image_stream), NULL, &ierror) != TRUE) {
+		g_propagate_prefixed_error(
+				error,
+				ierror,
+				"Failed closing input file: ");
+		return FALSE;
+	}
+
+	return TRUE;
+}
+
+static gboolean splice_file_to_process_stdin(const gchar *filename, GSubprocess *sproc,
+		GError **error)
+{
+	GUnixOutputStream *out_stream = G_UNIX_OUTPUT_STREAM(g_subprocess_get_stdin_pipe(sproc));
+	g_assert_nonnull(out_stream);
+
+	return splice_file_to_outstream(filename, out_stream, error);
+}
+
 static gboolean copy_raw_image(RaucImage *image, GUnixOutputStream *outstream, gsize len_header_last, GError **error)
 {
 	GError *ierror = NULL;
@@ -219,10 +411,7 @@ static gboolean copy_raw_image(RaucImage *image, GUnixOutputStream *outstream, g
 		}
 	}
 
-	size = g_output_stream_splice(G_OUTPUT_STREAM(outstream), instream,
-			G_OUTPUT_STREAM_SPLICE_NONE,
-			NULL,
-			&ierror);
+	size = copy_with_progress(G_OUTPUT_STREAM(outstream), instream, image->checksum.size, &ierror);
 	if (size == -1) {
 		g_propagate_prefixed_error(error, ierror,
 				"Failed splicing data: ");
@@ -370,9 +559,7 @@ static gboolean casync_extract(RaucImage *image, gchar *dest, int out_fd, const
 					"Failed to parse casync extra args: ");
 			goto out;
 		}
-		for (gchar **casync_args = casync_argvp; *casync_args != NULL; casync_args++) {
-			g_ptr_array_add(args, g_strdup(*casync_args));
-		}
+		r_ptr_array_addv(args, casync_argvp, TRUE);
 	}
 	g_ptr_array_add(args, g_strdup(image->filename));
 	g_ptr_array_add(args, g_strdup(out_fd >= 0 ? "-" : dest));
@@ -548,6 +735,13 @@ static gboolean copy_raw_image_to_dev(RaucImage *image, RaucSlot *slot, GError *
 		goto out;
 	}
 
+	/* check size */
+	if (!check_image_size(g_unix_output_stream_get_fd(outstream), image, &ierror)) {
+		res = FALSE;
+		g_propagate_error(error, ierror);
+		goto out;
+	}
+
 	/* copy */
 	g_message("writing data to device %s", slot->device);
 	res = copy_raw_image(image, outstream, 0, &ierror);
@@ -596,6 +790,12 @@ static gboolean copy_block_hash_index_image_to_dev(RaucImage *image, RaucSlot *s
 		res = FALSE;
 		goto out;
 	}
+	if (!check_image_size(tmp->data_fd, image, &ierror)) {
+		g_propagate_error(error, ierror);
+		res = FALSE;
+		goto out;
+	}
+
 	g_ptr_array_add(sources, g_steal_pointer(&tmp));
 
 	/* Open and append seed slot. */
@@ -773,6 +973,8 @@ static gboolean copy_adaptive_image_to_dev(RaucImage *image, RaucSlot *slot, GEr
 	g_return_val_if_fail(error == NULL || *error == NULL, FALSE);
 
 	if (g_strv_contains((const gchar * const*)image->adaptive, "block-hash-index")) {
+		g_info("Selected adaptive update method 'block-hash-index'");
+
 		if (!copy_block_hash_index_image_to_dev(image, slot, &ierror)) {
 			g_propagate_error(error, ierror);
 			return FALSE;
@@ -782,7 +984,7 @@ static gboolean copy_adaptive_image_to_dev(RaucImage *image, RaucSlot *slot, GEr
 
 	temp_string = g_strjoinv(" ", (gchar**) image->adaptive);
 	g_set_error(error, R_UPDATE_ERROR, R_UPDATE_ERROR_UNSUPPORTED_ADAPTIVE_MODE,
-			"No compatible adaptive mode found in '%s'", temp_string);
+			"No compatible adaptive method found in '%s'", temp_string);
 	return FALSE;
 }
 
@@ -803,7 +1005,12 @@ static gboolean write_image_to_dev(RaucImage *image, RaucSlot *slot, GError **er
 	}
 
 	/* Try adaptive mode */
-	if (image->adaptive && slot->data_directory) {
+	if (image->adaptive) {
+		if (!slot->data_directory) {
+			g_message("Ignoring adaptive method since 'data-directory' is not configured");
+			goto raw_copy;
+		}
+
 		if (!copy_adaptive_image_to_dev(image, slot, &ierror)) {
 			if (g_error_matches(ierror, R_UPDATE_ERROR, R_UPDATE_ERROR_UNSUPPORTED_ADAPTIVE_MODE)) {
 				g_info("%s", ierror->message);
@@ -817,6 +1024,7 @@ static gboolean write_image_to_dev(RaucImage *image, RaucSlot *slot, GError **er
 		}
 	}
 
+raw_copy:
 	/* Finally, try a raw copy */
 	if (!copy_raw_image_to_dev(image, slot, &ierror)) {
 		g_propagate_error(error, ierror);
@@ -1092,10 +1300,10 @@ static gboolean nand_write_slot(const gchar *image, const gchar *device, GError
 	g_ptr_array_add(args, g_strdup("--pad"));
 	g_ptr_array_add(args, g_strdup("--quiet"));
 	g_ptr_array_add(args, g_strdup(device));
-	g_ptr_array_add(args, g_strdup(image));
+	g_ptr_array_add(args, g_strdup("-"));
 	g_ptr_array_add(args, NULL);
 
-	sproc = r_subprocess_newv(args, G_SUBPROCESS_FLAGS_NONE, &ierror);
+	sproc = r_subprocess_newv(args, G_SUBPROCESS_FLAGS_STDIN_PIPE, &ierror);
 	if (sproc == NULL) {
 		g_propagate_prefixed_error(
 				error,
@@ -1104,6 +1312,15 @@ static gboolean nand_write_slot(const gchar *image, const gchar *device, GError
 		goto out;
 	}
 
+	res = splice_file_to_process_stdin(image, sproc, &ierror);
+	if (!res) {
+		g_propagate_prefixed_error(
+				error,
+				ierror,
+				"failed to splice data to nandwrite: ");
+		goto out;
+	}
+
 	res = g_subprocess_wait_check(sproc, NULL, &ierror);
 	if (!res) {
 		g_propagate_prefixed_error(
@@ -1117,6 +1334,45 @@ out:
 	return res;
 }
 
+struct suffix_tar_flag {
+	const char *suffix;
+	const char *tar_flag;
+};
+
+static struct suffix_tar_flag suffixes[] = {
+	{".tar",	NULL},
+	{".gz",		"-z"},
+	{".tgz",	"-z"},
+	{".taz",	"-z"},
+	{".Z",		"-Z"},
+	{".taZ",	"-Z"},
+	{".bz2",	"-j"},
+	{".tbz",	"-j"},
+	{".tbz2",	"-j"},
+	{".tz2",	"-j"},
+	{".lz",		"--lzip"},
+	{".lzma",	"--lzma"},
+	{".tlz",	"--lzma"},
+	{".lzo",	"--lzop"},
+	{".xz",		"-J"},
+	{".txz",	"-J"},
+	{".zst",	"--zstd"},
+	{".tzst",	"--zstd"},
+	{NULL,		NULL}
+};
+
+static const gchar *suffix_to_tar_flag(const gchar *filename)
+{
+	g_return_val_if_fail(filename, NULL);
+
+	for (int i = 0; suffixes[i].suffix != NULL; i++) {
+		if (g_str_has_suffix(filename, suffixes[i].suffix))
+			return suffixes[i].tar_flag;
+	}
+
+	return NULL;
+}
+
 static gboolean untar_image(RaucImage *image, gchar *dest, GError **error)
 {
 	g_autoptr(GSubprocess) sproc = NULL;
@@ -1126,13 +1382,14 @@ static gboolean untar_image(RaucImage *image, gchar *dest, GError **error)
 
 	g_ptr_array_add(args, g_strdup("tar"));
 	g_ptr_array_add(args, g_strdup("xf"));
-	g_ptr_array_add(args, g_strdup(image->filename));
+	g_ptr_array_add(args, g_strdup("-"));
 	g_ptr_array_add(args, g_strdup("-C"));
 	g_ptr_array_add(args, g_strdup(dest));
 	g_ptr_array_add(args, g_strdup("--numeric-owner"));
+	g_ptr_array_add(args, g_strdup(suffix_to_tar_flag(image->filename)));
 	g_ptr_array_add(args, NULL);
 
-	sproc = r_subprocess_newv(args, G_SUBPROCESS_FLAGS_NONE, &ierror);
+	sproc = r_subprocess_newv(args, G_SUBPROCESS_FLAGS_STDIN_PIPE, &ierror);
 	if (sproc == NULL) {
 		g_propagate_prefixed_error(
 				error,
@@ -1141,6 +1398,15 @@ static gboolean untar_image(RaucImage *image, gchar *dest, GError **error)
 		goto out;
 	}
 
+	res = splice_file_to_process_stdin(image->filename, sproc, &ierror);
+	if (!res) {
+		g_propagate_prefixed_error(
+				error,
+				ierror,
+				"failed to splice data to tar: ");
+		goto out;
+	}
+
 	res = g_subprocess_wait_check(sproc, NULL, &ierror);
 	if (!res) {
 		g_propagate_prefixed_error(
@@ -1962,6 +2228,7 @@ out:
 	return res;
 }
 
+#if ENABLE_GPT == 1
 G_GNUC_UNUSED
 static gboolean img_to_boot_gpt_switch_handler(RaucImage *image, RaucSlot *dest_slot, const gchar *hook_name, GError **error)
 {
@@ -2051,6 +2318,7 @@ static gboolean img_to_boot_gpt_switch_handler(RaucImage *image, RaucSlot *dest_
 out:
 	return res;
 }
+#endif
 
 #if ENABLE_EMMC_BOOT_SUPPORT == 1
 static gboolean img_to_boot_emmc_handler(RaucImage *image, RaucSlot *dest_slot, const gchar *hook_name, GError **error)
@@ -2271,8 +2539,7 @@ static gboolean check_if_area_is_clear(const gchar *device, guint64 start, gsize
 		if (read_count < 0)
 			goto out;
 
-		for (gint i = 0; i < read_count; i++)
-		{
+		for (gint i = 0; i < read_count; i++) {
 			if ((read_buf[i] != 0x00) && (read_buf[i] != 0xFF)) {
 				*clear = FALSE;
 				break;
@@ -2373,8 +2640,7 @@ static gboolean img_to_boot_raw_fallback_handler(RaucImage *image, RaucSlot *des
 	if (primary_clear)
 		first_part_desc_index++;
 
-	for (gint i = 0; i < 2; i++)
-	{
+	for (gint i = 0; i < 2; i++) {
 		struct part_desc *pd = &part_desc[(first_part_desc_index + i) % 2];
 
 		g_message("Updating %s partition at %"G_GUINT64_FORMAT " on %s", pd->name, pd->partition.start, dest_slot->device);
