@@ -12,6 +12,11 @@
 
 #include "utils.h"
 
+GQuark r_utils_error_quark(void)
+{
+	return g_quark_from_static_string("r_utils_error_quark");
+}
+
 GSubprocess *r_subprocess_new(GSubprocessFlags flags, GError **error, const gchar *argv0, ...)
 {
 	GSubprocess *result;
@@ -222,6 +227,22 @@ gchar * key_file_consume_string(
 	return result;
 }
 
+gboolean value_check_tab_whitespace(const gchar *str, GError **error)
+{
+	g_return_val_if_fail(str != NULL, FALSE);
+	g_return_val_if_fail(error == NULL || *error == NULL, FALSE);
+
+	if (strchr(str, '\t') || strchr(str, ' ')) {
+		g_set_error(error, G_KEY_FILE_ERROR, G_KEY_FILE_ERROR_PARSE,
+				"The value '%s' can not contain tab or whitespace characters",
+				str
+				);
+		return FALSE;
+	}
+
+	return TRUE;
+}
+
 gint key_file_consume_integer(
 		GKeyFile *key_file,
 		const gchar *group_name,
@@ -370,6 +391,10 @@ gboolean r_read_exact(const int fd, guint8 *data, size_t size, GError **error)
 					"Failed to read: %s", g_strerror(err));
 			return FALSE;
 		} else if (ret == 0) { /* end of file */
+			g_set_error(error,
+					G_FILE_ERROR,
+					G_FILE_ERROR_FAILED,
+					"Unexpected end of file");
 			return FALSE;
 		} else if ((size_t)ret <= remaining) {
 			pos += ret;
@@ -426,6 +451,10 @@ gboolean r_pread_exact(const int fd, guint8 *data, size_t size, off_t offset, GE
 					"Failed to read: %s", g_strerror(err));
 			return FALSE;
 		} else if (ret == 0) { /* end of file */
+			g_set_error(error,
+					G_FILE_ERROR,
+					G_FILE_ERROR_FAILED,
+					"Unexpected end of file");
 			return FALSE;
 		} else if ((size_t)ret <= remaining) {
 			pos += ret;
@@ -491,3 +520,104 @@ guint get_sectorsize(gint fd)
 
 	return sector_size;
 }
+
+goffset get_device_size(gint fd, GError **error)
+{
+	guint64 size = 0;
+
+	g_return_val_if_fail(error == NULL || *error == NULL, 0);
+
+	if (ioctl(fd, BLKGETSIZE64, &size) != 0) {
+		int err = errno;
+		if (err == ENOTTY) {
+			g_set_error(error,
+					R_UTILS_ERROR,
+					R_UTILS_ERROR_INAPPROPRIATE_IOCTL,
+					"Failed to get device size: Not a block device");
+		} else {
+			g_set_error(error,
+					G_FILE_ERROR,
+					g_file_error_from_errno(err),
+					"Failed to get device size: %s", g_strerror(err));
+		}
+		return 0;
+	}
+
+	g_assert(size <= G_MAXOFFSET);
+
+	return size;
+}
+
+gchar *r_prepare_env_key(const gchar *key, GError **error)
+{
+	g_autofree gchar *result = NULL;
+	size_t len = 0;
+
+	g_return_val_if_fail(key != NULL, NULL);
+	g_return_val_if_fail(error == NULL || *error == NULL, 0);
+
+	len = strlen(key);
+	result = g_ascii_strup(key, len);
+
+	for (size_t i = 0; i < len; i++) {
+		if (g_ascii_isalnum(result[i]))
+			continue;
+		if (result[i] == '_')
+			continue;
+		if (result[i] == '-') {
+			result[i] = '_';
+			continue;
+		}
+		g_set_error(error,
+				R_UTILS_ERROR,
+				R_UTILS_ERROR_INVALID_ENV_KEY,
+				"Character '%c' is unsuitable for environment variables",
+				key[i]);
+		return NULL;
+	}
+
+	return g_steal_pointer(&result);
+}
+
+gboolean r_update_symlink(const gchar *target, const gchar *name, GError **error)
+{
+	GError *ierror = NULL;
+	g_autofree gchar *old_target = NULL;
+	g_autofree gchar *tmp_name = NULL;
+
+	old_target = g_file_read_link(name, &ierror);
+	if (old_target == NULL) {
+		if (!g_error_matches(ierror, G_FILE_ERROR, G_FILE_ERROR_NOENT)) {
+			g_propagate_error(error, ierror);
+			return FALSE;
+		}
+		g_clear_error(&ierror);
+	} else if (g_strcmp0(old_target, target) == 0) {
+		return TRUE;
+	}
+
+	tmp_name = g_strdup_printf("%s.tmp-link", name);
+
+	if (symlink(target, tmp_name) == -1) {
+		int err = errno;
+		g_set_error(error,
+				G_FILE_ERROR,
+				g_file_error_from_errno(err),
+				"Failed to create symlink: %s", g_strerror(err));
+		return FALSE;
+	}
+
+	if (rename(tmp_name, name) == -1) {
+		int err = errno;
+		g_set_error(error,
+				G_FILE_ERROR,
+				g_file_error_from_errno(err),
+				"Failed to replace symlink: %s", g_strerror(err));
+		/* try to remove the temporary symlink */
+		unlink(tmp_name);
+
+		return FALSE;
+	}
+
+	return TRUE;
+}
