@@ -5,6 +5,7 @@
 #include <glib.h>
 #include <glib/gstdio.h>
 #include <gio/gio.h>
+#include <fcntl.h>
 
 #include <bundle.h>
 #include <manifest.h>
@@ -28,7 +29,7 @@ guint8* random_bytes(gsize size, guint32 seed)
 gchar* write_random_file(const gchar *tmpdir, const gchar *filename,
 		gsize size, const guint32 seed)
 {
-	gchar *pathname;
+	g_autofree gchar *pathname = NULL;
 	g_autofree guint8 *content = NULL;
 
 	pathname = g_build_filename(tmpdir, filename, NULL);
@@ -40,7 +41,7 @@ gchar* write_random_file(const gchar *tmpdir, const gchar *filename,
 		return NULL;
 	}
 
-	return pathname;
+	return g_steal_pointer(&pathname);
 }
 
 /* Helper that writes string to new file in tmpdir/filename, returns entire
@@ -51,7 +52,7 @@ gchar* write_tmp_file(
 		const gchar* content,
 		GError **error)
 {
-	gchar *pathname;
+	g_autofree gchar *pathname = NULL;
 	GError *ierror = NULL;
 
 	pathname = g_build_filename(tmpdir, filename, NULL);
@@ -62,7 +63,7 @@ gchar* write_tmp_file(
 		return NULL;
 	}
 
-	return pathname;
+	return g_steal_pointer(&pathname);
 }
 
 int test_prepare_dummy_file(const gchar *dirname, const gchar *filename,
@@ -113,7 +114,7 @@ int test_prepare_dummy_file(const gchar *dirname, const gchar *filename,
 
 int test_mkdir_relative(const gchar *dirname, const gchar *filename, int mode)
 {
-	gchar *path;
+	g_autofree gchar *path = NULL;
 	int res;
 
 	path = g_strdup_printf("%s/%s", dirname, filename);
@@ -121,13 +122,12 @@ int test_mkdir_relative(const gchar *dirname, const gchar *filename, int mode)
 
 	res = g_mkdir(path, mode);
 
-	g_free(path);
 	return res;
 }
 
 int test_rmdir(const gchar *dirname, const gchar *filename)
 {
-	gchar *path;
+	g_autofree gchar *path = NULL;
 	int res;
 
 	path = g_build_filename(dirname, filename, NULL);
@@ -135,13 +135,12 @@ int test_rmdir(const gchar *dirname, const gchar *filename)
 
 	res = g_rmdir(path);
 
-	g_free(path);
 	return res;
 }
 
 int test_remove(const gchar *dirname, const gchar *filename)
 {
-	gchar *path;
+	g_autofree gchar *path = NULL;
 	int res;
 
 	path = g_build_filename(dirname, filename, NULL);
@@ -149,13 +148,12 @@ int test_remove(const gchar *dirname, const gchar *filename)
 
 	res = g_remove(path);
 
-	g_free(path);
 	return res;
 }
 
 gboolean test_rm_tree(const gchar *dirname, const gchar *filename)
 {
-	gchar *path;
+	g_autofree gchar *path = NULL;
 	gboolean res;
 
 	path = g_build_filename(dirname, filename, NULL);
@@ -163,13 +161,12 @@ gboolean test_rm_tree(const gchar *dirname, const gchar *filename)
 
 	res = rm_tree(path, NULL);
 
-	g_free(path);
 	return res;
 }
 
 int test_prepare_manifest_file(const gchar *dirname, const gchar *filename, const ManifestTestOptions *options)
 {
-	gchar *path = g_build_filename(dirname, filename, NULL);
+	g_autofree gchar *path = g_build_filename(dirname, filename, NULL);
 	RaucManifest *rm = g_new0(RaucManifest, 1);
 	RaucImage *img;
 
@@ -207,9 +204,9 @@ int test_prepare_manifest_file(const gchar *dirname, const gchar *filename, cons
 
 gboolean test_make_filesystem(const gchar *dirname, const gchar *filename)
 {
-	GSubprocess *sub;
+	g_autoptr(GSubprocess) sub = NULL;
 	GError *error = NULL;
-	gchar *path;
+	g_autofree gchar *path = NULL;
 	gboolean res = FALSE;
 
 	path = g_build_filename(dirname, filename, NULL);
@@ -242,13 +239,11 @@ gboolean test_mount(const gchar *src, const gchar *dest)
 	return r_mount_full(src, dest, NULL, NULL, NULL);
 }
 
-
 gboolean test_do_chmod(const gchar *path)
 {
-	GSubprocess *sub;
+	g_autoptr(GSubprocess) sub = NULL;
 	GError *error = NULL;
-	gboolean res = FALSE;
-	GPtrArray *args = g_ptr_array_new_full(10, g_free);
+	g_autoptr(GPtrArray) args = g_ptr_array_new_full(10, g_free);
 
 	g_ptr_array_add(args, g_strdup("chmod"));
 	g_ptr_array_add(args, g_strdup("777"));
@@ -260,24 +255,21 @@ gboolean test_do_chmod(const gchar *path)
 	if (!sub) {
 		g_warning("chmod failed: %s", error->message);
 		g_clear_error(&error);
-		goto out;
+		return FALSE;
 	}
 
-	res = g_subprocess_wait_check(sub, NULL, &error);
-	if (!res) {
+	if (!g_subprocess_wait_check(sub, NULL, &error)) {
 		g_warning("chmod failed: %s", error->message);
 		g_clear_error(&error);
-		goto out;
+		return FALSE;
 	}
 
-out:
-	g_ptr_array_unref(args);
-	return res;
+	return TRUE;
 }
 
 gboolean test_umount(const gchar *dirname, const gchar *mountpoint)
 {
-	gchar *path;
+	g_autofree gchar *path = NULL;
 	gboolean res;
 
 	path = g_build_filename(dirname, mountpoint, NULL);
@@ -296,8 +288,8 @@ gboolean test_copy_file(const gchar *srcprefix, const gchar *srcfile, const gcha
 gboolean test_make_slot_user_writable(const gchar* path, const gchar* file)
 {
 	gboolean res = FALSE;
-	gchar *slotpath;
-	gchar *mountpath;
+	g_autofree gchar *slotpath = NULL;
+	g_autofree gchar *mountpath = NULL;
 
 	slotpath = g_build_filename(path, file, NULL);
 	g_assert_nonnull(slotpath);
@@ -391,3 +383,51 @@ out:
 
 	return size;
 }
+
+void flip_bits_fd(int fd, off_t offset, guint8 mask)
+{
+	guint8 buf;
+	g_assert_cmpint(fd, >, 0);
+	g_assert_cmphex(mask, !=, 0);
+	g_assert(pread(fd, &buf, 1, offset) == 1);
+	buf = buf ^ mask;
+	g_assert(pwrite(fd, &buf, 1, offset) == 1);
+}
+
+void flip_bits_filename(gchar *filename, off_t offset, guint8 mask)
+{
+	int fd = g_open(filename, O_RDWR|O_CLOEXEC, 0);
+	g_assert_cmpint(fd, >, 0);
+	flip_bits_fd(fd, offset, mask);
+	g_assert(fsync(fd) == 0);
+	g_close(fd, NULL);
+}
+
+void replace_strdup(gchar **dst, const gchar *src)
+{
+	g_free(*dst);
+	*dst = g_strdup(src);
+}
+
+void* dup_test_mem(GPtrArray *ptrs, const void *mem, gsize len)
+{
+	void *result = g_memdup(mem, len);
+
+	g_ptr_array_add(ptrs, result);
+
+	return result;
+}
+
+void* dup_test_printf(GPtrArray *ptrs, const gchar *format, ...)
+{
+	gchar *result;
+	va_list args;
+
+	va_start(args, format);
+	result = g_strdup_vprintf(format, args);
+	va_end(args);
+
+	g_ptr_array_add(ptrs, result);
+
+	return result;
+}
