@@ -49,10 +49,9 @@ static void config_file_full_config(ConfigFileFixture *fixture,
 	GError *ierror = NULL;
 	gboolean res;
 	GList *slotlist;
-	RaucConfig *config;
+	g_autoptr(RaucConfig) config = NULL;
 	RaucSlot *slot;
 
-
 	const gchar *cfg_file = "\
 [system]\n\
 compatible=FooCorp Super BarBazzer\n\
@@ -107,7 +106,7 @@ type=ext4\n\
 parent=rootfs.1\n\
 install-same=false\n";
 
-	gchar* pathname = write_tmp_file(fixture->tmpdir, "full_config.conf", cfg_file, NULL);
+	g_autofree gchar* pathname = write_tmp_file(fixture->tmpdir, "full_config.conf", cfg_file, NULL);
 	g_assert_nonnull(pathname);
 
 	res = load_config(pathname, &config, &ierror);
@@ -189,16 +188,14 @@ install-same=false\n";
 	g_list_free(slotlist);
 
 	g_assert(find_config_slot_by_device(config, "/dev/xxx0") == NULL);
-
-	free_config(config);
 }
 
 static void config_file_invalid_items(ConfigFileFixture *fixture,
 		gconstpointer user_data)
 {
-	RaucConfig *config;
+	g_autoptr(RaucConfig) config = NULL;
 	GError *ierror = NULL;
-	gchar* pathname;
+	g_autofree gchar* pathname = NULL;
 
 	const gchar *unknown_group_cfg_file = "\
 [system]\n\
@@ -225,6 +222,8 @@ foo=bar\n\
 	g_assert_cmpstr(ierror->message, ==, "Invalid group '[unknown]'");
 	g_clear_error(&ierror);
 
+	g_free(pathname);
+	g_clear_pointer(&config, free_config);
 
 	pathname = write_tmp_file(fixture->tmpdir, "unknown_key.conf", unknown_key_cfg_file, NULL);
 	g_assert_nonnull(pathname);
@@ -238,9 +237,9 @@ foo=bar\n\
 static void config_file_bootloaders(ConfigFileFixture *fixture,
 		gconstpointer user_data)
 {
-	RaucConfig *config;
+	g_autoptr(RaucConfig) config = NULL;
 	GError *ierror = NULL;
-	gchar* pathname;
+	g_autofree gchar* pathname = NULL;
 
 	const gchar *boot_inval_cfg_file = "\
 [system]\n\
@@ -260,6 +259,8 @@ mountprefix=/mnt/myrauc/\n";
 	g_assert_cmpstr(ierror->message, ==, "Unsupported bootloader 'superloader2000' selected in system config");
 	g_clear_error(&ierror);
 
+	g_free(pathname);
+	g_clear_pointer(&config, free_config);
 
 	pathname = write_tmp_file(fixture->tmpdir, "invalid_bootloader.conf", boot_missing_cfg_file, NULL);
 	g_assert_nonnull(pathname);
@@ -269,12 +270,86 @@ mountprefix=/mnt/myrauc/\n";
 	g_clear_error(&ierror);
 }
 
+static void config_file_boot_attempts(ConfigFileFixture *fixture,
+		gconstpointer user_data)
+{
+	g_autoptr(RaucConfig) config = NULL;
+	GError *ierror = NULL;
+	g_autofree gchar* pathname = NULL;
+
+	const gchar *valid_boot_attempts_cfg_file = "\
+[system]\n\
+compatible=FooCorp Super BarBazzer\n\
+bootloader=uboot\n\
+boot-attempts=5\n\
+boot-attempts-primary=10\n\
+mountprefix=/mnt/myrauc/\n";
+	const gchar *invalid_boot_attempts_cfg_file = "\
+[system]\n\
+compatible=FooCorp Super BarBazzer\n\
+bootloader=uboot\n\
+boot-attempts=urks\n\
+boot-attempts-primary=10\n\
+mountprefix=/mnt/myrauc/\n";
+	const gchar *boot_attempts_invalid_bootloader_cfg_file = "\
+[system]\n\
+compatible=FooCorp Super BarBazzer\n\
+bootloader=grub\n\
+boot-attempts=5\n\
+boot-attempts-primary=10\n\
+mountprefix=/mnt/myrauc/\n";
+	const gchar *negative_boot_attempts_cfg_file = "\
+[system]\n\
+compatible=FooCorp Super BarBazzer\n\
+bootloader=uboot\n\
+boot-attempts=5\n\
+boot-attempts-primary=-10\n\
+mountprefix=/mnt/myrauc/\n";
+
+
+	pathname = write_tmp_file(fixture->tmpdir, "valid_bootloader.conf", valid_boot_attempts_cfg_file, NULL);
+	g_assert_nonnull(pathname);
+
+	g_assert_true(load_config(pathname, &config, &ierror));
+	g_assert_no_error(ierror);
+
+	g_free(pathname);
+	g_clear_pointer(&config, free_config);
+
+	pathname = write_tmp_file(fixture->tmpdir, "invalid_bootloader.conf", invalid_boot_attempts_cfg_file, NULL);
+	g_assert_nonnull(pathname);
+
+	g_assert_false(load_config(pathname, &config, &ierror));
+	g_assert_error(ierror, G_KEY_FILE_ERROR, G_KEY_FILE_ERROR_INVALID_VALUE);
+	g_clear_error(&ierror);
+
+	g_free(pathname);
+	g_clear_pointer(&config, free_config);
+
+	pathname = write_tmp_file(fixture->tmpdir, "invalid_bootloader.conf", boot_attempts_invalid_bootloader_cfg_file, NULL);
+	g_assert_nonnull(pathname);
+
+	g_assert_false(load_config(pathname, &config, &ierror));
+	g_assert_cmpstr(ierror->message, ==, "Configuring boot attempts is valid for uboot or barebox only (not for grub)");
+	g_clear_error(&ierror);
+
+	g_free(pathname);
+	g_clear_pointer(&config, free_config);
+
+	pathname = write_tmp_file(fixture->tmpdir, "invalid_bootloader.conf", negative_boot_attempts_cfg_file, NULL);
+	g_assert_nonnull(pathname);
+
+	g_assert_false(load_config(pathname, &config, &ierror));
+	g_assert_cmpstr(ierror->message, ==, "Value for \"boot-attempts-primary\" must not be negative");
+	g_clear_error(&ierror);
+}
+
 static void config_file_slots_invalid_type(ConfigFileFixture *fixture,
 		gconstpointer user_data)
 {
-	RaucConfig *config;
+	g_autoptr(RaucConfig) config = NULL;
 	GError *ierror = NULL;
-	gchar* pathname;
+	g_autofree gchar* pathname = NULL;
 
 	const gchar *invalid_slot_type = "\
 [system]\n\
@@ -299,9 +374,9 @@ type=oups\n\
 static void config_file_invalid_parent(ConfigFileFixture *fixture,
 		gconstpointer user_data)
 {
-	RaucConfig *config;
+	g_autoptr(RaucConfig) config = NULL;
 	GError *ierror = NULL;
-	gchar* pathname;
+	g_autofree gchar* pathname = NULL;
 
 	const gchar *nonexisting_parent = "\
 [system]\n\
@@ -325,11 +400,11 @@ parent=invalid\n\
 
 static void config_file_parent_has_parent(ConfigFileFixture *fixture, gconstpointer user_data)
 {
-	RaucConfig *config;
+	g_autoptr(RaucConfig) config = NULL;
 	RaucSlot *parentslot;
 	RaucSlot *childslot;
 	RaucSlot *grandchildslot;
-	gchar* pathname;
+	g_autofree gchar* pathname = NULL;
 
 	const gchar *contents = "\
 [system]\n\
@@ -362,9 +437,9 @@ parent=child.0\n";
 
 static void config_file_parent_loop(ConfigFileFixture *fixture, gconstpointer user_data)
 {
-	RaucConfig *config;
+	g_autoptr(RaucConfig) config = NULL;
 	GError *ierror = NULL;
-	gchar* pathname;
+	g_autofree gchar* pathname = NULL;
 
 	const gchar *contents = "\
 [system]\n\
@@ -389,9 +464,9 @@ parent=rootfs.0\n";
 
 static void config_file_bootname_set_on_child(ConfigFileFixture *fixture, gconstpointer user_data)
 {
-	RaucConfig *config;
+	g_autoptr(RaucConfig) config = NULL;
 	GError *ierror = NULL;
-	gchar* pathname;
+	g_autofree gchar* pathname = NULL;
 
 	const gchar *contents = "\
 [system]\n\
@@ -418,9 +493,9 @@ bootname=slotchild0\n";
 
 static void config_file_duplicate_bootname(ConfigFileFixture *fixture, gconstpointer user_data)
 {
-	RaucConfig *config;
+	g_autoptr(RaucConfig) config = NULL;
 	GError *ierror = NULL;
-	gchar* pathname;
+	g_autofree gchar* pathname = NULL;
 
 	const gchar *contents = "\
 [system]\n\
@@ -446,9 +521,9 @@ bootname=theslot\n";
 
 static void config_file_typo(ConfigFileFixture *fixture, const gchar *cfg_file)
 {
-	RaucConfig *config;
+	g_autoptr(RaucConfig) config = NULL;
 	GError *ierror = NULL;
-	gchar* pathname;
+	g_autofree gchar* pathname = NULL;
 
 	pathname = write_tmp_file(fixture->tmpdir, "typo.conf", cfg_file, NULL);
 	g_assert_nonnull(pathname);
@@ -571,13 +646,37 @@ mountprefix=/mnt/myrauc/\n\
 activate-installed=typo\n");
 }
 
+static void config_file_bootname_tab(ConfigFileFixture *fixture, gconstpointer user_data)
+{
+	g_autoptr(RaucConfig) config = NULL;
+	GError *ierror = NULL;
+	g_autofree gchar* pathname = NULL;
+
+	const gchar *contents = "\
+[system]\n\
+compatible=FooCorp Super BarBazzer\n\
+bootloader=barebox\n\
+\n\
+[slot.rootfs.0]\n\
+device=/dev/null\n\
+bootname=the\tslot\n";
+
+	pathname = write_tmp_file(fixture->tmpdir, "bootname_tab.conf", contents, NULL);
+	g_assert_nonnull(pathname);
+
+	g_assert_false(load_config(pathname, &config, &ierror));
+	g_assert_error(ierror, G_KEY_FILE_ERROR, G_KEY_FILE_ERROR_PARSE);
+	g_assert_cmpstr(ierror->message, ==, "Invalid bootname for slot rootfs.0: The value 'the\tslot' can not contain tab or whitespace characters");
+	g_clear_error(&ierror);
+}
+
 static void config_file_no_max_bundle_download_size(ConfigFileFixture *fixture,
 		gconstpointer user_data)
 {
-	RaucConfig *config;
+	g_autoptr(RaucConfig) config = NULL;
 	GError *ierror = NULL;
 	gboolean res;
-	gchar* pathname;
+	g_autofree gchar* pathname = NULL;
 
 	const gchar *cfg_file = "\
 [system]\n\
@@ -592,16 +691,14 @@ bootloader=barebox\n";
 	g_assert_true(res);
 	g_assert_nonnull(config);
 	g_assert_cmpuint(config->max_bundle_download_size, ==, DEFAULT_MAX_BUNDLE_DOWNLOAD_SIZE);
-
-	free_config(config);
 }
 
 static void config_file_zero_max_bundle_download_size(ConfigFileFixture *fixture,
 		gconstpointer user_data)
 {
-	RaucConfig *config;
+	g_autoptr(RaucConfig) config = NULL;
 	GError *ierror = NULL;
-	gchar* pathname;
+	g_autofree gchar* pathname = NULL;
 
 	const gchar *cfg_file = "\
 [system]\n\
@@ -632,10 +729,10 @@ max-bundle-download-size=no-uint64\n");
 static void config_file_activate_installed_set_to_true(ConfigFileFixture *fixture,
 		gconstpointer user_data)
 {
-	RaucConfig *config;
+	g_autoptr(RaucConfig) config = NULL;
 	GError *ierror = NULL;
 	gboolean res;
-	gchar* pathname;
+	g_autofree gchar* pathname = NULL;
 
 	const gchar *cfg_file = "\
 [system]\n\
@@ -653,17 +750,15 @@ activate-installed=true\n";
 	g_assert_true(res);
 	g_assert_nonnull(config);
 	g_assert_true(config->activate_installed);
-
-	free_config(config);
 }
 
 static void config_file_activate_installed_set_to_false(ConfigFileFixture *fixture,
 		gconstpointer user_data)
 {
-	RaucConfig *config;
+	g_autoptr(RaucConfig) config = NULL;
 	GError *ierror = NULL;
 	gboolean res;
-	gchar* pathname;
+	g_autofree gchar* pathname = NULL;
 
 	const gchar *cfg_file = "\
 [system]\n\
@@ -681,17 +776,15 @@ activate-installed=false\n";
 	g_assert_true(res);
 	g_assert_nonnull(config);
 	g_assert_false(config->activate_installed);
-
-	free_config(config);
 }
 
 static void config_file_system_variant(ConfigFileFixture *fixture,
 		gconstpointer user_data)
 {
-	RaucConfig *config;
-	GError *ierror = NULL;
+	g_autoptr(RaucConfig) config = NULL;
+	g_autoptr(GError) ierror = NULL;
 	gboolean res;
-	gchar* pathname;
+	g_autofree gchar* pathname = NULL;
 
 	const gchar *cfg_file_no_variant = "\
 [system]\n\
@@ -739,7 +832,7 @@ variant-name=xxx";
 	g_assert_nonnull(config);
 	g_assert_null(config->system_variant);
 
-	free_config(config);
+	g_clear_pointer(&config, free_config);
 
 	pathname = write_tmp_file(fixture->tmpdir, "name_variant.conf", cfg_file_name_variant, NULL);
 	g_assert_nonnull(pathname);
@@ -753,7 +846,7 @@ variant-name=xxx";
 	g_assert(config->system_variant_type == R_CONFIG_SYS_VARIANT_NAME);
 	g_assert_cmpstr(config->system_variant, ==, "variant-name");
 
-	free_config(config);
+	g_clear_pointer(&config, free_config);
 
 	pathname = write_tmp_file(fixture->tmpdir, "dtb_variant.conf", cfg_file_dtb_variant, NULL);
 	g_assert_nonnull(pathname);
@@ -767,7 +860,7 @@ variant-name=xxx";
 	g_assert(config->system_variant_type == R_CONFIG_SYS_VARIANT_DTB);
 	g_assert_null(config->system_variant);
 
-	free_config(config);
+	g_clear_pointer(&config, free_config);
 
 	pathname = write_tmp_file(fixture->tmpdir, "file_variant.conf", cfg_file_file_variant, NULL);
 	g_assert_nonnull(pathname);
@@ -781,11 +874,12 @@ variant-name=xxx";
 	g_assert(config->system_variant_type == R_CONFIG_SYS_VARIANT_FILE);
 	g_assert_cmpstr(config->system_variant, ==, "/path/to/file");
 
+	g_clear_pointer(&config, free_config);
+
 	pathname = write_tmp_file(fixture->tmpdir, "conflict_variant.conf", cfg_file_conflicting_variants, NULL);
 	g_assert_nonnull(pathname);
 
 	g_assert_false(load_config(pathname, &config, &ierror));
-	g_free(pathname);
 	g_assert_error(ierror, R_CONFIG_ERROR, R_CONFIG_ERROR_INVALID_FORMAT);
 	g_assert_null(config);
 }
@@ -793,7 +887,7 @@ variant-name=xxx";
 static void config_file_no_extra_mount_opts(ConfigFileFixture *fixture,
 		gconstpointer user_data)
 {
-	RaucConfig *config;
+	g_autoptr(RaucConfig) config = NULL;
 	GError *ierror = NULL;
 	gboolean res;
 	g_autofree gchar* pathname = NULL;
@@ -822,15 +916,13 @@ device=/dev/null\n";
 	slot = g_hash_table_lookup(config->slots, "rootfs.0");
 	g_assert_nonnull(slot);
 	g_assert_cmpstr(slot->extra_mount_opts, ==, NULL);
-
-	free_config(config);
 }
 
 
 static void config_file_extra_mount_opts(ConfigFileFixture *fixture,
 		gconstpointer user_data)
 {
-	RaucConfig *config;
+	g_autoptr(RaucConfig) config = NULL;
 	GError *ierror = NULL;
 	gboolean res;
 	g_autofree gchar* pathname = NULL;
@@ -859,17 +951,15 @@ extra-mount-opts=ro,noatime\n";
 	slot = g_hash_table_lookup(config->slots, "rootfs.0");
 	g_assert_nonnull(slot);
 	g_assert_cmpstr(slot->extra_mount_opts, ==, "ro,noatime");
-
-	free_config(config);
 }
 
 static void config_file_statusfile_missing(ConfigFileFixture *fixture,
 		gconstpointer user_data)
 {
-	RaucConfig *config;
+	g_autoptr(RaucConfig) config = NULL;
 	GError *ierror = NULL;
 	gboolean res;
-	gchar* pathname;
+	g_autofree gchar* pathname = NULL;
 
 	const gchar *cfg_file = "\
 [system]\n\
@@ -887,8 +977,6 @@ mountprefix=/mnt/myrauc/\n";
 	g_assert_nonnull(config);
 	g_assert_nonnull(config->statusfile_path);
 	g_assert_cmpstr(config->statusfile_path, ==, "per-slot");
-
-	free_config(config);
 }
 
 
@@ -934,13 +1022,6 @@ static void config_file_test_write_slot_status(void)
 	r_slot_free_status(ss);
 }
 
-static void config_file_system_serial(ConfigFileFixture *fixture,
-		gconstpointer user_data)
-{
-	g_assert_nonnull(r_context()->system_serial);
-	g_assert_cmpstr(r_context()->system_serial, ==, "1234");
-}
-
 static void config_file_test_global_slot_status(ConfigFileFixture *fixture,
 		gconstpointer user_data)
 {
@@ -997,10 +1078,10 @@ static void config_file_test_global_slot_status(ConfigFileFixture *fixture,
 static void config_file_keyring_checks(ConfigFileFixture *fixture,
 		gconstpointer user_data)
 {
-	RaucConfig *config;
+	g_autoptr(RaucConfig) config = NULL;
 	GError *ierror = NULL;
 	gboolean res = FALSE;
-	gchar* pathname;
+	g_autofree gchar* pathname;
 
 	const gchar *simple_cfg_file = "\
 [system]\n\
@@ -1029,7 +1110,8 @@ check-purpose=codesign\n";
 	g_assert_false(config->keyring_check_crl);
 	g_assert_cmpstr(config->keyring_check_purpose, ==, NULL);
 
-	free_config(config);
+	g_free(pathname);
+	g_clear_pointer(&config, free_config);
 
 	pathname = write_tmp_file(fixture->tmpdir, "checking.conf", checking_cfg_file, NULL);
 	g_assert_nonnull(pathname);
@@ -1041,17 +1123,15 @@ check-purpose=codesign\n";
 	g_assert_true(config->keyring_allow_partial_chain);
 	g_assert_true(config->keyring_check_crl);
 	g_assert_cmpstr(config->keyring_check_purpose, ==, "codesign");
-
-	free_config(config);
 }
 
 static void config_file_bundle_formats(ConfigFileFixture *fixture,
 		gconstpointer user_data)
 {
-	RaucConfig *config;
+	g_autoptr(RaucConfig) config = NULL;
 	g_autoptr(GError) ierror = NULL;
 	gboolean res = FALSE;
-	gchar* pathname;
+	g_autofree gchar* pathname;
 
 	const gchar *default_cfg_file = "\
 [system]\n\
@@ -1082,7 +1162,8 @@ bundle-formats=-plain -verity -crypt\n";
 	g_assert_nonnull(config);
 	g_assert_cmphex(config->bundle_formats_mask, ==, 0x7);
 
-	free_config(config);
+	g_free(pathname);
+	g_clear_pointer(&config, free_config);
 
 	pathname = write_tmp_file(fixture->tmpdir, "set.conf", set_cfg_file, NULL);
 	g_assert_nonnull(pathname);
@@ -1093,7 +1174,8 @@ bundle-formats=-plain -verity -crypt\n";
 	g_assert_nonnull(config);
 	g_assert_cmphex(config->bundle_formats_mask, ==, 0x1);
 
-	free_config(config);
+	g_free(pathname);
+	g_clear_pointer(&config, free_config);
 
 	pathname = write_tmp_file(fixture->tmpdir, "modify.conf", modify_cfg_file, NULL);
 	g_assert_nonnull(pathname);
@@ -1104,7 +1186,8 @@ bundle-formats=-plain -verity -crypt\n";
 	g_assert_nonnull(config);
 	g_assert_cmphex(config->bundle_formats_mask, ==, 0x6);
 
-	free_config(config);
+	g_free(pathname);
+	g_clear_pointer(&config, free_config);
 
 	pathname = write_tmp_file(fixture->tmpdir, "none.conf", none_cfg_file, NULL);
 	g_assert_nonnull(pathname);
@@ -1173,7 +1256,7 @@ static void config_file_test_parse_bundle_formats(void)
 static void config_file_use_desync_set_to_true(ConfigFileFixture *fixture,
 		gconstpointer user_data)
 {
-	RaucConfig *config;
+	g_autoptr(RaucConfig) config = NULL;
 	g_autoptr(GError) ierror = NULL;
 	gboolean res;
 	g_autofree gchar* pathname = NULL;
@@ -1200,8 +1283,6 @@ use-desync=true";
 	g_assert_cmpstr(config->tmp_path, ==, "/tmp/");
 	g_assert_cmpstr(config->casync_install_args, ==, "--seed /my/path/additional_seed.caibx");
 	g_assert_true(config->use_desync);
-
-	free_config(config);
 }
 
 int main(int argc, char *argv[])
@@ -1219,6 +1300,9 @@ int main(int argc, char *argv[])
 	g_test_add("/config-file/bootloaders", ConfigFileFixture, NULL,
 			config_file_fixture_set_up, config_file_bootloaders,
 			config_file_fixture_tear_down);
+	g_test_add("/config-file/boot_attempts", ConfigFileFixture, NULL,
+			config_file_fixture_set_up, config_file_boot_attempts,
+			config_file_fixture_tear_down);
 	g_test_add("/config-file/slots/invalid_type", ConfigFileFixture, NULL,
 			config_file_fixture_set_up, config_file_slots_invalid_type,
 			config_file_fixture_tear_down);
@@ -1258,6 +1342,9 @@ int main(int argc, char *argv[])
 	g_test_add("/config-file/typo-in-boolean-activate-installed-key", ConfigFileFixture, NULL,
 			config_file_fixture_set_up, config_file_typo_in_boolean_activate_installed_key,
 			config_file_fixture_tear_down);
+	g_test_add("/config-file/bootname-tab", ConfigFileFixture, NULL,
+			config_file_fixture_set_up, config_file_bootname_tab,
+			config_file_fixture_tear_down);
 	g_test_add("/config-file/no-max-bundle-download-size", ConfigFileFixture, NULL,
 			config_file_fixture_set_up, config_file_no_max_bundle_download_size,
 			config_file_fixture_tear_down);
@@ -1284,9 +1371,6 @@ int main(int argc, char *argv[])
 			config_file_fixture_tear_down);
 	g_test_add_func("/config-file/read-slot-status", config_file_test_read_slot_status);
 	g_test_add_func("/config-file/write-read-slot-status", config_file_test_write_slot_status);
-	g_test_add("/config-file/system-serial", ConfigFileFixture, NULL,
-			config_file_fixture_set_up, config_file_system_serial,
-			config_file_fixture_tear_down);
 	g_test_add("/config-file/statusfile-missing", ConfigFileFixture, NULL,
 			config_file_fixture_set_up, config_file_statusfile_missing,
 			config_file_fixture_tear_down);
