@@ -55,6 +55,8 @@ static void test_load_manifest(void)
 	g_assert_no_error(error);
 	g_assert_true(res);
 	manifest_check_common(rm);
+	g_assert_false(rm->bundle_format_explicit);
+	g_assert_cmpuint(rm->warnings->len, ==, 4);
 
 	g_clear_pointer(&rm, free_manifest);
 	g_assert_null(rm);
@@ -80,6 +82,8 @@ static void test_load_manifest(void)
 
 static void check_manifest_contents(const RaucManifest *rm)
 {
+	GHashTable *kvs = NULL;
+
 	g_assert_nonnull(rm);
 	g_assert_cmpstr(rm->update_compatible, ==, "BarCorp FooBazzer");
 	g_assert_cmpstr(rm->update_version, ==, "2011.03-1");
@@ -102,6 +106,17 @@ static void check_manifest_contents(const RaucManifest *rm)
 	g_assert_true(((RaucImage*)g_list_nth_data(rm->images, 0))->hooks.post_install);
 
 	g_assert_true(rm->hooks.install_check);
+
+	g_assert_cmpuint(g_hash_table_size(rm->meta), ==, 2);
+
+	kvs = g_hash_table_lookup(rm->meta, "foocorp");
+	g_assert_nonnull(kvs);
+	g_assert_cmpstr((g_hash_table_lookup(kvs, "release-type")), ==, "beta");
+	g_assert_cmpstr((g_hash_table_lookup(kvs, "release-notes")), ==, "https://foocorp.example/releases/release-notes-2015.04-1.rst");
+
+	kvs = g_hash_table_lookup(rm->meta, "example");
+	g_assert_nonnull(kvs);
+	g_assert_cmpstr((g_hash_table_lookup(kvs, "counter")), ==, "42");
 }
 
 /* Test manifest/save_load:
@@ -117,10 +132,12 @@ static void test_save_load_manifest(void)
 	gboolean res = FALSE;
 	RaucManifest *rm = g_new0(RaucManifest, 1);
 	RaucImage *new_image;
+	GHashTable *kvs = NULL;
 	GBytes *mem = NULL;
 
 	rm->update_compatible = g_strdup("BarCorp FooBazzer");
 	rm->update_version = g_strdup("2011.03-1");
+	rm->bundle_format_explicit = TRUE;
 	rm->handler_name = g_strdup("myhandler.sh");
 	rm->handler_args = g_strdup("--foo");
 	rm->hook_name = g_strdup("hook.sh");
@@ -142,7 +159,7 @@ static void test_save_load_manifest(void)
 	new_image->variant = g_strdup("variant-1");
 	new_image->checksum.type = G_CHECKSUM_SHA256;
 	new_image->checksum.digest = g_strdup("768c36e72bedd35dac67c39b6145f97ef174179f5903a31c4c03abc0eb5d954c");
-	new_image->filename = g_strdup("myrootimg_vareiant1.ext4");
+	new_image->filename = g_strdup("myrootimg_variant1.ext4");
 	new_image->hooks.pre_install = TRUE;
 	new_image->hooks.post_install = TRUE;
 	new_image->adaptive = g_strsplit("invalid-method;another-invalid-method", ";", 0);
@@ -158,6 +175,19 @@ static void test_save_load_manifest(void)
 
 	g_assert_cmpuint(g_list_length(rm->images), ==, 3);
 
+	rm->meta = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, (GDestroyNotify)g_hash_table_destroy);
+
+	kvs = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, g_free);
+	g_hash_table_insert(kvs, g_strdup("release-type"), g_strdup("beta"));
+	g_hash_table_insert(kvs, g_strdup("release-notes"), g_strdup("https://foocorp.example/releases/release-notes-2015.04-1.rst"));
+	g_hash_table_insert(rm->meta, g_strdup("foocorp"), kvs);
+
+	kvs = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, g_free);
+	g_hash_table_insert(kvs, g_strdup("counter"), g_strdup("42"));
+	g_hash_table_insert(rm->meta, g_strdup("example"), kvs);
+
+	g_assert_cmpuint(g_hash_table_size(rm->meta), ==, 2);
+
 	res = save_manifest_file("test/savedmanifest.raucm", rm, &error);
 	g_assert_no_error(error);
 	g_assert_true(res);
@@ -168,6 +198,8 @@ static void test_save_load_manifest(void)
 	g_assert_no_error(error);
 	g_assert_true(res);
 
+	g_assert_cmpuint(rm->bundle_format, ==, R_MANIFEST_FORMAT_PLAIN);
+	g_assert_true(rm->bundle_format_explicit);
 	check_manifest_contents(rm);
 
 	res = save_manifest_mem(&mem, rm);
@@ -176,6 +208,9 @@ static void test_save_load_manifest(void)
 
 	g_clear_pointer(&rm, free_manifest);
 
+	/* with bundle_format_explicit set, we need to find the format=plain */
+	g_assert_nonnull(g_strstr_len(g_bytes_get_data(mem, NULL), -1, "[bundle]\nformat=plain\n"));
+
 	g_message("manifest in memory:\n%s", (gchar*)g_bytes_get_data(mem, NULL));
 	res = load_manifest_mem(mem, &rm, &error);
 	g_assert_no_error(error);
@@ -183,6 +218,8 @@ static void test_save_load_manifest(void)
 	g_bytes_unref(mem);
 	mem = NULL;
 
+	g_assert_cmpuint(rm->bundle_format, ==, R_MANIFEST_FORMAT_PLAIN);
+	g_assert_true(rm->bundle_format_explicit);
 	check_manifest_contents(rm);
 
 	free_manifest(rm);
@@ -197,7 +234,7 @@ static void test_save_load_manifest(void)
  */
 static void test_save_manifest_writefail(void)
 {
-	GError *error = NULL;
+	g_autoptr(GError) error = NULL;
 	gboolean res;
 	g_autoptr(RaucManifest) rm = g_new0(RaucManifest, 1);
 
@@ -215,7 +252,7 @@ static void test_save_manifest_writefail(void)
  */
 static void test_load_manifest_mem(void)
 {
-	GBytes *data = NULL;
+	g_autoptr(GBytes) data = NULL;
 	RaucManifest *rm = NULL;
 
 	data = read_file("test/manifest.raucm", NULL);
@@ -342,6 +379,7 @@ static void test_manifest_load_meta(void)
 	gboolean res;
 	GError *error = NULL;
 	RaucImage *test_img = NULL;
+	GHashTable *kvs = NULL;
 	const gchar *mffile = "\
 [update]\n\
 compatible=FooCorp Super BarBazzer\n\
@@ -376,6 +414,17 @@ counter=42\n\
 	test_img = (RaucImage*)g_list_nth_data(rm->images, 0);
 	g_assert_nonnull(test_img);
 
+	g_assert_cmpuint(g_hash_table_size(rm->meta), ==, 2);
+
+	kvs = g_hash_table_lookup(rm->meta, "foocorp");
+	g_assert_nonnull(kvs);
+	g_assert_cmpstr((g_hash_table_lookup(kvs, "release-type")), ==, "beta");
+	g_assert_cmpstr((g_hash_table_lookup(kvs, "release-notes")), ==, "https://foocorp.example/releases/release-notes-2015.04-1.rst");
+
+	kvs = g_hash_table_lookup(rm->meta, "example");
+	g_assert_nonnull(kvs);
+	g_assert_cmpstr((g_hash_table_lookup(kvs, "counter")), ==, "42");
+
 	free_manifest(rm);
 }
 
@@ -556,6 +605,23 @@ evilkey=foo\n\
 compatible=SuperBazzer\n\
 \n\
 [evilgroup]\n\
+"
+
+	// invalid meta group
+#define MANIFEST6 "\
+[update]\n\
+compatible=SuperBazzer\n\
+\n\
+[meta.foo/]\n\
+"
+
+	// invalid meta key
+#define MANIFEST7 "\
+[update]\n\
+compatible=SuperBazzer\n\
+\n\
+[meta.foo]\n\
+bar!=baz\n\
 "
 
 	data = g_bytes_new_static(MANIFEST1, sizeof(MANIFEST1));
@@ -593,6 +659,22 @@ compatible=SuperBazzer\n\
 	g_clear_error(&error);
 	g_assert_null(rm);
 	g_clear_pointer(&data, g_bytes_unref);
+
+	data = g_bytes_new_static(MANIFEST6, sizeof(MANIFEST6));
+	g_assert_false(load_manifest_mem(data, &rm, &error));
+	g_assert_error(error, R_UTILS_ERROR, R_UTILS_ERROR_INVALID_ENV_KEY);
+	g_assert_cmpstr("Invalid metadata section name 'foo/': Character '/' is unsuitable for environment variables", ==, error->message);
+	g_clear_error(&error);
+	g_assert_null(rm);
+	g_clear_pointer(&data, g_bytes_unref);
+
+	data = g_bytes_new_static(MANIFEST7, sizeof(MANIFEST7));
+	g_assert_false(load_manifest_mem(data, &rm, &error));
+	g_assert_error(error, R_UTILS_ERROR, R_UTILS_ERROR_INVALID_ENV_KEY);
+	g_assert_cmpstr("Invalid metadata key name 'bar!': Character '!' is unsuitable for environment variables", ==, error->message);
+	g_clear_error(&error);
+	g_assert_null(rm);
+	g_clear_pointer(&data, g_bytes_unref);
 }
 
 int main(int argc, char *argv[])
