@@ -27,6 +27,7 @@ typedef enum {
 	TEST_UPDATE_HANDLER_NO_HOOK_FILE                                  = BIT(7),
 	TEST_UPDATE_HANDLER_HOOK_FAIL                                     = BIT(8),
 	TEST_UPDATE_HANDLER_INCR_BLOCK_HASH_IDX                           = BIT(9),
+	TEST_UPDATE_HANDLER_IMAGE_TOO_LARGE                               = BIT(10),
 } TestUpdateHandlerParams;
 
 typedef struct {
@@ -47,8 +48,8 @@ typedef struct {
  * a valid handler or not */
 static void test_get_update_handler(UpdateHandlerFixture *fixture, gconstpointer user_data)
 {
-	RaucImage *image;
-	RaucSlot *targetslot;
+	g_autoptr(RaucImage) image = NULL;
+	g_autoptr(RaucSlot) targetslot = NULL;
 	img_to_slot_handler handler;
 	UpdateHandlerTestPair *test_pair = (UpdateHandlerTestPair*) user_data;
 	GError *ierror = NULL;
@@ -67,6 +68,7 @@ static void test_get_update_handler(UpdateHandlerFixture *fixture, gconstpointer
 	if (test_pair->params & TEST_UPDATE_HANDLER_EXPECT_FAIL) {
 		g_assert_error(ierror, R_UPDATE_ERROR, R_UPDATE_ERROR_NO_HANDLER);
 		g_assert_null(handler);
+		g_clear_error(&ierror);
 	} else {
 		g_assert_no_error(ierror);
 		g_assert_nonnull(handler);
@@ -80,8 +82,8 @@ static void test_get_update_handler(UpdateHandlerFixture *fixture, gconstpointer
  */
 static void test_get_custom_update_handler(UpdateHandlerFixture *fixture, gconstpointer user_data)
 {
-	RaucImage *image;
-	RaucSlot *targetslot;
+	g_autoptr(RaucImage) image = NULL;
+	g_autoptr(RaucSlot) targetslot = NULL;
 	img_to_slot_handler handler;
 	GError *ierror = NULL;
 
@@ -119,6 +121,20 @@ static void update_handler_fixture_set_up(UpdateHandlerFixture *fixture,
 			g_assert(test_make_filesystem(fixture->tmpdir, "rootfs-0"));
 		}
 	}
+
+	if ((test_pair->params & TEST_UPDATE_HANDLER_IMAGE_TOO_LARGE) &&
+	    (test_pair->params & TEST_UPDATE_HANDLER_INCR_BLOCK_HASH_IDX)) {
+		g_test_expect_message(G_LOG_DOMAIN, G_LOG_LEVEL_MESSAGE,
+				"Checking image type for slot type: *");
+		g_test_expect_message(G_LOG_DOMAIN, G_LOG_LEVEL_MESSAGE,
+				"Image detected as type: *");
+		g_test_expect_message(G_LOG_DOMAIN, G_LOG_LEVEL_INFO,
+				"Selected adaptive update method *");
+		g_test_expect_message(G_LOG_DOMAIN, G_LOG_LEVEL_INFO,
+				"building*");
+		g_test_expect_message(G_LOG_DOMAIN, G_LOG_LEVEL_WARNING,
+				"Continuing after adaptive mode error: Slot * is too small for image *");
+	}
 }
 
 static void update_handler_fixture_tear_down(UpdateHandlerFixture *fixture,
@@ -135,6 +151,12 @@ static void update_handler_fixture_tear_down(UpdateHandlerFixture *fixture,
 		test_rm_tree(fixture->tmpdir, "rootfs-0-datadir");
 	}
 	g_assert(test_rmdir(fixture->tmpdir, "") == 0);
+
+	g_clear_pointer(&fixture->tmpdir, g_free);
+
+	if (!g_test_failed()) {
+		g_test_assert_expected_messages();
+	}
 }
 
 static gboolean tar_image(const gchar *dest, const gchar *dir, GError **error)
@@ -304,6 +326,7 @@ static void test_update_handler(UpdateHandlerFixture *fixture,
 {
 	UpdateHandlerTestPair *test_pair = (UpdateHandlerTestPair*) user_data;
 	gchar *slotpath, *imagename, *imagepath, *mountprefix, *hookpath = NULL;
+	goffset image_size;
 	RaucImage *image;
 	RaucSlot *targetslot;
 	img_to_slot_handler handler;
@@ -317,6 +340,7 @@ static void test_update_handler(UpdateHandlerFixture *fixture,
 	g_test_message("installing '%s' image to '%s' slot", test_pair->imagetype, test_pair->slottype);
 
 	/* prepare image and slot information */
+	image_size = IMAGE_SIZE;
 	imagename = g_strconcat("image.", test_pair->imagetype, NULL);
 	if (g_strcmp0(test_pair->slottype, "ubivol") == 0||
 	    g_strcmp0(test_pair->slottype, "ubifs") == 0) {
@@ -341,7 +365,18 @@ static void test_update_handler(UpdateHandlerFixture *fixture,
 			return;
 		}
 	} else {
-		slotpath = g_build_filename(fixture->tmpdir, "rootfs-0", NULL);
+		if (test_pair->params & TEST_UPDATE_HANDLER_IMAGE_TOO_LARGE) {
+			/* Try to use loop block device for this test. */
+			slotpath = g_strdup(g_getenv("RAUC_TEST_BLOCK_LOOP"));
+			if (!slotpath) {
+				g_test_message("no block device for testing found (define RAUC_TEST_BLOCK_LOOP)");
+				g_test_skip("RAUC_TEST_BLOCK_LOOP undefined");
+				return;
+			}
+			image_size = 65*1024*1024; /* loop dev is only 64 MiB */
+		} else {
+			slotpath = g_build_filename(fixture->tmpdir, "rootfs-0", NULL);
+		}
 	}
 	imagepath = g_build_filename(fixture->tmpdir, imagename, NULL);
 
@@ -357,7 +392,7 @@ static void test_update_handler(UpdateHandlerFixture *fixture,
 	image = g_new0(RaucImage, 1);
 	image->slotclass = g_strdup("rootfs");
 	image->filename = g_strdup(imagepath);
-	image->checksum.size = IMAGE_SIZE;
+	image->checksum.size = image_size;
 	image->checksum.digest = g_strdup("0xdeadbeef");
 	if (test_pair->params & TEST_UPDATE_HANDLER_HOOKS) {
 		const gchar *hook_content_success = "#!/bin/sh\nexit 0";
@@ -368,8 +403,10 @@ static void test_update_handler(UpdateHandlerFixture *fixture,
 		image->hooks.install = (test_pair->params & TEST_UPDATE_HANDLER_INSTALL_HOOK);
 		image->hooks.post_install = (test_pair->params & TEST_UPDATE_HANDLER_POST_HOOK);
 		if (!(test_pair->params & TEST_UPDATE_HANDLER_NO_HOOK_FILE)) {
-			g_assert_true(write_tmp_file(fixture->tmpdir, "hook.sh", hook_content, NULL));
-			test_do_chmod(hookpath);
+			g_autofree gchar *tmp_filename = NULL;
+			tmp_filename = write_tmp_file(fixture->tmpdir, "hook.sh", hook_content, NULL);
+			g_assert_nonnull(tmp_filename);
+			test_do_chmod(tmp_filename);
 		}
 	}
 	if (test_pair->params & TEST_UPDATE_HANDLER_INCR_BLOCK_HASH_IDX) {
@@ -381,14 +418,13 @@ static void test_update_handler(UpdateHandlerFixture *fixture,
 	}
 
 	if (g_strcmp0(test_pair->imagetype, "img") == 0) {
-		g_autofree gchar* pathname = write_random_file(fixture->tmpdir, "image.img", IMAGE_SIZE, 0x2abff992);
+		g_autofree gchar* pathname = write_random_file(fixture->tmpdir, "image.img", image_size, 0x2abff992);
 		g_assert_nonnull(pathname);
 	} else if (g_strcmp0(test_pair->imagetype, "ext4") == 0) {
-		g_assert(test_prepare_dummy_file(fixture->tmpdir, "image.ext4",
-				IMAGE_SIZE, "/dev/zero") == 0);
+		g_assert(test_prepare_dummy_file(fixture->tmpdir, "image.ext4", image_size, "/dev/zero") == 0);
 		g_assert(test_make_filesystem(fixture->tmpdir, "image.ext4"));
-	} else if (g_strcmp0(test_pair->imagetype, "tar.bz2") == 0) {
-		g_assert_true(test_prepare_dummy_archive(fixture->tmpdir, "image.tar.bz2", "testfile.txt"));
+	} else if (g_strcmp0(test_pair->imagetype, "tar") == 0) {
+		g_assert_true(test_prepare_dummy_archive(fixture->tmpdir, "image.tar", "testfile.txt"));
 	} else if (g_strcmp0(test_pair->imagetype, "img.caibx") == 0) {
 		gchar* storepath = test_prepare_dummy_caibx(fixture->tmpdir, "image.img.caibx");
 		g_assert_nonnull(storepath);
@@ -437,6 +473,7 @@ no_image:
 	if (test_pair->params & TEST_UPDATE_HANDLER_EXPECT_FAIL) {
 		g_assert_error(ierror, test_pair->err_domain, test_pair->err_code);
 		g_assert_false(res);
+		g_clear_error(&ierror);
 		goto out;
 	} else {
 		g_assert_no_error(ierror);
@@ -450,12 +487,12 @@ no_image:
 
 	/* Sanity check updated slot */
 	if (g_strcmp0(test_pair->imagetype, "img") == 0) {
-		g_assert_cmpint(get_file_size(imagepath, NULL), ==, IMAGE_SIZE);
+		g_assert_cmpint(get_file_size(imagepath, NULL), ==, image_size);
 	} else if (g_strcmp0(test_pair->imagetype, "ext4") == 0) {
-		g_assert_cmpint(get_file_size(imagepath, NULL), ==, IMAGE_SIZE);
+		g_assert_cmpint(get_file_size(imagepath, NULL), ==, image_size);
 		g_assert(test_mount(slotpath, mountprefix));
 		g_assert(r_umount(slotpath, NULL));
-	} else if ((g_strcmp0(test_pair->imagetype, "tar.bz2") == 0) || ((g_strcmp0(test_pair->imagetype, "caidx") == 0))) {
+	} else if ((g_strcmp0(test_pair->imagetype, "tar") == 0) || ((g_strcmp0(test_pair->imagetype, "caidx") == 0))) {
 		gchar *testpath = g_build_filename(mountprefix, "testfile.txt", NULL);
 		g_assert(test_mount(slotpath, mountprefix));
 		g_assert_true(g_file_test(testpath, G_FILE_TEST_IS_REGULAR));
@@ -480,24 +517,28 @@ no_image:
 		g_assert_cmpstr(stats->label, ==, "zero chunk");
 		count_zero = stats->count;
 		sum_zero = stats->sum;
+		r_stats_free(stats);
 
 		stats = r_test_stats_next();
 		g_assert_nonnull(stats);
 		g_assert_cmpstr(stats->label, ==, "target_slot_written (reusing source_image)");;
 		count_target_written = stats->count;
 		sum_target_written = stats->sum;
+		r_stats_free(stats);
 
 		stats = r_test_stats_next();
 		g_assert_nonnull(stats);
 		g_assert_cmpstr(stats->label, ==, "target_slot");
 		count_target = stats->count;
 		sum_target = stats->sum;
+		r_stats_free(stats);
 
 		stats = r_test_stats_next();
 		g_assert_nonnull(stats);
 		g_assert_cmpstr(stats->label, ==, "source_image");
 		count_source = stats->count;
 		sum_source = stats->sum;
+		r_stats_free(stats);
 
 		/* all non-zero chunks must result in a lookup in target_slot_written */
 		g_assert_cmpint(count_zero + count_target_written, ==, IMAGE_SIZE/4096);
@@ -538,8 +579,8 @@ out:
 			g_assert(g_remove(imagepath) == 0);
 		} else if (g_strcmp0(test_pair->imagetype, "ext4") == 0) {
 			g_assert(g_remove(imagepath) == 0);
-		} else if (g_strcmp0(test_pair->imagetype, "tar.bz2") == 0) {
-			g_assert(test_remove(fixture->tmpdir, "image.tar.bz2") == 0);
+		} else if (g_strcmp0(test_pair->imagetype, "tar") == 0) {
+			g_assert(test_remove(fixture->tmpdir, "image.tar") == 0);
 		} else if (g_strcmp0(test_pair->imagetype, "img.caibx") == 0) {
 			g_assert(test_remove(fixture->tmpdir, "image.img.caibx") == 0);
 			g_assert_true(rm_tree(r_context()->install_info->mounted_bundle->storepath, NULL));
@@ -571,72 +612,72 @@ out:
 int main(int argc, char *argv[])
 {
 	UpdateHandlerTestPair testpair_matrix[] = {
-		{"ext4", "tar.bz2", TEST_UPDATE_HANDLER_DEFAULT, 0, 0},
+		{"ext4", "tar", TEST_UPDATE_HANDLER_DEFAULT, 0, 0},
 		{"ext4", "ext4", TEST_UPDATE_HANDLER_DEFAULT, 0, 0},
-		{"ubifs", "tar.bz2", TEST_UPDATE_HANDLER_DEFAULT, 0, 0},
+		{"ubifs", "tar", TEST_UPDATE_HANDLER_DEFAULT, 0, 0},
 		{"ubifs", "ext4", TEST_UPDATE_HANDLER_EXPECT_FAIL, 0, 0},
 
 		{"raw", "img", TEST_UPDATE_HANDLER_DEFAULT, 0, 0},
 		{"ext4", "img", TEST_UPDATE_HANDLER_DEFAULT, 0, 0},
-		{"ext4", "tar.bz2", TEST_UPDATE_HANDLER_DEFAULT, 0, 0},
+		{"ext4", "tar", TEST_UPDATE_HANDLER_DEFAULT, 0, 0},
 		{"raw", "ext4", TEST_UPDATE_HANDLER_DEFAULT, 0, 0},
 
-		{"ext4", "tar.bz2", TEST_UPDATE_HANDLER_NO_IMAGE_FILE | TEST_UPDATE_HANDLER_EXPECT_FAIL, G_SPAWN_EXIT_ERROR, 2},
+		{"ext4", "tar", TEST_UPDATE_HANDLER_NO_IMAGE_FILE | TEST_UPDATE_HANDLER_EXPECT_FAIL, G_IO_ERROR, G_IO_ERROR_NOT_FOUND},
 		{"raw", "img", TEST_UPDATE_HANDLER_NO_IMAGE_FILE | TEST_UPDATE_HANDLER_EXPECT_FAIL, G_IO_ERROR, G_IO_ERROR_NOT_FOUND},
 		{"raw", "ext4", TEST_UPDATE_HANDLER_NO_IMAGE_FILE | TEST_UPDATE_HANDLER_EXPECT_FAIL, G_IO_ERROR, G_IO_ERROR_NOT_FOUND},
 		{"ext4", "ext4", TEST_UPDATE_HANDLER_NO_IMAGE_FILE | TEST_UPDATE_HANDLER_EXPECT_FAIL, G_IO_ERROR, G_IO_ERROR_NOT_FOUND},
 
-		{"ext4", "tar.bz2", TEST_UPDATE_HANDLER_NO_TARGET_DEV | TEST_UPDATE_HANDLER_EXPECT_FAIL, G_SPAWN_EXIT_ERROR, 1},
+		{"ext4", "tar", TEST_UPDATE_HANDLER_NO_TARGET_DEV | TEST_UPDATE_HANDLER_EXPECT_FAIL, G_SPAWN_EXIT_ERROR, 1},
 		{"raw", "img", TEST_UPDATE_HANDLER_NO_TARGET_DEV | TEST_UPDATE_HANDLER_EXPECT_FAIL, R_UPDATE_ERROR, R_UPDATE_ERROR_FAILED},
 		{"raw", "ext4", TEST_UPDATE_HANDLER_NO_TARGET_DEV | TEST_UPDATE_HANDLER_EXPECT_FAIL, R_UPDATE_ERROR, R_UPDATE_ERROR_FAILED},
 		{"ext4", "ext4", TEST_UPDATE_HANDLER_NO_TARGET_DEV | TEST_UPDATE_HANDLER_EXPECT_FAIL, R_UPDATE_ERROR, R_UPDATE_ERROR_FAILED},
 
-		{"ext4", "tar.bz2", TEST_UPDATE_HANDLER_HOOKS | TEST_UPDATE_HANDLER_PRE_HOOK, 0, 0},
+		{"ext4", "tar", TEST_UPDATE_HANDLER_HOOKS | TEST_UPDATE_HANDLER_PRE_HOOK, 0, 0},
 		{"raw", "img", TEST_UPDATE_HANDLER_HOOKS | TEST_UPDATE_HANDLER_PRE_HOOK, 0, 0},
 		{"raw", "ext4", TEST_UPDATE_HANDLER_HOOKS | TEST_UPDATE_HANDLER_PRE_HOOK, 0, 0},
 		{"ext4", "ext4", TEST_UPDATE_HANDLER_HOOKS | TEST_UPDATE_HANDLER_PRE_HOOK, 0, 0},
 
-		{"ext4", "tar.bz2", TEST_UPDATE_HANDLER_HOOKS | TEST_UPDATE_HANDLER_POST_HOOK, 0, 0},
+		{"ext4", "tar", TEST_UPDATE_HANDLER_HOOKS | TEST_UPDATE_HANDLER_POST_HOOK, 0, 0},
 		{"raw", "img", TEST_UPDATE_HANDLER_HOOKS | TEST_UPDATE_HANDLER_POST_HOOK, 0, 0},
 		{"raw", "ext4", TEST_UPDATE_HANDLER_HOOKS | TEST_UPDATE_HANDLER_POST_HOOK, 0, 0},
 		{"ext4", "ext4", TEST_UPDATE_HANDLER_HOOKS | TEST_UPDATE_HANDLER_POST_HOOK, 0, 0},
 
-		{"ext4", "tar.bz2", TEST_UPDATE_HANDLER_HOOKS | TEST_UPDATE_HANDLER_INSTALL_HOOK, 0, 0},
+		{"ext4", "tar", TEST_UPDATE_HANDLER_HOOKS | TEST_UPDATE_HANDLER_INSTALL_HOOK, 0, 0},
 		{"raw", "img", TEST_UPDATE_HANDLER_HOOKS | TEST_UPDATE_HANDLER_INSTALL_HOOK, 0, 0},
 		{"raw", "ext4", TEST_UPDATE_HANDLER_HOOKS | TEST_UPDATE_HANDLER_INSTALL_HOOK, 0, 0},
 		{"ext4", "ext4", TEST_UPDATE_HANDLER_HOOKS | TEST_UPDATE_HANDLER_INSTALL_HOOK, 0, 0},
 
-		{"ext4", "tar.bz2", TEST_UPDATE_HANDLER_HOOKS | TEST_UPDATE_HANDLER_PRE_HOOK | TEST_UPDATE_HANDLER_HOOK_FAIL | TEST_UPDATE_HANDLER_EXPECT_FAIL, G_SPAWN_EXIT_ERROR, 1},
+		{"ext4", "tar", TEST_UPDATE_HANDLER_HOOKS | TEST_UPDATE_HANDLER_PRE_HOOK | TEST_UPDATE_HANDLER_HOOK_FAIL | TEST_UPDATE_HANDLER_EXPECT_FAIL, G_SPAWN_EXIT_ERROR, 1},
 		{"raw", "img", TEST_UPDATE_HANDLER_HOOKS | TEST_UPDATE_HANDLER_PRE_HOOK | TEST_UPDATE_HANDLER_HOOK_FAIL | TEST_UPDATE_HANDLER_EXPECT_FAIL, G_SPAWN_EXIT_ERROR, 1},
 		{"raw", "ext4", TEST_UPDATE_HANDLER_HOOKS | TEST_UPDATE_HANDLER_PRE_HOOK | TEST_UPDATE_HANDLER_HOOK_FAIL | TEST_UPDATE_HANDLER_EXPECT_FAIL, G_SPAWN_EXIT_ERROR, 1},
 		{"ext4", "ext4", TEST_UPDATE_HANDLER_HOOKS | TEST_UPDATE_HANDLER_PRE_HOOK | TEST_UPDATE_HANDLER_HOOK_FAIL | TEST_UPDATE_HANDLER_EXPECT_FAIL, G_SPAWN_EXIT_ERROR, 1},
 
-		{"ext4", "tar.bz2", TEST_UPDATE_HANDLER_HOOKS | TEST_UPDATE_HANDLER_POST_HOOK | TEST_UPDATE_HANDLER_HOOK_FAIL | TEST_UPDATE_HANDLER_EXPECT_FAIL, G_SPAWN_EXIT_ERROR, 1},
+		{"ext4", "tar", TEST_UPDATE_HANDLER_HOOKS | TEST_UPDATE_HANDLER_POST_HOOK | TEST_UPDATE_HANDLER_HOOK_FAIL | TEST_UPDATE_HANDLER_EXPECT_FAIL, G_SPAWN_EXIT_ERROR, 1},
 		{"raw", "img", TEST_UPDATE_HANDLER_HOOKS | TEST_UPDATE_HANDLER_POST_HOOK | TEST_UPDATE_HANDLER_HOOK_FAIL | TEST_UPDATE_HANDLER_EXPECT_FAIL, G_SPAWN_EXIT_ERROR, 1},
 		{"raw", "ext4", TEST_UPDATE_HANDLER_HOOKS | TEST_UPDATE_HANDLER_POST_HOOK | TEST_UPDATE_HANDLER_HOOK_FAIL | TEST_UPDATE_HANDLER_EXPECT_FAIL, G_SPAWN_EXIT_ERROR, 1},
 		{"ext4", "ext4", TEST_UPDATE_HANDLER_HOOKS | TEST_UPDATE_HANDLER_POST_HOOK | TEST_UPDATE_HANDLER_HOOK_FAIL | TEST_UPDATE_HANDLER_EXPECT_FAIL, G_SPAWN_EXIT_ERROR, 1},
 
-		{"ext4", "tar.bz2", TEST_UPDATE_HANDLER_HOOKS | TEST_UPDATE_HANDLER_INSTALL_HOOK | TEST_UPDATE_HANDLER_HOOK_FAIL | TEST_UPDATE_HANDLER_EXPECT_FAIL, G_SPAWN_EXIT_ERROR, 1},
+		{"ext4", "tar", TEST_UPDATE_HANDLER_HOOKS | TEST_UPDATE_HANDLER_INSTALL_HOOK | TEST_UPDATE_HANDLER_HOOK_FAIL | TEST_UPDATE_HANDLER_EXPECT_FAIL, G_SPAWN_EXIT_ERROR, 1},
 		{"raw", "img", TEST_UPDATE_HANDLER_HOOKS | TEST_UPDATE_HANDLER_INSTALL_HOOK | TEST_UPDATE_HANDLER_HOOK_FAIL | TEST_UPDATE_HANDLER_EXPECT_FAIL, G_SPAWN_EXIT_ERROR, 1},
 		{"raw", "ext4", TEST_UPDATE_HANDLER_HOOKS | TEST_UPDATE_HANDLER_INSTALL_HOOK | TEST_UPDATE_HANDLER_HOOK_FAIL | TEST_UPDATE_HANDLER_EXPECT_FAIL, G_SPAWN_EXIT_ERROR, 1},
 		{"ext4", "ext4", TEST_UPDATE_HANDLER_HOOKS | TEST_UPDATE_HANDLER_INSTALL_HOOK | TEST_UPDATE_HANDLER_HOOK_FAIL | TEST_UPDATE_HANDLER_EXPECT_FAIL, G_SPAWN_EXIT_ERROR, 1},
 
-		{"ext4", "tar.bz2", TEST_UPDATE_HANDLER_HOOKS | TEST_UPDATE_HANDLER_PRE_HOOK | TEST_UPDATE_HANDLER_NO_HOOK_FILE | TEST_UPDATE_HANDLER_EXPECT_FAIL, G_SPAWN_ERROR, G_SPAWN_ERROR_NOENT},
+		{"ext4", "tar", TEST_UPDATE_HANDLER_HOOKS | TEST_UPDATE_HANDLER_PRE_HOOK | TEST_UPDATE_HANDLER_NO_HOOK_FILE | TEST_UPDATE_HANDLER_EXPECT_FAIL, G_SPAWN_ERROR, G_SPAWN_ERROR_NOENT},
 
 		/* vfat tests */
-		{"vfat", "tar.bz2", TEST_UPDATE_HANDLER_DEFAULT, 0, 0},
-		{"vfat", "tar.bz2", TEST_UPDATE_HANDLER_DEFAULT, 0, 0},
-		{"vfat", "tar.bz2", TEST_UPDATE_HANDLER_NO_IMAGE_FILE | TEST_UPDATE_HANDLER_EXPECT_FAIL, G_SPAWN_EXIT_ERROR, 2},
-		{"vfat", "tar.bz2", TEST_UPDATE_HANDLER_NO_TARGET_DEV | TEST_UPDATE_HANDLER_EXPECT_FAIL, G_SPAWN_EXIT_ERROR, 1},
-		{"vfat", "tar.bz2", TEST_UPDATE_HANDLER_HOOKS | TEST_UPDATE_HANDLER_PRE_HOOK, 0, 0},
-		{"vfat", "tar.bz2", TEST_UPDATE_HANDLER_HOOKS | TEST_UPDATE_HANDLER_POST_HOOK, 0, 0},
-		{"vfat", "tar.bz2", TEST_UPDATE_HANDLER_HOOKS | TEST_UPDATE_HANDLER_INSTALL_HOOK, 0, 0},
-		{"vfat", "tar.bz2", TEST_UPDATE_HANDLER_HOOKS | TEST_UPDATE_HANDLER_PRE_HOOK | TEST_UPDATE_HANDLER_HOOK_FAIL | TEST_UPDATE_HANDLER_EXPECT_FAIL, G_SPAWN_EXIT_ERROR, 1},
-		{"vfat", "tar.bz2", TEST_UPDATE_HANDLER_HOOKS | TEST_UPDATE_HANDLER_POST_HOOK | TEST_UPDATE_HANDLER_HOOK_FAIL | TEST_UPDATE_HANDLER_EXPECT_FAIL, G_SPAWN_EXIT_ERROR, 1},
-		{"vfat", "tar.bz2", TEST_UPDATE_HANDLER_HOOKS | TEST_UPDATE_HANDLER_INSTALL_HOOK | TEST_UPDATE_HANDLER_HOOK_FAIL | TEST_UPDATE_HANDLER_EXPECT_FAIL, G_SPAWN_EXIT_ERROR, 1},
+		{"vfat", "tar", TEST_UPDATE_HANDLER_DEFAULT, 0, 0},
+		{"vfat", "tar", TEST_UPDATE_HANDLER_DEFAULT, 0, 0},
+		{"vfat", "tar", TEST_UPDATE_HANDLER_NO_IMAGE_FILE | TEST_UPDATE_HANDLER_EXPECT_FAIL, G_IO_ERROR, G_IO_ERROR_NOT_FOUND},
+		{"vfat", "tar", TEST_UPDATE_HANDLER_NO_TARGET_DEV | TEST_UPDATE_HANDLER_EXPECT_FAIL, G_SPAWN_EXIT_ERROR, 1},
+		{"vfat", "tar", TEST_UPDATE_HANDLER_HOOKS | TEST_UPDATE_HANDLER_PRE_HOOK, 0, 0},
+		{"vfat", "tar", TEST_UPDATE_HANDLER_HOOKS | TEST_UPDATE_HANDLER_POST_HOOK, 0, 0},
+		{"vfat", "tar", TEST_UPDATE_HANDLER_HOOKS | TEST_UPDATE_HANDLER_INSTALL_HOOK, 0, 0},
+		{"vfat", "tar", TEST_UPDATE_HANDLER_HOOKS | TEST_UPDATE_HANDLER_PRE_HOOK | TEST_UPDATE_HANDLER_HOOK_FAIL | TEST_UPDATE_HANDLER_EXPECT_FAIL, G_SPAWN_EXIT_ERROR, 1},
+		{"vfat", "tar", TEST_UPDATE_HANDLER_HOOKS | TEST_UPDATE_HANDLER_POST_HOOK | TEST_UPDATE_HANDLER_HOOK_FAIL | TEST_UPDATE_HANDLER_EXPECT_FAIL, G_SPAWN_EXIT_ERROR, 1},
+		{"vfat", "tar", TEST_UPDATE_HANDLER_HOOKS | TEST_UPDATE_HANDLER_INSTALL_HOOK | TEST_UPDATE_HANDLER_HOOK_FAIL | TEST_UPDATE_HANDLER_EXPECT_FAIL, G_SPAWN_EXIT_ERROR, 1},
 
 		/* ubifs tests */
-		{"ubifs", "tar.bz2", TEST_UPDATE_HANDLER_DEFAULT, 0, 0},
+		{"ubifs", "tar", TEST_UPDATE_HANDLER_DEFAULT, 0, 0},
 		{"ubifs", "img", TEST_UPDATE_HANDLER_DEFAULT, 0, 0},
 		{"ubivol", "img", TEST_UPDATE_HANDLER_DEFAULT, 0, 0},
 
@@ -658,6 +699,10 @@ int main(int argc, char *argv[])
 		{"ext4", "caidx", TEST_UPDATE_HANDLER_DEFAULT, 0, 0},
 		{"vfat", "caidx", TEST_UPDATE_HANDLER_DEFAULT, 0, 0},
 
+		/* image too large */
+		{"ext4", "ext4", TEST_UPDATE_HANDLER_IMAGE_TOO_LARGE | TEST_UPDATE_HANDLER_EXPECT_FAIL, R_UPDATE_ERROR, R_UPDATE_ERROR_FAILED},
+		{"ext4", "ext4", TEST_UPDATE_HANDLER_INCR_BLOCK_HASH_IDX | TEST_UPDATE_HANDLER_IMAGE_TOO_LARGE | TEST_UPDATE_HANDLER_EXPECT_FAIL, R_UPDATE_ERROR, R_UPDATE_ERROR_FAILED},
+
 		{0}
 	};
 	setlocale(LC_ALL, "C");
@@ -676,7 +721,7 @@ int main(int argc, char *argv[])
 			NULL,
 			test_get_update_handler,
 			NULL);
-	g_test_add("/update_handler/get_handler/tar.bz2_to_ubifs",
+	g_test_add("/update_handler/get_handler/tar_to_ubifs",
 			UpdateHandlerFixture,
 			&testpair_matrix[2],
 			NULL,
@@ -1074,5 +1119,19 @@ int main(int argc, char *argv[])
 	                update_handler_fixture_tear_down);
 	 */
 
+	/* too large */
+	g_test_add("/update_handler/too_large/normal",
+			UpdateHandlerFixture,
+			&testpair_matrix[63],
+			update_handler_fixture_set_up,
+			test_update_handler,
+			update_handler_fixture_tear_down);
+	g_test_add("/update_handler/too_large/adaptive",
+			UpdateHandlerFixture,
+			&testpair_matrix[64],
+			update_handler_fixture_set_up,
+			test_update_handler,
+			update_handler_fixture_tear_down);
+
 	return g_test_run();
 }
